theINCLUDE

theHEADER( priority_list )

SECTION(Usage, [
<pre><code>
#include "mnemonic/priority_list.hh"

priority_list<char> lst;
priority_list<char>::iterator i;

// Add some objects
lst.insert(1,'H');
lst.insert(8,'d');
lst.insert(5,'o');
lst.insert(4,'W');
lst.insert(7,'l');
lst.insert(3,' ');
lst.insert(2,'I');
lst.insert(6,'r');

// Print the objects in priority order (Result: "Hi World")
for (i=lst.begin(); i!=lst.end(); i++) {
	cout << *i ;
}
cout << endl;

// Clear the list
for (i=lst.begin(); i!=lst.end(); i++) {
	lst.erase(i);
}

</code></pre>
])

SECTION(Overview, [

This is a implementation of a priority list based on the STL list template. It stores object associated with an integer priority number. The list is kept sorted acording to this number starting with the lowest one.
<P>
It contains an iterator that are capable of iteration through a list (forward and backwards) while the list is modified. If the element the iterator is pointing to is removed from the list the iterator will be changed to point to the element before that one. 
<p>
The iterator is guaranteed to always be valid as an iterator, but it might be assigned to the a speciall past_end value which means that we have iterated pasted the list boundaries (which also happen if an iterator is pointing to the first element and it is deleted). In this case the iterator is not pointing to a valid object in the list. The iterator keeps track of whatever it excits the list at the end or at the begining. This means that it is only equal to the iterator returned by the end() method (se below) if it passed by the end of the list.
<p>
The list method does not contain any mutex locking to prevent two threads from making a mess out of common data. This is because such a lock often has to be placed over several method calls and therefor it has to be preformed by the user of the list (se above for an usage example).
])

SECTION(List Methods, [

<dl>

<dt>
<code>
priority_list<char> lst;
</code>
</dt>
<dd>
The priority_list is a template allowing you to specify the type of objects that should be stored in the list. So this declaration creats a list called <code>lst</code> that stoes objects of type <code>char</code>.
</dd>

<dt>
<code>
lst.begin()
</code>
</dt>
<dd>
Returns a iterator poiting to the first element.
</dd>

<dt>
<code>
lst.end()
</code>
</dt>
<dd>
Returns a iterator poiting past the end of the list reached by passing the last element in the list.
</dd>

<dt>
<code>
lst.erase(pos)
</code>
</dt>
<dd>

Removes the element pointed to by the iterator <code>pos</code>. All iterators pointing to this object (including <code>pos</code>) will be reassign to element directly before this one.
</dd>

<dt>
<code>
lst.insert(pri,obj)
</code>
</dt>
<dd>
Inserts the object <code>obj</code> into the list with priority <code>pri</code>. <code>obj</code> should be of the type specified when declearing the list object and <code>pri</code> should be an integer.
</dd>

</dl>
])
SECTION(Iterator Methods, [
<dl>
<dt>
<code>
priority_list<char>::iterator i;
</code>
</dt>
<dd>
Declears a new iterator variabel to a priority_list with <code>char</code> objects.
</dd>

<dt>
<code>
i=lst.begin();
</code>
</dt>
<dd>
Assign the itterator <code>i</code> to point to the first object of <code>lst</code>.
</dd>

<dt>
<code>
i++
</code>
</dt>
<dd>
Move the iterator <code>i</code> one step forward.
</dd>

<dt>
<code>
i--
</code>
</dt>
<dd>
Move the iterator <code>i</code> one step backwards.
</dd>

<dt>
<code>
i==lst.end()
</code>
</dt>
<dd>
True if the itterators <code>i</code> and <code>lst.end()</code> are pointing to the same element in the list and reached it from the same direction (eg by means of -- or ++).
</dd>

<dt>
<code>
*i
</code>
</dt>
<dd>
Dereferes the iterator <code>i</code> to the object in the list it is refering to.
</dd>


</dl>
])
