// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Memory leak finder.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* As always, memory debuggers are a hack. This code provides the
   possibility to get a list of still allocated blocks. It tries to
   give as much information as possible by redefining alloc and
   dealloc routines, and linking in replacements.

	Sometimes it is necesary to shuffle around the includes to prevent
	problems with the defines. Objects with replacements for new or
	delete are NOT supported by this code so this include file should
	not be used together with such code.
	
	Because the code uses a global class with a destructor to show the
	list, it is not garanteed that after this destructor nothing is
	freed anymore.

	memdebug needs -ldl 
*/

#include "optionals.h"

#ifdef MEMDEBUG

#ifndef MEMDEBUGXX
#define MEMDEBUGXX
#include <stdlib.h> /* for size_t */
#include <new.h>    /* to prevent it being include AFTER this file */
#include <stdio.h>  /* for stderr */

void mxfree(void *p);
void rmkm(int t,void *p);
void printkm();

void *operator new(size_t k,char *fname,long lnum);
void *operator new[](size_t k,char *fname,long lnum);
void *operator new(size_t k);
void *operator new[](size_t k);

inline void operator delete(void *p)
	{
	if (p==0) return;
	rmkm(0,p);
	mxfree(p);
	}

inline void operator delete[](void *p)
	{
	if (p==0) return;
	rmkm(1,p);
	mxfree(p);
	}


void *mdmalloc(size_t k,char *fname,long lnum);
void *mdrealloc(void *p,size_t k,char *fname,long lnum);
char *mdstrdup(const char *p,char *fname,long lnum);
void mdfree(void *p);

void printkm();

#define new new(__FILE__,__LINE__)
#define malloc(x) mdmalloc(x,__FILE__,__LINE__)
#define realloc(p,x) mdrealloc(p,x,__FILE__,__LINE__)
#define strdup(x) mdstrdup(x,__FILE__,__LINE__)
#define free(x) mdfree(x)


#endif

#endif
