// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Mnemonic Object Manager.             

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef objectmanager_hh__
#define objectmanager_hh__

#include <string>
#include <vector.h>
#include <map.h>
#include <stdexcept>
#include <mnemonic/thread.hh>
#include <mnemonic/thread_mutex.hh>

class dlobject;
class object_broker;

class object_manager {
   public:
		virtual ~object_manager();
		static object_manager& instance(void);
		static  void           free_instance(void);
		virtual void           load_interests(void);
		virtual dlobject      *new_object_by_filename(const char *name, void **ctorargs);
		virtual void           delete_object_by_id(dlobject * Object, char *objID);

		class om_exception : public logic_error {
			public:
				enum om_error { success, opendir_failed, object_nonexistent, object_already_loaded, 
									 create_failed, object_not_loaded };
				om_exception(om_error, const char *txt);
				om_error code(void) const;
				virtual const char* what () const;
			private:
				string local_what_;
				om_error status_;
		};
   private:
		object_manager(void);
		static object_manager  *instance_;
		object_broker          *OB_;
		vector<string>          dirs_;
		map<string, dlobject *> loaded_;
		thread_mutex            create_lock_;
};

#endif
