// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Mnemonic threadsafe list with priorities (based on STL); templates.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef PRIORITY_LIST_HH
#define PRIORITY_LIST_HH

#include <stl.h>

template<class o, class pri> 
class priority_list {
	public:
		class iterator {
			public:
				iterator(priority_list<o,pri> *p, list<pair<pri,o> >::iterator i) 
					{
					parent=p; itt=i;
					//deced=false;
					if (parent) parent->register_iterator(this);
					}
				~iterator() 
					{ 
					if(parent) parent->unregister_iterator(this);
					}
				iterator() 
					{ 
					parent=NULL; 
					//deced=false;
					}
				iterator(const iterator &i) 
					{
					parent=i.parent; itt=i.itt;
					//deced=i.deced;
					if (parent) parent->register_iterator(this);
					}
		
				//iterator& operator++(int)  { itt++; deced=false; return *this; } //
				//iterator& operator--(int)  { itt--; deced=true; return *this; }
				iterator& operator++(void) { 
				    itt++; 
				//    deced=false; 
				    return *this; 
				}
				//iterator& operator--(void) { itt--; deced=true; return *this; }
				bool operator==(const iterator& i) {
				    //return deced==i.deced && itt==i.itt;
				    return itt==i.itt;
				}
				bool operator!=(const iterator& i) {return !( *this==i);}
				o& operator* ()  { return (*itt).second;}
				iterator& operator=(const iterator &i);
			private:
				list<pair<pri,o> >::iterator itt;
				// static list<pair<pri,o> >::iterator past_begin;
				priority_list<o,pri> *parent;
				//bool deced; // Keep track if we reach end() by decing or incing
				friend priority_list<o,pri>; 
		};
		iterator begin();
		iterator end();
		int size(void) const
			{
			return the_list.size();
			}
		void erase (const iterator & pos);
		void insert(pri priority, o obj);
	private:
		list<pair<pri,o> > the_list;      // gets instantiated in messagebase.cc
		list<iterator *> iterator_list;
		
		void register_iterator(iterator *i) {iterator_list.push_back(i);}
		void unregister_iterator(iterator *i) {iterator_list.remove(i);}
		
		friend iterator;
};

template<class o, class pri> 
void priority_list<o,pri>::insert(pri priority, o obj) 
	{
	pair<pri,o> node(priority,obj);
	list<pair<pri,o> >::iterator i=the_list.begin();
	
	while ( (*i).first < priority && i != the_list.end() ) i++;
	the_list.insert(i, 1, node);
	}

template<class o, class pri> 
priority_list<o,pri>::iterator priority_list<o,pri>::begin() 
	{
	iterator start(this, the_list.begin());
	return start;
	}

template<class o, class pri> 
priority_list<o,pri>::iterator priority_list<o,pri>::end() 
	{
	iterator stop(this,the_list.end());
	return stop;
	}

template<class o, class pri> 
void priority_list<o,pri>::erase(const iterator &pos) 
	{
	// Save pointer to the object to remove
	list<pair<pri,o> >::iterator del=pos.itt;
	
	// Decrement all iterators pointing to the erased object
	list<iterator *>::iterator i;
	for (i=iterator_list.begin(); i!=iterator_list.end(); i++) {
		if ( distance( (*i)->itt, pos.itt ) == 0) {
			(**i).itt--; 
			}
		}
	
	// Erase it
	the_list.erase(del);
	}

template<class o, class pri> 
priority_list<o,pri>::iterator & priority_list<o,pri>::iterator::operator=(const iterator &i)
	{
	// This properly handles self-assignment
	if (parent) parent->unregister_iterator(this);
	parent=i.parent; 
	itt=i.itt; 
	//deced=i.deced;
	if (parent) parent->register_iterator(this);
	return *this;
	}


#endif
