// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Template for reference counting pointers. All pointers share one
  `counter' object which contains the real pointer. These reference
  counting pointers can be used with any object (ie. they do not
  have to derive from a special base class).

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef ptr_hh__
#define ptr_hh__

#include <assert.h>
#include <mnemonic/thread_cond.hh>
#include <stdexcept>

//#include <typeinfo>
//#define STATE cerr<<this<<"ptr.hh:"<<__LINE__<<" "<<typeid(*this).name()<<" "<<c_<<" "<<p_<<"\n"<<flush;
#define STATE

class counter : public thread_cond {
   public:
		counter(void);
		~counter();
		int count_; 
};


template<class T> 
class ptr {
	public:
		ptr(void)
			{
			p_=0;
//			c_=0;
			STATE;
			}
		ptr(T *p)
			{
			p_=p;
			if (p_) c_=new counter();
//			else c_=0;
			STATE;
			}
		ptr(const ptr<T> &p)
			{
			p_=p.p_;
			if (p_) {
				c_=p.c_;
				c_->count_++;
				} 
//			else c_=0;
			STATE;
			}

		template<class D>
		ptr(const ptr<D> &p) 
			{
			// this is base of p
			p_=p.p_;
			if (p_) {
				p.c_->lock();
				c_=p.c_;
				c_->count_++;
				p.c_->unlock();
				} 
//			else c_=0;
			STATE;
			}

		template<class D>
		void cast(const ptr<D> &p)
			{
			if (p_==static_cast<T*>(p.p_)) return;
			if (p_) {
				c_->lock();
				c_->count_--;
				if (c_->count_==0) {
					c_->unlock(); 
					delete p_;
					delete c_; 
					} 
				else {
					if (c_->count_==1) c_->broadcast();
					c_->unlock();
					}
				}
			p_=static_cast<T*>(p.p_);
			if (p_) {
				c_=p.c_;  
				c_->count_++;
				}
			STATE;
			return;
			}

		template<class D>
		void dynamiccast(const ptr<D> &p) 
			{
			//test that we are equal already.
			STATE;
			if (p_==dynamic_cast<T*>(p.p_)) return;
			if (p_) {
				c_->lock();
				c_->count_--;
				if (c_->count_==0) {
					c_->unlock(); 
					delete p_;
					delete c_; 
					} 
				else {
					if (c_->count_==1) c_->broadcast();
					c_->unlock();
					}
				}
			p_=dynamic_cast<T*>(p.p_);
			if (p_) {
				c_=p.c_;  
				c_->count_++;
				}
			STATE;
			return;
			}

		~ptr()
			{
			STATE
				if (p_) {
					c_->lock();
					c_->count_--;
					if (c_->count_==0) {
						c_->unlock();
//						cerr << "DELETING object at " << p_ << endl;
						delete p_; 
						delete c_;
						} 
					else  {
						if (c_->count_==1) c_->broadcast();
						c_->unlock();
						}
					}
			}

		T* operator-> () const 
			{
			STATE;
			assert(p_!=0);
			return p_; 
			}
		
		T& operator*  () const 
			{
			STATE;
			return *(p_); 
			}
		
		bool operator== (const ptr<T> &p) const 
			{
			STATE;
			if ((p_!=0)&&(p_==p.p_)&&(c_!=p.c_)) throw logic_error("two instances of one ptr");
			return p_==p.p_;
			}
		
		bool operator!= (const ptr<T> &p) const 
			{
			STATE;
			if ((p_!=0)&&(p_==p.p_)&&(c_!=p.c_)) throw logic_error("two instances of one ptr");
			return p_!=p.p_;
			}
		//maybe you should rethink your code if you need this
		//the pair template defines a greater/smaller for pairs
		//and because of that we have to define these
		bool operator> (const ptr<T> &p) const 
			{
			STATE;
			assert(p_!=0);
			assert(p.p_!=0);
			if ((p_==p.p_)&&(c_!=p.c_)) throw logic_error("two instances of one ptr");
			return p_>p.p_;
			}

		bool operator< (const ptr<T> &p) const 
			{
			STATE;
			assert(p_!=0);
			assert(p.p_!=0);
			if ((p_==p.p_)&&(c_!=p.c_)) throw logic_error("two instances of one ptr");
			return p_<p.p_;
			}
    		ptr& operator= (const ptr<T> &p) 
			{
			//test that we are equal already.
			STATE;
			if (p_==p.p_) return *this;
			if (p_) {
				c_->lock();
				c_->count_--;
				if (c_->count_==0) {
					c_->unlock(); 
					delete p_;
					delete c_; 
					} 
				else {
					if (c_->count_==1) c_->broadcast();
					c_->unlock();
					}
				}
			p_=p.p_;
			if (p_) {
				c_=p.c_;  
				c_->count_++;
				}
			STATE;
			return *this;
			}
#ifdef notrequired			
		ptr& operator= ( T *p) 
			{
			//test if we contain this pointer
			STATE;
			if (p_) {
				if (p_==p) return *this;
				c_->lock();
				c_->count_--;
				if (c_->count_==0) {
					c_->unlock();
					delete p_;
					delete c_;
					}
				else {
					if (c_->count_==1) c_->broadcast();
					c_->unlock();
					}
				}
			
			p_=p;
			if (p_) c_=new counter();
			STATE;
			return *this;
			}
#endif			
		//return when we are the last user
		//if there are two trying this then we'll wait forever
		void wait()
			{
			STATE;
			assert(p_!=0);
			if (c_->count_<2) return;
			c_->lock();
			c_->wait();
			c_->unlock();
			}
		
		int count()
			{
			STATE;
			if (p_) return c_->count_;
			return -1;
			}

		template<class D> friend class ptr;

	private:  
		T *p_;
		counter *c_;
};
/*
template <class T> T* operator->(const ptr<T> &p){
	
}
*/
#endif
