// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  refcnt.hh - A reference counting template
  (c) Copyright 1998 Hakan Ardo <hakan@debian.org>

  NO LONGER USED IN THE MNEMONIC CORE, PLEASE LOOK AT `ptr.hh'.


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


   This code was derived from the C++ FAQ, with permission from 
   Marshall Cline, author of the C++ FAQ. The FAQ is accessible at:

     http://www.cerfnet.com/~mpcline/c++-faq-lite/
*/

#error "The `refcnt.hh' pointers should be replaced by `ptr.hh' pointers."

#ifndef refcnt_hh__
#define refcnt_hh__

#include <assert.h>
#include <mnemonic/thread.hh>
#include <mnemonic/thread_mutex.hh>
#include <mnemonic/thread_cond.hh>

template<class T> class ptr {
	public:
		T* operator-> () { assert(p_ != 0); return p_; }
		T& operator* ()  { assert(p_ != 0); return *p_; }
		void wait(void) const
			{
			p_->count_changed.lock();
			while(p_->count_>1) {
			   p_->count_changed.wait();
				}
			p_->count_changed.unlock();
			};
		ptr():p_(0) {}
		ptr(T* p)    : p_(p) 
			{ 
			if (p_) {
				p_->lock_.lock();
				++p_->count_;
				p_->lock_.unlock();
				p_->count_changed.lock();
				p_->count_changed.broadcast();
				p_->count_changed.unlock();
				}
			}  
		~ptr() 
			{ 
			if (p_) { 
				p_->lock_.lock();
				if (--p_->count_ == 0) {
					p_->lock_.unlock();
					delete p_; 
				} else { 
					p_->lock_.unlock();
					p_->count_changed.lock();
					p_->count_changed.broadcast();
					p_->count_changed.unlock();
					}
				}
			}
		ptr(const ptr<T>& p) : p_(p.p_) 
			{ 
			if (p_) {
				p_->lock_.lock();
				++p_->count_; 
				p_->lock_.unlock();
				p_->count_changed.lock();
				p_->count_changed.broadcast();
				p_->count_changed.unlock();
				}
			}
		bool operator== (const ptr<T>& p) const { return p_==p.p_; }
		bool operator < (const ptr<T>& p) const { return p_<p.p_;  }
		bool operator > (const ptr<T>& p) const { return p_>p.p_;  }
		ptr& operator= (const ptr<T>& p)
			{ // DO NOT CHANGE THE ORDER OF THESE STATEMENTS!
			// (This order properly handles self-assignment)
			if (p.p_) {
				p.p_->lock_.lock();
				++p.p_->count_;
				p.p_->lock_.unlock();
				}
			if (p_) {
				p_->lock_.lock();
				if(--p_->count_ == 0) {
					p_->lock_.unlock();
					delete p_;
				} else p_->lock_.unlock();
				}
			p_ = p.p_;
			p_->count_changed.lock();
			p_->count_changed.broadcast();
			p_->count_changed.unlock();
			return *this;
			}
		ptr& operator= (T* p)
			{ 
			if (p) {
				p->lock_.lock();
				++p->count_;
				p->lock_.unlock();
				}
			if (p_) {
				p_->lock_.lock();
				if(--p_->count_ == 0) {
					p_->lock_.unlock();
					delete p_;
				} else p_->lock_.unlock();
				}
			p_ = p;
			p_->count_changed.lock();
			p_->count_changed.broadcast();
			p_->count_changed.unlock();
			return *this;
			}
	private:
		T* p_;
}; 	

class untyped_referenced {
	public:
		thread_cond count_changed;
		int count(void) const;
	protected:
		int count_;	         // the number of ptr<T> objects that point at this
		thread_mutex lock_;  // lock whenever accessing count_
};

template<class T> class referenced : public untyped_referenced {
	protected:
		referenced(void) 
			{
			lock_.lock();
			count_=0;
			lock_.unlock();
			}
		referenced(const referenced &p) 
			{
			lock_.lock();
			count_=0;
			lock_.unlock();
			}
		referenced& operator= (const referenced &p) {return *this;}
		~referenced() 
			{
//			cerr << "count_ = " << count_ << endl;
			assert(count_ == 0);
			} 
	private:
		friend ptr<T>;
};

#endif
