#
# Mnemonic generic makefile rules.
#
# These makefile rules support creation of normal shared libraries, 
# message libraries and OIL libraries, as well as building RPM and
# Debian packages for distribution. They also provide automatic 
# support for installation. 
#
# To use these rules in your own project, add the include statements
# 
#   ifeq ($(installprefix),)
#     include @prefix@/share/mnemonic/Mnemonic.Make.vars
#   else
#     include $(installprefix)/share/mnemonic/Mnemonic.Make.vars
#   endif
#
#   all:       all-mnemonic
#   install:   install-mnemonic
#   clean:     clean-mnemonic
#   distclean: distclean-mnemonic
#
# This sets some variables and indicates that the usual rules 
# (all/install/clean/distclean) should make use of the Mnemonic 
# rules. Then define the variables described below. After 
# those definitions, add the include statements
# 
#   ifeq ($(installprefix),)
#     include @prefix@/share/mnemonic/Mnemonic.Rules.make
#   else
#     include $(installprefix)/share/mnemonic/Mnemonic.Rules.make
#   endif
#
# This produces a working Makefile.in (which of course still has
# to be processed by a configure script).
#
# If you want to build other programs in the same directory, just
# add those to the 'all:' target above (and to the others as well, 
# of course).
#
# The variables (if you are not creating one of these, you do not
# have to set the variables either) which trigger the Mnemonic 
# rules are:
#
# Creating an OIL:
#    OIL                  :  name of the OIL (just the name, no '.oil')
#    OIL_OBJS             :  .o files with implicit template instantiation
#    OIL_OBJS_MIO         :  .o files (message related, usually) with
#                               -fno-implicit-templates
#    OIL_LIBS             :  libraries to which the OIL should be 
#                               linked (including '-l')
#
# Creating a message library:
#    MSGS                 :  name of message group (just the name, no 
#                                'lib' or '_messages.so')
#    MSGS_OBJS            :  .o files (templates have to be instantiated 
#                                by hand)
#    MSGS_OBJS_NMIO       :  .o files (templates are instantiated 
#                                automatically)
#    MSGS_LIBS            :  libraries to be linked to the message library
#
# Creating a normal shared library:
#    SO                   :  name of the library (without 'lib' and
#                                 without '.so.*')
#    SO_OBJS              :  .o files to go into the library
#    SO_LIBS              :  libraries to be linked to the shared 
#                                 library being created
#

VSTRING=$(VERSION).$(PATCHLEVEL).$(SUBLEVEL)
LVSTRING=$(VERSION).$(PATCHLEVEL)

all-mnemonic: $(MSGS) $(SO) $(OIL) $(OIL_INTEREST) subdirs

#-----------------------------------------------------------------------
$(OIL_OBJS): %.o: %.cc
	$(CPLUSPLUS) $(CFLAGS) $(PROFILE) $(GCFLAGS) -fPIC -c -o $@ $<

$(OIL_OBJS_MIO): %.o: %.cc
	$(CPLUSPLUS) $(CFLAGS) $(PROFILE) $(GCFLAGS) -fno-implicit-templates -fPIC -c -o $@ $<

$(MSGS_OBJS): %.o: %.cc
	$(CPLUSPLUS) $(CFLAGS) $(PROFILE) $(GCFLAGS) -fno-implicit-templates -fPIC -c -o $@ $<

$(MSGS_OBJS_NMIO): %.o: %.cc
	$(CPLUSPLUS) $(CFLAGS) $(PROFILE) $(GCFLAGS) -fPIC -c -o $@ $<

$(SO_OBJS): %.o: %.cc
	$(CPLUSPLUS) $(CFLAGS) $(PROFILE) $(GCFLAGS) -fPIC -c -o $@ $<

#-----------------------------------------------------------------------
ifneq ($(strip $(OIL)),)
$(OIL): $(OIL).oil.$(VSTRING)
	@echo "=> oil '$(OIL)' built"

$(OIL).oil.$(VSTRING): $(OIL_OBJS) $(OIL_OBJS_MIO)
	@PATH=$(prefix)/bin:$(PATH) \
	mnemonic-createlib $(OIL).oil $(VSTRING) $(LVSTRING) \
                      $(OIL_OBJS) $(OIL_OBJS_MIO) $(LDPATHS) $(OIL_LIBS) $(PROFILE) \
                      -L$(prefix)/lib -lmnemonic 
#	ln -fs lib$(OIL).oil.$(VSTRING) lib$(OIL).oil.$(LVSTRING)
#	ln -fs lib$(OIL).oil.$(LVSTRING) lib$(OIL).oil
	@PATH=$(prefix)/bin:$(PATH) \
	LD_LIBRARY_PATH=$(prefix)/lib:$(prefix)/lib/mnemonic/msg:$(LD_LIBRARY_PATH) \
   dlopentest ./$(OIL).oil.$(VSTRING)
endif


#-----------------------------------------------------------------------
ifneq ($(strip $(SO)),)
$(SO): lib$(SO).so.$(VSTRING)
	@echo "=> shared library 'lib$(SO).so' built"

lib$(SO).so.$(VSTRING): $(SO_OBJS)
	@PATH=$(prefix)/bin:$(PATH) \
	mnemonic-createlib lib$(SO).so $(VSTRING) $(LVSTRING) \
                      $(SO_OBJS) $(LDPATHS) $(SO_LIBS) $(PROFILE) -L$(prefix)/lib -lmnemonic 
	ln -fs lib$(SO).so.$(VSTRING) lib$(SO).so.$(LVSTRING)
	ln -fs lib$(SO).so.$(LVSTRING) lib$(SO).so
	@PATH=$(prefix)/bin:$(PATH) \
	LD_LIBRARY_PATH=$(prefix)/lib:$(prefix)/lib/mnemonic/msg:$(LD_LIBRARY_PATH) \
   dlopentest ./lib$(SO).so.$(VSTRING)
endif

#-----------------------------------------------------------------------
ifneq ($(strip $(MSGS)),)
$(MSGS): lib$(MSGS)_messages.so.$(VSTRING)
	@echo "=> message library '$(MSGS)' built"

lib$(MSGS)_messages.so.$(VSTRING): $(MSGS_OBJS) $(MSGS_OBJS_NMIO)
	@PATH=$(prefix)/bin:$(PATH) \
	mnemonic-createlib lib$(MSGS)_messages.so $(VSTRING) $(LVSTRING) \
                               $(MSGS_OBJS) $(MSGS_OBJS_NMIO) $(MSGS_LIBS) $(PROFILE) \
                               -L$(prefix)/lib -lmnemonic 
	ln -fs lib$(MSGS)_messages.so.$(VSTRING) lib$(MSGS)_messages.so.$(LVSTRING)
	ln -fs lib$(MSGS)_messages.so.$(LVSTRING) lib$(MSGS)_messages.so
	@PATH=$(prefix)/bin:$(PATH) \
	LD_LIBRARY_PATH=$(prefix)/lib:$(prefix)/lib/mnemonic/msg:$(LD_LIBRARY_PATH) \
   dlopentest ./lib$(MSGS)_messages.so.$(VSTRING) 
endif

#-----------------------------------------------------------------------
install-mnemonic: 
ifneq ($(strip $(MSGS)),)
	install -d $(prefix)/lib/mnemonic/msg
	install -d $(prefix)/include/mnemonic/msg/$(MSGS)
	install lib$(MSGS)_messages.so.$(VSTRING) $(prefix)/lib/mnemonic/msg
	( cd $(prefix)/lib/mnemonic/msg && \
     ln -fs lib$(MSGS)_messages.so.$(VSTRING) \
            lib$(MSGS)_messages.so.$(LVSTRING) && \
	  ln -fs lib$(MSGS)_messages.so.$(LVSTRING) \
            lib$(MSGS)_messages.so \
   )
	cp -uvaf ../include/msg/$(MSGS)/*.h* $(prefix)/include/mnemonic/msg/$(MSGS)
	find $(prefix)/include/mnemonic/msg/$(MSGS) -type f -not -perm 644 -exec chmod 644 '{}' ';'
endif
ifneq ($(strip $(OIL)),)
	install -d $(prefix)/lib/mnemonic/oil
	install $(OIL).oil.$(VSTRING) $(prefix)/lib/mnemonic/oil
	( cd $(prefix)/lib/mnemonic/oil && \
     ln -fs $(OIL).oil.$(VSTRING) \
            $(OIL).oil.$(LVSTRING) && \
     ln -fs $(OIL).oil.$(LVSTRING) \
            $(OIL).oil \
   )
endif
ifneq ($(strip $(SO)),)
	install -d $(prefix)/lib
	install lib$(SO).so.$(VSTRING) $(prefix)/lib
	( cd $(prefix)/lib && \
     ln -fs lib$(SO).so.$(VSTRING) \
            lib$(SO).so.$(LVSTRING) && \
     ln -fs lib$(SO).so.$(LVSTRING) \
            lib$(SO).so \
   )
	install -d $(prefix)/include/$(SO)
	cp -uvaf ../include/$(SO)/*.h* $(prefix)/include/$(SO)
	find $(prefix)/include/$(SO) -type f -not -perm 644 -exec chmod 644 '{}' ';'
endif

#-----------------------------------------------------------------------
depend:
	if [ "$(wildcard *.cc)" != "" ]; then \
		$(DEPEND) $(wildcard *.cc) > .depend; \
	fi
	list="$(SUBDIRS)"; for subdir in $$list; do \
		(cd $$subdir && $(MAKE) depend); \
	done

#-----------------------------------------------------------------------
subdirs:
	list="$(SUBDIRS)"; set -e; for subdir in $$list; do \
	  $(MAKE) -C $$subdir all $(EXPORT); \
	done

#-----------------------------------------------------------------------
clean-mnemonic:
	-rm -f $(OIL_OBJS) $(OIL_OBJS_MIO) 
	-rm -f $(OIL).oil.*.*.* $(OIL).oil.*.* $(OIL).oil.* $(OIL).oil
	-rm -f $(MSGS_OBJS)
	-rm -f lib$(MSGS)_messages.*.*.*
	-rm -f stamp-h stamp-h.in
	-rm -f $(SOFILES)
	-rm -f *.o *~ *.a core
	-rm -f *.so* 
	list="$(SUBDIRS)"; for subdir in $$list; do \
	  (cd $$subdir && $(MAKE) $(EXPORT) clean); \
	done

#-----------------------------------------------------------------------
distclean-mnemonic:
	-rm -f optionals.h
	list="$(SUBDIRS)"; for subdir in $$list; do \
		(cd $$subdir && $(MAKE) $(EXPORT) distclean); \
	done
	$(MAKE) clean
	-rm -f Makefile config.h .depend Make.vars

#-----------------------------------------------------------------------
ifeq (.depend,$(wildcard .depend))
include .depend
endif


#-----------------------------------------------------------------------
# Rules used to build the debian packages

debian-build-stamp:
	dh_testdir

	./autogen.sh --prefix=/usr --with-gtk=/usr --with-jpeg=/usr

	echo The mnemonic packages are created from the distributions > debian/copyright
	echo availible at http://www.mnemonic.org/ by the mnemonic>> debian/copyright
	echo 'developers themselfes (debian-maintainers@mnemonic.org).' >> debian/copyright
	echo >> debian/copyright
	echo Copyright: GPL>> debian/copyright

	$(MAKE)

	(if test -f doc/Makefile; then cd doc && $(MAKE); fi)
	touch debian-build-stamp

debian-clean:
	dh_testdir
	dh_testroot
	rm -f debian-build-stamp

	# Add here commands to clean up after the build process.
	-$(MAKE) distclean
	-(cd doc && $(MAKE) clean)
	-rm debian/copyright

	dh_clean


debian-binary-arch:
#	dh_testversion
	dh_testdir -a
	dh_testroot -a
	dh_clean -a -k
	dh_installdirs -a

	# Add here commands to install the files into debian/tmp
	$(MAKE) install prefix=`pwd`/debian/tmp/usr
	mkdir `echo debian/tmp/usr/doc/*`/html
	-(if test -f doc/Makefile; then cp doc/www/*.html doc/*.html  debian/tmp/usr/doc/*/html; fi)

	dh_movefiles -a
	dh_installdocs -a
#	dh_installexamples -a
#	dh_installmenu -a
#	dh_installinit -a
#	dh_installcron -a
#	dh_installmanpages -a
	dh_undocumented
	dh_installchangelogs -a
	dh_strip -a
	dh_compress -a
	dh_fixperms -a
	dh_suidregister -a
	dh_installdeb -a
	dh_shlibdeps -a
	dh_gencontrol 
#	dh_makeshlibs -a
	dh_md5sums -a
	dh_builddeb -a
