// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Mnemonic debugging output routines, implementation.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <string>

#include <stdio.h>
#include <mnemonic/optionals.h>
extern "C" {
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>
}

#include "mnemonic/debugger.hh"

#ifdef DEBUG

void segvhandler(int i)
	{
	cerr<<"("<<pthread_self()<<")"<<"SIGNAL "<<i<<"\n"<<flush;
	exit(-1);
	}

debugger* debugger::instance_=0;

debugger::debugger(void)
	{
	}

debugger& debugger::instance(void)
   {
   if(instance_==0) {
     instance_=new debugger();
     signal(11,segvhandler);   
    }
   return *instance_;
   }

void debugger::turn_on(const char *cl) 
   {
   debugmod_[cl]=true;
   }

void debugger::turn_off(const char *cl) 
   {
   debugmod_[cl]=false;
   }

// Should be called without having the lock. Will release the 
// lock so you'll have to acquire it before outputting.
bool debugger::check_debug_class(const char *cl) const
	{
	bool dodebug=false;
	l_mutex.lock();
   if(debugmod_.find(cl) == debugmod_.end()) { 
		if(debugmod_.find("all") != debugmod_.end())
			dodebug=true;
		else if(debugmod_.find("ALL") != debugmod_.end())
		   dodebug=true;
		}
	else dodebug=true;
	l_mutex.unlock();
	return dodebug;
	}

// Only call this when you have acquired the lock.
void debugger::stamp(strstream &str,const char *file, int line) const
   {
	struct timeval  tv; 
	struct timezone tz;
	gettimeofday(&tv,&tz);
	str << "(" << pthread_self() << "/" 
		  << tv.tv_sec << ":"
		  << tv.tv_usec << ")"
		  << file << ":" << line << ": ";
	} 

void debugger::lock(void)
	{
	l_mutex.lock();
	}

void debugger::unlock(void)
	{
	l_mutex.unlock();
	}

void debugger::setsighandler()
	{
	signal(11,segvhandler);
	}

#endif
