
#include "mnemonic/optionals.h"

#ifdef MEMDEBUG
#include <stdio.h>
#include <dlfcn.h>
#include <pthread.h>
#include <iostream.h>
/*
 this code keeps a list of allocated chunks of memory and removes those which
 are freeed.

 if this doesn't want to load because it cant open libc.so.6 try libc.so
 or libc.so.5. I need libc.so.6 because libc.so seems to contain an
 incorrect malloc.
*/

static void * (*xmalloc)(size_t)=0;
static void (*xfree)(void *)=0;
static void * (*xrealloc)(void *,size_t)=0;
static char *(*xstrdup)(const char *)=0;
char *nct="unknown";
char *mtype[]={"new","new[]","malloc"};
void addkm(int t,long l,char *c,void *p,int s);
void rmkm(int t,void *p);
void printkm();

/*
  this is just a trick 
  the constructor doesn't have to be the first that is called
  and the destructor doesn't also have to be the last that is called
  so somethings reported allocated might get freeed afterall
  
  (this could be solved by adding extra checks which arr writing direct
  to the cerr.)
*/
class mdexit {
public:
  mdexit(){
  }
  ~mdexit(){
    printkm();
  }  
};

mdexit mdb;
int dln=0;

pthread_mutex_t memmutex=PTHREAD_MUTEX_INITIALIZER;

void findmalloc(){
  char *error;
  void *handle=dlopen("libc.so.6",RTLD_LAZY);
  if (!handle) {
    cerr<<"memdebug: dlopen has to be able to find the RIGHT libc\n";
    fputs(dlerror(), stderr);
    exit(1);
  }

  xmalloc=(void *(*)(size_t))dlsym(handle,"malloc");
  if ((error=dlerror()) != 0) {
    fputs(dlerror(), stderr);
    exit(1);
  }
  xrealloc=(void *(*)(void *,size_t))dlsym(handle,"realloc");
  if ((error=dlerror()) != 0) {
    fputs(dlerror(), stderr);
    exit(1);
  }

  xfree=(void (*)(void *))dlsym(handle,"free");
  if ((error=dlerror()) != 0) {
    fputs(dlerror(), stderr);
    exit(1);
  }

  xstrdup=(char *(*)(const char *))dlsym(handle,"strdup");
  if ((error=dlerror()) != 0) {
    fputs(dlerror(), stderr);
    exit(1);
  }

}

void *mxmalloc(size_t k) {
  if (xmalloc==0) findmalloc();
  return xmalloc(k);
}
char *mxstrdup(const char *r){
  if (xstrdup==0) findmalloc();
  return xstrdup(r);
}

void mxfree(void *p) {
  if (xfree==0) findmalloc();
  xfree(p);
}

void *mxrealloc(void *p,size_t k) {
  if (xrealloc==0) findmalloc();
  return xrealloc(p,k);
}

void *operator new(size_t k){
  void *p=mxmalloc(k);
  addkm(0,dln,nct,p,k);
  dln++;
  return p;
}

void *operator new[](size_t k){
  void *p=mxmalloc(k);
  addkm(0,dln,nct,p,k);
  dln++;
  return p;
}

void *malloc(size_t k){
  void *p=mxmalloc(k);
  addkm(2,dln,nct,p,k);
  dln++;
  return p; 
}

char *strdup(const char *p){
  char *r=mxstrdup(p);
  addkm(2,dln,nct,(void *)r,strlen(p)+1);
  dln++;
  return r;
}

void *realloc(void *p,size_t k){
  if (p!=0) rmkm(2,p);
  p=mxrealloc(p,k);
  addkm(2,dln,nct,p,k);
  dln++;
  return p;
}




void *operator new(size_t k,char *fname,long lnum) {
  if (k==0) return 0;
  void *p=mxmalloc(k);
  addkm(0,lnum,fname,p,k);
  return p;
}
void *operator new[](size_t k,char *fname,long lnum){
  if (k==0) return 0;
  void *p=mxmalloc(k);
  addkm(1,lnum,fname,p,k);
  return p;
}
void *mdmalloc(size_t k,char *fname,long lnum){
  if (k==0) return 0;
  void *p=mxmalloc(k);
  addkm(2,lnum,fname,p,k);
  return p;
}
void *mdrealloc(void *p,size_t k,char *fname,long lnum){
  fprintf(stderr,"realloc from %x\n",(int)p);
  if (p!=0) rmkm(2,p);
  if (k==0) return 0;
  p=mxrealloc(p,k);
  addkm(2,lnum,fname,p,k);
  fprintf(stderr,"realloc to %x\n",(int)p);
  return p;
}
char *mdstrdup(const char *p,char *fname,long lnum){
  char *r=mxstrdup(p);
  addkm(2,lnum,fname,(void *)r,strlen(p)+1);
  return r;
}

void mdfree(void *p){
//  fprintf(stderr,"free %x\n",(int)p);
  if (p==0) return;
  rmkm(2,p);
  mxfree(p);
}

struct km {
  int t;   /* type allocatie */
  long l;   /* line num */
  char *c; /* file name */
  void *p; /* the pointer*/
  int s;   /* size in byte */
};

km *kmlist=0;
int kmn=0;

void addkm(int t,long l,char *c,void *p,int s){
  pthread_mutex_lock(&memmutex);
  int i=0;
  while (i<kmn) {
    if (kmlist[i].p==0) {
      kmlist[i].l=l;
      kmlist[i].p=p;
      kmlist[i].c=c;
      kmlist[i].t=t;
      kmlist[i].s=s;
      pthread_mutex_unlock(&memmutex);
      return;
    }
    i++;
  }
  kmn++;
  kmlist=(km *)mxrealloc(kmlist,kmn*sizeof(km));
  kmlist[kmn-1].p=p;
  kmlist[kmn-1].c=c;
  kmlist[kmn-1].t=t;
  kmlist[kmn-1].l=l;
  kmlist[kmn-1].s=s;
  pthread_mutex_unlock(&memmutex);
  
}

void rmkm(int t,void *p){
  pthread_mutex_lock(&memmutex);
  int i=0;
  while (i<kmn) {
    if (kmlist[i].p==p) {
      kmlist[i].p=0;
      if (kmlist[i].t==t) {
        pthread_mutex_unlock(&memmutex);
        return;
      }
      fprintf(stderr,"dealloc error: type differs %s %s\n",
                      mtype[kmlist[i].t],mtype[t]);
      fprintf(stderr,"%li:%s\n",kmlist[i].l,kmlist[i].c);
      pthread_mutex_unlock(&memmutex);
      return;
    }
    i++;
  }
  fprintf(stderr,"dealloc error: pointer not found\n");
  pthread_mutex_unlock(&memmutex);

}

void printkm(){
  pthread_mutex_lock(&memmutex);
  int i;
  int k=0;
  fprintf(stderr,"tabel of still allocated memory chunks[%i]\n",kmn);
  fprintf(stderr,"type size:[ptr][line nr]file\n");
  for (i=0;i<kmn;i++) {
    if (kmlist[i].p!=0) {
    fprintf(stderr,"%-6s %10li:[%x][%6i]%s\n",mtype[kmlist[i].t],kmlist[i].s,(int)kmlist[i].p,kmlist[i].l,kmlist[i].c);
      k+=kmlist[i].s;
    }
  }
  fprintf(stderr,"total: %i\n",k);
  //flush(stderr);
  pthread_mutex_unlock(&memmutex);
}
#endif
