// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Mnemonic Object Broker.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dlfcn.h>
#include <stdexcept>
#include <string.h>

#include "objectbroker.hh"

#define DEBUG_CLASS "OBJECTBROKER"
#include "mnemonic/debug.hh"

object_broker::object_broker(void) 
   {
   }

object_broker::~object_broker() 
   {
   // FIXME: we should unload all OILs and free memory used by
   // new Interest and such which is claimed in the create_* functions.
   // Procedure: delete each object created using create_*, and
   // then call destroy_* (which we should still make using the
   // auto *interest.cc generator) on the OIL.
   /*   map<string,mnobject,less<string> >::iterator it;
   it=objects_.begin();
   while(it!=objects_.end())
	  { void* (*destr)(void**);  
	    destr = (void *(*)(void **) )(dlsym((*p.first).second.oilhandle, destroy_func));
   if ((error = dlerror()) != NULL)  
	  { debug("dlsym failed: " << error);
	    return (NULL);
	  }
		dlclose((*it).second.oilhandle);
	  } */
   }

// Load an oil. Throw an exception when loading failed. 
void *object_broker::loadoil(const char *oilname) 
   {
   debug("oilload: " << oilname);
   void *handle= dlopen(oilname, RTLD_NOW|RTLD_GLOBAL );
   if( handle == NULL ) { 
		const char* error;
		if ((error = dlerror()) != NULL) {
			debug("dlopen err: " << string(error) );
			string tmp="dlopen error while loading "+string(oilname)+": ";
			tmp+=string(error);
			debug("throw");
			throw logic_error(tmp.c_str());
		   }
		}
   return (handle);
   }

dlobject *object_broker::create_object(const char *oilname, 
								 const char *create_func, 
								 void **ctorArgs)
   {
   const char *error;
   dlobject * (*func)(void**); 
   
   mnobject nob;
   nob.loadcount=0;
   nob.oilhandle=0;
   pair<map<string,mnobject,less<string> >::iterator,bool> p = 
		objects_.insert(pair<const string,mnobject>(string(oilname),nob));
   
   debug("loading " << oilname << endl);
   (*p.first).second.oilhandle=loadoil(oilname);
   debug("trying to resolve " << create_func << endl);
   func = (dlobject *(*)(void **) )(dlsym((*p.first).second.oilhandle, create_func));
   if ((error = dlerror()) != NULL) { 
		debug("dlsym failed: " << error);
		throw logic_error("Error resolving create function in "+string(oilname));
		}
   (*p.first).second.loadcount++;
	debug ("done");
   return( (*func)(ctorArgs) ); 
   }

bool object_broker::delete_object(void *Object, char *objID)
   {
   map<string,mnobject,less<string> >::iterator it;
   if((it=objects_.find(objID)) == objects_.end())
		return false;

   dlclose((*it).second.oilhandle);
   if( --((*it).second.loadcount) == 0)
		objects_.erase(it);
   return true;
   }
