// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Mnemonic Object Manager.             

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <string>
#include <fstream.h>
#include <strstream.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <set.h>

#include "mnemonic/dlobject.hh"
#include "mnemonic/objectmanager.hh"
#include "objectbroker.hh"

#define DEBUG_CLASS "OBJECTMANAGER"
#include "mnemonic/debug.hh"

object_manager* object_manager::instance_=0;

dlobject::~dlobject()
	{
	}

object_manager& object_manager::instance(void)
   {
   if(instance_==0) instance_=new object_manager();
   return *instance_;
   }

void object_manager::free_instance(void)
   {
   if(instance_) {
      delete instance_;
      }
   }

object_manager::object_manager(void)
   {
   if(getenv("HOME"))
		dirs_.push_back(string(getenv("HOME"))+"/oil");
   if(getenv("OIL_PATH"))
		dirs_.push_back(string(getenv("OIL_PATH")));
   dirs_.push_back(string(PREFIX)+"/lib/mnemonic/oil");
   
   OB_         = new object_broker();
   }

void object_manager::load_interests(void)
   {
   DIR *d;
   struct dirent *ent;
   set<string,less<string> > names;
   set<string,less<string> >::iterator i;
   vector<string>::iterator dir;

	for (dir=dirs_.begin(); dir!=dirs_.end(); dir++) { 
		debug ("scanning " << *dir << " for OILs");
		if ((d=opendir((*dir).c_str()))) { 
			while ((ent=readdir(d))) { 
				if (strlen(ent->d_name) > 4 && 
					 !strcmp(".oil",
								ent->d_name+strlen(ent->d_name)-4)) { 
					debug("found OIL " << ent->d_name);
				   names.insert(string(ent->d_name));
					}
				}
		   } 
		else { 
			debug ("failed on opendir (non-fatal)");
		   }
		}
   
   for (i=names.begin(); i!=names.end(); i++) { 
#ifdef DEBUG
		debugger::instance().lock();
#endif
		cerr << "loading module " << *i << endl;
#ifdef DEBUG
		debugger::instance().unlock();
#endif
		this->new_object_by_filename((*i).c_str(), NULL);
		}
#ifdef DEBUG
	debugger::instance().lock();
#endif
	cerr << "all modules loaded." << endl;
#ifdef DEBUG
	debugger::instance().unlock();
#endif
   }

object_manager::~object_manager()
   {
	map<string, dlobject *>::iterator it=loaded_.begin();
	while(it!=loaded_.end()) {
    		debug("still have to unload " << (*it).first);
		if((*it).second!=0) {
			delete (*it).second;
			debug("delete successful");
		} else {
		    debug("second = 0");
		}
		// OB_->delete_object((*it).first);
		++it;
		}
    debug("almost done");
   delete OB_;
   debug("done");
   }

// Returns 0 if object was already loaded
// FIXME: that's ugly!
dlobject * object_manager::new_object_by_filename(const char *name, void **ctorargs)
   {
	create_lock_.lock();

   debug ("new_object_by_filename: " << name );
	if(loaded_.find(name)!=loaded_.end()) {
		create_lock_.unlock();
      //	object has already been loaded
		return 0;
		}

   ifstream oilstream;
   string filename;
   vector<string>::iterator dir=dirs_.begin();
   while (!oilstream.is_open()) { 
		if (dir==dirs_.end()) { 
			create_lock_.unlock();
			strstream buf;
			buf << "[object_manager]: requested object '" << filename << "' cannot be opened" << ends;
			om_exception exc(om_exception::object_nonexistent,buf.str());
			delete [] buf.str();
			debug("throw");
			throw exc;
		   }
		filename=*dir+"/"+name;
		oilstream.open(filename.c_str());
		dir++;
		}
   oilstream.close();
   debug ("found " << filename);
	
   string createfuncname=string("create_")+name;
   createfuncname=createfuncname.substr(0,createfuncname.length()-4);   // FIXME: dirty hack
	dlobject *Object;
   try { 
		Object=OB_->create_object(filename.c_str(), createfuncname.c_str(), ctorargs);
		}
   catch(exception& err) { 
		debug("cannot create object!!!");
		create_lock_.unlock();
		debug("throw");
		throw;
		}
	// FIXME: we allow for create_object to return a 0 pointer, still accepting this
	// as a valid return code (ie. your oil can just do something yet not produce a real object).
   // It's a bit ugly.
	loaded_[name]=Object;
	create_lock_.unlock();
   return (Object);
   }

void object_manager::delete_object_by_id(dlobject * Object, char *objID)
   {
   if(!(OB_->delete_object(Object, objID))) {
		debug("throw");
		throw om_exception(om_exception::object_not_loaded,"tried to delete nonexistent object");
		}
   }

object_manager::om_exception::om_exception(om_exception::om_error err, const char *str) :
	logic_error(str)
	{
	strstream buf;
	buf << "[om_exception]: status code: " << (int)err << ", " << str << ends;
	local_what_ = buf.str();
	status_=err;
	}

object_manager::om_exception::om_error object_manager::om_exception::code(void) const
	{
	return status_;
	}

const char* object_manager::om_exception::what () const
	{
	return local_what_.c_str();
	}

