
#include <iostream.h>

template<class T>
class message {
};

template<class T>
class receiver {
   protected:
     virtual bool accept(T&) {
    	cerr << "base accept" << endl;
	   };
};

class msg : public message<msg> {
   public:
};

class msg2 : public message<msg2> {
   public:
};

class rec {
	public:
      rec(void) {
        recmsg_.connect(receive_msg);
        recmsg2_.connect(receive_msg2);
		  };
	protected:
      virtual bool receive_msg(msg& obj);
      virtual bool receive_msg2(msg& obj);
   private:
      receiver<msg>  recmsg_;
		receiver<msg2> recmsg2_;
};

int main(int argc, char **argv)
	{
	return 0;
	}


/*

If we send a message in a new thread, a new receiver object should be created
(there is no point in sending a message in a new thread to an object that was
created in a previous thread, is there?)

But if you send a synchronous message, you may _still_ want a new
receiver object to be created (Our current 'load all at startup'
method is a bit weird really). This happens for instance when you want
to exec an external process listening to your message.




Wiring info (can be built using graphical tool if you want) contains
all info about which modules should be tried for a given message send. Eg.

mnemonic.cfg:

msg_get_url  { source: 
                 *;
               destination: 
                 /usr/local/mnemonic/urlhandlers/* : internal,
                 /usr/local/bin/wget_mn            : external
             };

How do we specify the connection from ls output to another module?
It is not typed (eg. just raw data).

raw          { source:      /usr/bin/ls;
               destination: /usr/local/bin/ 

};

Ehm, this gets more ugly all the time. You want this with command line options 
like we did before! The only realy difference is that we want the startup of
these programs to be delayed, not like a normal shell activation.


/usr/local/mnemonic/uslhandlers/*  --msg_get_url:(1,2)
/usr/local/bin/wget_mn             --msg_get_url:
/usr/bin/ls

But we still also want to be able to do quick things using a pipe notation,

  viewer:lscommand(raw)   | stdin(raw):ls:stdout(raw)   | lsview(raw):viewer
  viewer:get(msg_get_url) | stdin(raw):wget:stdout(raw) | sourceview(raw):viewer

or with more listeners

  viewer:get(msg_get_url) | stdin(raw):wget:stdout(raw)       | sourceview(raw):viewer
                          | stdin(ftpcommand):ftp:stdout(raw) |                       

or maybe better

  viewer:get(msg_get_url) | stdin(raw):wget:stdout(raw), stdin(ftpcommand):ftp:stdout(raw)
   | sourceview(raw):viewer
  
(or perhaps do all links in terms of single pipes and then use object identifiers to
get the graph structure). 

Then we need converters between various message types, eg something that turns a msg_get_url
object into a command understood by wget. Those type of converters could be simple
scripts. And we need a way to specify that two handlers in the chain are actually
supposed to be the same object, eg

  viewer[1]:get(msg_get_url) | stdin(raw):wget:stdout(raw) | viewer(raw):viewer[1]


conversion could also be done using

  viewer[1]:get | msg_get_url2wget | stdin:wget:stdout, stdin:ftp:stdout | view:viewer[1]

But this does not work, as you need to split _before_ converting the message type.
So perhaps

  viewer[1]:get | msg_get_url2wget | stdin:wget:stdout | 
                | msg_get_url2ftp  | stdin:ftp:stdout  | view:viewer[1]

The ending bar on the first line indicates that all output should go
into the last element on the last line. We could also do it in a table
format, because the above will become ugly very soon. Eg

viewer[1]:get_url         >1 &; \\   // this is not the right way to spawn new threads 
  urlhandlers/*:get_url     <1 #; \\
  urlhandlers/*:data        >4; \\
  msg_get_url2wget:stdin    <1; \\
  msg_get_url2wget:stdout   >2; \\
  wget:stdin                <2; \\
  wget:stdout               >4; \\
  msg_get_url2ftp:stdin     <1; \\
  msg_get_url2ftp:stdout    >3; \\
  ftp:stdin                 <3; \\
  ftp:stdout                >4; \\
  viewer[1]:view            <4;

 # : same address space
 & : separate process/thread

msg_run_client should go, it's unnecessary. (How do we open new windows?)

(this sequence will start 'viewer' and, in contrast to other shells, only open
pipes but not connect them yet to the next programs in the pipeline).
Then we need some way to identify programs that have multiple ports (as opposed
to stdin/stdout/stderr) so that we can do typechecking on the message conversion
whenever type info is available:

  viewer --list-ports

     get   : out, msg_get_url
     view  : in,  raw



Internal messages will have to identify themselves (but that's
automatic!), but need not be serialised/de-serialised.

class msg_get_url : public message<msg_get_url> {
  public:
     virtual string serialise(void);
     virtual void   deserialise(string);
};


Perhaps we should have some `global' directory with definitions
of `shareable objects', together with language dependent modules which
allow you to (de)serialise the way you want, eg.

   messages
     Makefile.global
     get_url
       xml (contains enough to make interactive construction of messages possible)
       c++
       python
       wget_wrap
       ftp_wrap

Then you can just download get_url.tar.gz, plug it in and compile.


 */
