// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Mnemonic thread wrapper classes for portable threads, implementation.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "mnemonic/thread.hh"
#include "mnemonic/thread_mutex.hh"
#include "mnemonic/thread_cond.hh"
#include "mnemonic/messagebase.hh"
#include <pthread.h>
#include <errno.h>
#include <unistd.h>

#define DEBUG_CLASS "THREAD"
#include "mnemonic/debug.hh"

#define NOTHREAD 0
#define RUNNING 1

thread::thread(void) 
	{
	ok_=NOTHREAD;
	}

void *threadstart(void *p)
	{
	thread *t=(thread *)p;
	t->prebegin();
	return 0;
	}

void thread::prebegin()
	{
	try {
		//make sure we are at the wait
		ok_=RUNNING;
		thr_.lock();				
		thr_.broadcast(); 
		thr_.unlock();
		begin();
		}
	catch(messagebase_error &ex) {
		cerr << "exception: " << ex << endl;
		throw;
		}
	catch(exception &ex) {
		cerr << "exception: " << ex.what() << endl;
		throw;
		}
	}

void thread::start(void)
	{
	thr_.lock();
	if (pthread_create(&rthread, 0, threadstart, (void *)this) == 0) {
		thr_.wait();
		thr_.unlock();
		return;
		}
	debug("throw");
	throw logic_error("out of resources on creation of a thread\n");
	}

void thread::cancel()
	{
	//replace with anything needed to signal the thread to stop
	}

void thread::stop(void)
	{
	debug(this<<" cancel_blocked");
	thr_.lock();
	if (ok_==NOTHREAD) {//NOTHREAD 
		//no thread running
		thr_.unlock();
		return;
		}
	switch (pthread_join(rthread,0)){
		case 0:
		break;
		case ESRCH:
		// rthread is not running
			break;
		case EINVAL:
			// rthread detached or another thread is waiting on a join
			debug("throw");
			throw logic_error("thread detached or another is waiting on a join"); 
		case EDEADLK:         
			// BUG? we try to join our self! rthread=thread_self()
			debug("throw");
			throw logic_error("BUG: do not try to cancel yourself");
		default:
			debug("throw");
			throw logic_error("unknown error in thread::cancel_blocked");
		}
	ok_=NOTHREAD;
	thr_.unlock();
	}

thread::~thread() 
	{
	thr_.lock();
	if (ok_==RUNNING) {
		thr_.unlock();
		debug("throw");
		throw logic_error("thread is still running cancel and stop it first");
		}  
	thr_.unlock();
	}

