// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Thread conditions, implementation.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <time.h>
#include <mnemonic/thread_cond.hh>
#include <stdexcept>
#include <errno.h>

#define DEBUG_CLASS "THREAD"
#include "mnemonic/debug.hh"

thread_cond::thread_cond() : thread_mutex()
	{
        debug("thread_cond"<<this);
	pthread_cond_init(&cond, pthread_condattr_default);
	}

void thread_cond::broadcast() 
	{ 
//	debug("broadcast "<<this);
	pthread_cond_broadcast(&cond); 
	}

void thread_cond::signal() 
	{ 
	debug("signal "<<this);
	pthread_cond_signal(&cond); 
	}

void thread_cond::wait() 
	{ 
	debug("wait "<<this); // FIXME: I think trylock() should NOT be called here, but
	// I might be wrong [kasper]. 
//	if (trylock()==EBUSY) {
//		debug("throw");
//		throw logic_error("a condition should be locked before wait is called");
//		}
	pthread_cond_wait(&cond, &mutex); 
	}

int thread_cond::wait(int delay) 
	{ 
	debug("wait "<<this);
	struct timespec ts={time(NULL)+delay,0};
	int retval;
	retval=pthread_cond_timedwait(&cond, &mutex,&ts); 
	while (retval==EINTR) retval=pthread_cond_timedwait(&cond, &mutex,&ts); //we ignore signals
	if (retval==ETIMEDOUT) return 1;
	return 0;
	}

thread_cond::~thread_cond() 
	{ 
	debug("destruct "<<this);
	pthread_cond_destroy(&cond); 
	}
