// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Thread mutexes, implementation.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <iostream.h>
#include <stdexcept>
#include <errno.h>
#include "mnemonic/thread_mutex.hh"

/* DO NOT use the debug function here. Debug depends on the thread classes
   to make sure that output of various threads does not get mixed, so if 
   you call debug in this module it will recurse without end.
*/


#ifdef __USE_UNIX98
thread_mutex::thread_mutex(int type=PTHREAD_MUTEX_ERRORCHECK)
#else
thread_mutex::thread_mutex(int type=1)
#endif
   {
	pthread_mutexattr_t errchkmutex;

#ifdef __USE_UNIX98
	pthread_mutexattr_settype(&errchkmutex, type);
#endif

   if (pthread_mutex_init(&mutex, &errchkmutex) != 0) {
		cerr << "mutex failed to initialise" << endl;
		throw logic_error("mutex failed to initialise");
		}
   }

int thread_mutex::lock(void) 
   { 
//  cerr<<this<<" locked\n";cerr.flush();
   int retval=pthread_mutex_lock(&mutex); 
	switch(retval) {
		case EDEADLK:
			cerr << "tried to lock a mutex which was already locked" << endl;
			throw logic_error("tried to lock a mutex which was already locked");
		}
   return retval; 
   }

int thread_mutex::unlock(void) 
   { 
	int retval=pthread_mutex_unlock(&mutex); 
	switch(retval) {
		case EPERM:
			cerr << "calling thread does not own mutex for unlocking" << endl;
			throw logic_error("calling thread does not own mutex for unlocking");
		case EINVAL:
			cerr << "tried unlocking an uninitialised mutex" << endl;
			throw logic_error("tried unlocking an uninitialised mutex");
		}
//   cerr<<this<<" unlocked\n";cerr.flush();
	return retval;
	}

// returns true if you got the lock
bool thread_mutex::trylock(void) 
   { 
   return (pthread_mutex_trylock(&mutex)!=EBUSY);
   }

thread_mutex::~thread_mutex() 
   {
//   cerr<<this<<" destructed\n";cerr.flush();
	if (pthread_mutex_destroy(&mutex)==EBUSY) {
		cerr << "unlock the mutex before destroying it" << endl;
		throw logic_error("unlock the mutex before destroying it");
		}
   }
