// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Layout backend based on ncurses, interface.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef cursesboxes_hh__
#define cursesboxes_hh__

#include <cursesapp.h>
#include <cursesm.h>
#include <cursesf.h>

using namespace layoutables;

class cursbox;
class cursparbox;
class curshvbox;
class curscanvas;
class curscontainerbox;

UNDER DEVELOPMENT!!!

class curscanvas : public NCursesPanel {
	public:
		curscanvas(int, int);
	private:
};

class cursbox : virtual public abstract_box {
   public:
		cursbox(void);
		virtual cursbox          *outcast(int) const;
		virtual void              set_container(curscontainerbox *);
		virtual void              do_draw(int x, int y) {};   
		virtual NCursesFormField *widget(void)=0;  
	protected:
		curscontainerbox *container_;
};

class curscontainerbox : virtual public cursbox {
   public:
		virtual ~curscontainerbox() {};
		virtual void           add_child(cursbox *)=0; 
		virtual int            num_children(void) const;
		virtual curscanvas     *canvas(void)=0;
	protected:
		vector<cursbox *> children_;
};


// the various types of boxes are declared below.

class cursviewer : virtual public curscontainerbox, virtual public box, public NCursesPanel {
   public:
		cursviewer(void);
		virtual ~cursviewer();
		virtual stretch_number width(unit) const;
		virtual stretch_number height(unit) const;
		virtual stretch_number depth(void) const;
		virtual void freeze_width(super_number);
		virtual void freeze_height(super_number);
		virtual abstract_box *child(int) const;
		
		void show(void);
		virtual void add_child(curscontainerbox *);
		virtual void add_child(cursbox *) { cerr << "DO NOT USE THIS, IDIOT!" << endl; };
		virtual NCursesFormField *widget(void);
		virtual curscanvas       *canvas(void);
		gint                      relayout(void);
   private:
		curscanvas canvas_;
};

class curstextstyle {
	public:
		textstyle(void);
};

class curstext_box : virtual public cursbox, virtual public box {
   public:
		curstext_box(const string&,textstyle *);
		virtual stretch_number width(unit) const;
		virtual stretch_number height(unit) const;
		virtual stretch_number depth(void) const;
	   virtual void freeze_location(point);
		virtual void           do_draw(int, int);
		virtual Curs_Widget    *widget(void);
	private:
		textstyle *style_;
		string     text_;
};

class cursbuttonbox : virtual public cursbox, virtual public box {
   public:
		cursbuttonbox(const string&);
		virtual stretch_number width(unit) const;
		virtual stretch_number height(unit) const;
		virtual stretch_number depth(void) const;
		virtual Curs_Widget    *widget(void);
   private:
		Curs_Button *img_;
};

class curspreviewbox : virtual public cursbox, virtual public box {
   public:
		curspreviewbox(int, int);
		virtual stretch_number width(unit) const;
		virtual stretch_number height(unit) const;
		virtual stretch_number depth(void) const;
		virtual Curs_Widget    *widget(void);
   private:
		Curs_Preview *img_;
};

class curshvbox : virtual public curscontainerbox, virtual public box {
   public:
		curshvbox(box::direction);
		
		virtual void add_child(cursbox *);
		stretch_number width(unit) const;
		stretch_number height(unit) const;
		stretch_number depth(void) const;
		virtual abstract_box *child(int) const;
		virtual cursbox       *curs_child(int) const;
		virtual void          start_adoption(void);
		virtual void          adopt_children(int fromchild);
		virtual void          forget_adopted_children(void);

	   virtual void freeze_width(super_number);  
	   virtual void freeze_height(super_number);

		virtual void           do_draw(int, int);
		virtual Curs_Widget    *widget(void);
		virtual curscanvas     *canvas(void);
   protected:
		virtual gint     expose_event_handler_(GdkEventExpose *event);
		vector<cursbox *> children_backup_;
		int              adoption_index_;
//		curscanvas        canvas_;
};

class cursparbox : virtual public curscontainerbox, public parbox {
   public:
		cursparbox(parbox::direction, unit size);
		cursparbox(parbox::direction, float size_fraction);
		virtual void          add_child(cursbox *);
		virtual int           num_outcasts(void) const;
		virtual abstract_box *child(int) const;
		virtual cursbox       *outcast(int) const;
		virtual Curs_Widget    *widget(void);
		virtual curscanvas     *canvas(void);
   protected:
		virtual abstract_box* create_hvbox_(const vector<int>&);
		virtual void          take_back_hvboxes_(void);
		vector<curshvbox *>    pushed_out_;
};

class cursglue : virtual public cursbox, virtual public glue {
   public:
		cursglue(stretch_number w, stretch_number h);
		virtual Curs_Widget    *widget(void) { return 0; } ;
};

#endif

