// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  C++ classes to represent DOM level 1 HTML.

  Copyright 1999  Kasper Peeters.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef dom_hh_
#define dom_hh_

#include <string>
#include <vector.h>
#include <stdexcept>

#include <dom/dom.hh>
#include <dom/css.hh>

namespace DOM {

class HTMLDocument;
class HTMLElement;
class HTMLStyleElement;

class HTMLDocument : public Document {
	public:
		DOMString           title(void) const;
		DOMString           referrer(void) const;
		DOMString           domain(void) const;
		DOMString           URL(void) const;
		HTMLElement        *body(void) const;
		DOMString           cookie(void) const;

		// HTMLCollection is braindead, not implemented.
		// open/close/write do not belong here, not implemented.
};

class HTMLElement : public Element {
	public:
		DOMString            id(void);
		DOMString            title(void);
		DOMString            lang(void);
		DOMString            dir(void);
		DOMString            className(void);

		// DOM 2
		CSSStyleDeclaration *style(void);
};

class HTMLStyleElement : public HTMLElement {
	public:
		boolean     disabled(void) const;
		DOMString   media(void) const;
		DOMString   type(void) const;

		// DOM 2
		StyleSheet *styleSheet(void) const;
};

}; // namespace

#endif
