// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Test program to test lib-dom.

  Copyright 1999  Kasper Peeters.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <dom/dom.hh>
#include <dom/css.hh>

using namespace DOM;
using namespace CSS;

int main(int argc, char **argv)
	{
	/* Creation of a simple document,

        <html>
        <body>
        <h1>test</h1>
        <img src="picture.jpg">
		  This is a test file.
        </body>
        </html>

		We do this argv[1] times, deleting each Document after it has
      been created. If memory keeps growing, there's a leak.
	*/

	unsigned int maxloop=1;
	if(argc>1)
		maxloop=atoi(argv[1]);

	cout << "memory usage:" << endl
		  << "  sizeof(Node)       = " << sizeof(Node) << endl
		  << "  sizeof(Document)   = " << sizeof(Document) << endl << endl;
	

	cout << "testing DOM ..." << endl;

	for(unsigned int i=0; i<maxloop; ++i) {
		DocumentType *mydoctype = new DocumentType("HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\"");
		NamespaceDecl *defns    = new NamespaceDecl("html","http://www.w3.org/html");
		ptr<Document> mydoc=new Document("","text/html",mydoctype);
		mydoc->set_default_namespace(defns);
		
		Element *html=mydoc->createElement("HTML");
		Element *body=mydoc->createElement("BODY");
		Element *h1=mydoc->createElement("H1");
		Element *img=mydoc->createElement("IMG");
		Text    *txt1=mydoc->createTextNode("test");
		Text    *txt2=mydoc->createTextNode("This is a test file.");
		NamespaceDecl *dec=new NamespaceDecl("math","http://www.w3.org/Math");
		
		try {
			mydoc->appendChild(html);
			html->appendChild(body);
			body->appendChild(h1);
			h1->appendChild(txt1);
			mydoc->setNamespace(h1,dec);
			body->appendChild(img);
			img->setAttribute("src","\"picture.jpg\"");
			body->appendChild(txt2);
			}
		catch(DOMException& de) {
			cerr << "[DOMException]: " << (int)de.code << endl;
			} 
		if(i==maxloop-1) {
			Node::NodeIterator it(&*(mydoc),true);
			int level=0;
			while(it.thisnode()!=0) {
				for(int i=0; i<3*level; ++i)
					cout << " ";
				cout << it.thisnode()->nodeName() << endl;
				it.nextnode();
				level+=it.levels_changed();
				}

			cout << "testing CSS ..." << endl;

			for(unsigned int i=0; i<maxloop; ++i) {
				stylesheet mysheet(mydoc,0);
				
			   declaration    *decl1=new declaration;
			   selector       *sele1=new selector;
				
				try {
					sele1->elements_.push_back(selector::selector_element("H1"));
					decl1->set_border_spacing(
						new declaration::numberproperty(10,declaration::numberproperty::IN));
					decl1->set_color(
						new declaration::colorproperty("red"));
					decl1->set_display(declaration::block);
					rule *rule1=new rule(sele1,decl1);
					mysheet.add_rule(rule1);
					}
				catch(DOMException& de) {
					cerr << "[DOMException]: " << (int)de.code << endl;
					} 
				if(i==maxloop-1) {
					Node::NodeIterator it(&*(mydoc),true);
					int level=0;
					while(it.thisnode()!=0) {
						for(int i=0; i<3*level; ++i)
							cout << " ";
						cout << it.thisnode()->nodeName() << ":" << endl;
						for(int i=0; i<3*level; ++i)
							cout << " ";
						declaration::numberproperty *np=mysheet.style((*it))->get_border_spacing();
						if(np) {
							cout << np->value() << endl;
							for(int i=0; i<3*level; ++i)
								cout << " ";
							}
						cout << static_cast<int>(mysheet.style((*it))->get_display()) << " ";
						cout << mysheet.style((*it))->get_color() << " ";
						cout << (*it)->nodeNamespace()->uri() << " ";
						cout << endl;
						it.nextnode();
						level+=it.levels_changed();
						}
					} 
				}		
			} 
		}		

	}

