// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Canvas widgets based on Gtk::Layout widget.

  Copyright 1998-1999 Kasper Peeters.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef canvaswidgets_hh_
#define canvaswidgets_hh_

#include <string>

#include <gdk--/pixmap.h>
#include <gdk--/colormap.h>
#include <gdk--/gc.h>
#include <gdk--/window.h>

#include <gtk--/window.h>
#include <gtk--/scrollbar.h>
#include <gtk--/box.h>
#include <gtk--/button.h>
#include <gtk--/preview.h>
#include <gtk--/layout.h>
#include <gtk--/fixed.h>

// The gtkcanvas widget is a canvas widget on which arbitrary
// things can be drawn, while it can also contain arbitrary
// gtk widgets at specified (x,y) positions.

class gtkcanvas;
class gtkscrolledcanvas;

class gtkcanvas : public Gtk::Layout {
	public:
		gtkcanvas(Gtk::Adjustment& hadj, Gtk::Adjustment& vadj, gtkscrolledcanvas *owner);

		virtual void      realize_impl(void);
		virtual void      draw_impl(GdkRectangle *);
		virtual gint      expose_event_impl(GdkEventExpose *event);
		virtual void      size_allocate_impl(GtkAllocation *);
		void              flush(int x, int y, int w, int h);

		Gdk_Pixmap      & pixmap(void);
		Gdk_Colormap    & colormap(void);
		Gdk_Window      & window(void);
		Gdk_GC          & gc(void);

		void              renew_pixmap(int,int);

		void              relayout_all(void);

		class point {
			public:
				point(void);

				long x, y;
				bool operator==(const point& other) const;
		};
		class draw_info {
			public:
				point viewport_loc, viewport_size; // what's actually visible
				point bitmap_offset;  // bitmap origin does not have to coincide with viewport origin
				bool  move_too;

				bool operator==(const draw_info& other) const;
		};
	protected:
		virtual gint      button_press_event_impl(GdkEventButton *event);
		virtual gint      focus_in_event_impl(GdkEventFocus *);
		virtual gint      focus_out_event_impl(GdkEventFocus *);
		virtual gint      key_press_event_impl(GdkEventKey *);
		bool              has_focus_; // FIXME: gtk should keep track of this, right?
	private:
		Gdk_Pixmap         pixmap_, null_pixmap_;
		bool               canvas_frozen_;
		bool               initial_move_done_;
		Gdk_Window         window_;
		Gdk_GC             gc_;
		Gdk_Colormap       colormap_;
		long               previous_hadj, previous_vadj;
		gtkscrolledcanvas *owner_;
		Gtk::Button        dummy_;
		gtkcanvas::draw_info  last_draw_offscreen_data_;
		gtkcanvas::point      total_size;
};


// The gtkscrolledcanvas wraps a gtkcanvas. It adds vertical and
// horizontal scrollbars and takes care of resizing and moving the
// canvas when necessary.

class gtkscrolledcanvas : public Gtk::VBox {
   public:
		gtkscrolledcanvas(void);

		virtual void show_impl(void);
		virtual void hide_impl(void);
		virtual gint delete_event_impl(GdkEventAny *);

		gtkcanvas      *canvas(void);

		// Call this to force redraw of entire widget. Can call back draw_offscreen_impl, so
		// do not call it from there.
		void            redraw_all(void);
		void            redraw_rectangle(int32_t x, int32_t y, int32_t w, int32_t h);
		// On/off switching of scrollbars. Returns true if a change occurred.
		bool            scrollbar_vertical(bool);
		bool            scrollbar_horizontal(bool);

		// Override this to implement (re)drawing of parts of the canvas.
		virtual void    draw_offscreen_impl(gtkcanvas::draw_info& );
		// Override this to implement layout computations. Return the internal size.
		virtual gtkcanvas::point  layout_objects_impl(gtkcanvas::point& viewport_size);
	protected:
		virtual gint      focus_in_event_impl(GdkEventFocus *);
		virtual gint      key_press_event_impl(GdkEventKey *);
   private:
		bool             uninitialised_;
		Gtk::HBox        hbox_;
		bool             hs_on, vs_on;
		Gtk::Adjustment  vadj_, hadj_;
		gtkcanvas        canvas_;
		Gtk::VScrollbar  vscroll_;
		Gtk::HScrollbar  hscroll_;
};



#endif
