// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  GTK-- layout boxes, interface.

  Copyright 1998-1999 Kasper Peeters.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef gtkboxes_hh_
#define gtkboxes_hh_

#include <string>

#include "canvaswidgets.hh"
#include <layout/boxes.hh>
#include <layout/fonts.hh>
#include <layout/colors.hh>
#include <layout/factory.hh>
#include <rasterimage/raster.hh>
#include <mnemonic/ptr.hh>
#include <gtklayout/gtkcolors.hh>
#include <gtklayout/gtkfonts.hh>

// new classes defined here
class gtkfactory;

class viewercanvas;
class gtkviewer;
class gtktext_box;
class gtkbuttonbox;
class gtkimagebox;
class gtkglue;

class gtkfactory : public layout::factory {
	public:
		virtual layout::viewer_box *create_viewer_box(void);
		virtual layout::text_box   *create_text_box(const string& );
		virtual layout::glue       *create_glue(void);
		virtual layout::hbox       *create_hbox(void);
		virtual layout::vbox       *create_vbox(void); 
		virtual layout::border_box *create_border_box(void);
		virtual layout::event_box  *create_event_box(void);
		virtual layout::button_box *create_button_box(const string& );
};

class viewercanvas : public layout::area, public gtkscrolledcanvas {
	public:
		viewercanvas(gtkviewer *owner);
		
		// The two below will be called from the widget. If you want to 
		// force a relayout or redraw, call the relayout_all or redraw_all
		// members of gtkscrolledcanvas. They will call-back when needed.
		virtual void                draw_offscreen_impl(gtkcanvas::draw_info& );
		virtual gtkcanvas::point    layout_objects_impl(gtkcanvas::point& );
	protected:
		virtual gint motion_notify_event_impl(GdkEventMotion* p1);
		virtual gint button_press_event_impl(GdkEventButton *event);
		virtual gint button_release_event_impl(GdkEventButton *event);
	private:
		layout::box::event_info    inf_; // for button press/release 
		gtkviewer                 *owner_;
		layout::box::draw_info     previousinfo;
};

class gtkviewer : public layout::viewer_box {
   public:
		gtkviewer(void);

		virtual void reflow(void);
		virtual void redraw(layout::rectangle);

		virtual int                     draw(layout::point, const draw_info &, box *);
		virtual void                    freeze(int32_t, int32_t, box *); 

		virtual layout::font_manager&   font_mgr(void);
		virtual layout::color_manager&  color_mgr(void);
		virtual layout::area           *current_area(void) const;

		viewercanvas&                   scanvas(void) const;  // avoiding a cast
	private:
	   mutable viewercanvas   scanvas_;
		gtk_color_manager      color_mgr_;
		gtk_font_manager       font_mgr_;
};

class gtktext_box : public layout::text_box {
   public:
		gtktext_box(const string&);
		virtual int draw(layout::point, const draw_info &, box *);
};

class gtkbuttonbox : public layout::box {
   public:
		gtkbuttonbox(const string&);

		virtual layout::point frozen_size(void) const;
		virtual int           position(layout::point loc, const layout::box::draw_info&, layout::box *);

		virtual void          box_added_impl(void);
   private:
	   Gtk::Button img_;
};

class gtkhbox : public layout::hbox {
	protected:
		virtual void fill_background_impl(layout::point, const draw_info&);
};

class gtkvbox : public layout::vbox {
	protected:
		virtual void fill_background_impl(layout::point, const draw_info&);
};

class gtk_border_box : public layout::border_box {
	public:
		virtual int draw(layout::point, const draw_info&, box *);
};

class gtkeventbox : public layout::event_box {
	public:
		gtkeventbox(void);
		virtual ~gtkeventbox();

		virtual bool          accept_event(event_info&);
	private:
		GdkCursor    *hand_;
};

class gtkimagebox : public layout::container_box {
	public:
		gtkimagebox(int32_t, int32_t); // initial size to be used when there's no image data yet.
		virtual ~gtkimagebox();

		virtual int           draw(layout::point, const draw_info &, box *);
		void                  modify(ptr<scaled_raster_image>); // For updates. Can be called from any thread
		                                                 // even when gtk loop is still running.
   private:
		Gdk_Pixmap                 pixmap_;
		Gdk_Bitmap                 clipmap_;
		ptr<scaled_raster_image>   rim_;
		bool                       allocated_;
		int                        row_stride_; 
		bool                       modified_;
};

class gtkglue : public layout::glue {
   public:
		virtual int       draw(layout::point, const draw_info &, box *);
};

#endif
