// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Canvas widgets based on Gtk::Layout widget.

  Copyright 1998-1999 Kasper Peeters.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "gtklayout/canvaswidgets.hh"
#include <gdk/gdkkeysyms.h>
#include <gdk/gdkrgb.h>
#include <algo.h>
#include <mnemonic/stopwatch.hh>

#define DEBUG_CLASS "CANVASWIDGETS"
#include <mnemonic/debug.hh>

gtkcanvas::gtkcanvas(Gtk::Adjustment& hadj, Gtk::Adjustment& vadj, gtkscrolledcanvas *own) 
	: Gtk::Layout(hadj,vadj), null_pixmap_(0), initial_move_done_(false), dummy_("hi")
	{
	set_app_paintable(true);
	canvas_frozen_=false;
	owner_=own;
	previous_hadj=-1;
	previous_vadj=-1;
	colormap_=gdk_rgb_get_cmap();//get_default_colormap();

	// Gtk::Layout does not propagate the set_size signal if there is
	// no widget present. So we put a dummy on the canvas just to be sure.
// Seems to be fixed in recent gtk versions ;-)

//	put(dummy_,-100,-100);
//	dummy_.show();
//	move(dummy_,-110,-110);
//	set_events(GDK_EXPOSURE_MASK);

	Gtk::Layout::set_events(GDK_ALL_EVENTS_MASK);
	set_flags(GTK_CAN_FOCUS);
	has_focus_=false;
	}

Gdk_Pixmap & gtkcanvas::pixmap(void)
	{
	return pixmap_;
	}

Gdk_Colormap & gtkcanvas::colormap(void)
	{
	return colormap_;
	}

Gdk_Window & gtkcanvas::window(void) 
	{ 
	return window_; 
	}

Gdk_GC & gtkcanvas::gc(void)
	{
	return gc_;
	}

void gtkcanvas::realize_impl(void)
	{
	debug("gtkcanvas::realize");
	Gtk::Layout::realize_impl();
	renew_pixmap(width(),height());
	if(width()==1) {
		debug("something stupid happened, let's not re-layout");
		}
	else {
		gtkcanvas::point visible_size;
		visible_size.x=width();
		visible_size.y=height();
		total_size=owner_->layout_objects_impl(visible_size);
		debug("frozen size " << total_size.x << "," << total_size.y);
		if(total_size.y<height())  { // not enough content to fill visible area
			total_size.y=height();  
			owner_->scrollbar_vertical(false);
			}
		else owner_->scrollbar_vertical(true);
		debug("before " << height());
		if(total_size.x>visible_size.x) owner_->scrollbar_horizontal(true);
		else                            owner_->scrollbar_horizontal(false);
		debug("after " << height());
		set_size(total_size.x, total_size.y);
		}
	}

void gtkcanvas::size_allocate_impl(GtkAllocation *p)
	{
	debug("gtkcanvas::size_allocate_impl");

	// First, make sure all widgets on the Gtk::Layout widgets have 
	// fixed their size.
	Gtk::Layout::size_allocate_impl(p);

	gint wwidth=0, wheight=0;
	if(pixmap_.gdkobj()!=0) {
		gdk_window_get_size(pixmap_.gdkobj(), &wwidth, &wheight);
		debug("old pixmap_size is " << wwidth << "x" << wheight );
		debug("viewport height is " << height());
		}
	debug("requested size " << p->width << "x" << p->height );
	if(wwidth!=p->width || wheight!=p->height) {
		if(pixmap_.gdkobj()!=0) {
			debug("renewing pixmap");
			renew_pixmap(p->width, p->height);

			gtkcanvas::point visible_size;
			visible_size.x=p->width;
			visible_size.y=p->height;
			total_size=owner_->layout_objects_impl(visible_size);
			debug("frozen size " << total_size.x << "," << total_size.y);
			if(total_size.y<height())  { // not enough content to fill visible area
				total_size.y=height();  
				owner_->scrollbar_vertical(false);
				}
			else owner_->scrollbar_vertical(true);
			debug("before " << height());
			if(total_size.x>visible_size.x) owner_->scrollbar_horizontal(true);
			else                            owner_->scrollbar_horizontal(false);
			debug("after " << height());

			set_size(total_size.x, total_size.y);
			gfloat value=get_vadjustment()->get_value();
			debug("vadjustment value " << value);
			debug("visible.y         " << visible_size.y);
			if(value>total_size.y-visible_size.y) {
				debug("setting vadjustment to " << total_size.y-visible_size.y);
				get_vadjustment()->set_value(total_size.y-visible_size.y);
				}
			}
		}
	else {
		debug("no resize needed");
		}

	// Now do it again to propagate the size to the scrollbars.
	Gtk::Layout::size_allocate_impl(p);
	}

gtkcanvas::point::point(void)
	{
	x=0;
	y=0;
	}

bool gtkcanvas::point::operator==(const point& other) const
	{
	if(x==other.x && y==other.y) return true;
	else return false;
	}

bool gtkcanvas::draw_info::operator==(const draw_info& other) const
	{
	if(viewport_loc==other.viewport_loc &&
		viewport_size==other.viewport_size &&
		bitmap_offset==other.bitmap_offset) return true;
	else return false;
	}

// This is the analogue of realize_impl, called after a resize.
void gtkcanvas::draw_impl(GdkRectangle *p)
	{
	debug("gtkcanvas::draw_impl " << p->width << "x" << p->height);
	Gtk::Layout::draw_impl(p);

	gtkcanvas::draw_info inf;
	inf.viewport_loc.x=(long)(get_hadjustment()->get_value())+p->x;
	inf.viewport_loc.y=(long)(get_vadjustment()->get_value())+p->y;
	inf.bitmap_offset.x=p->x;
	inf.bitmap_offset.y=p->y;
	inf.viewport_size.x=p->width;
	inf.viewport_size.y=p->height;
	inf.move_too=true;

	if(!(last_draw_offscreen_data_==inf)) {
		// only draw on the canvas if we haven't drawn here before
		owner_->draw_offscreen_impl(inf);
		last_draw_offscreen_data_=inf;
		}
	else {
		debug("no re-draw necessary " << p->width << "," << p->height);
		}
	debug("flushing " << p->x << "," << p->y << " " << p->width << "," << p->height);
	flush(p->x,p->y,p->width,p->height);
	} 

gint gtkcanvas::button_press_event_impl(GdkEventButton *event)
	{
	grab_focus();
	return 0;
	}

gint gtkcanvas::focus_in_event_impl(GdkEventFocus *ev)
	{
	has_focus_=true;
	return 1;
//	return Gtk::Layout::focus_in_event_impl(ev);
	}

gint gtkcanvas::focus_out_event_impl(GdkEventFocus *ev)
	{
	has_focus_=false;
	return 1;
//	return Gtk::Layout::focus_out_event_impl(ev);
	}

gint gtkcanvas::key_press_event_impl(GdkEventKey *ev)
	{
	gfloat value=get_vadjustment()->get_value();
	gfloat hvalue=get_hadjustment()->get_value();
	debug(value << " " << total_size.y);
	gint width, height;
	gdk_window_get_size(pixmap_.gdkobj(), &width, &height);
	switch(ev->keyval) {
		case GDK_BackSpace:
			if(!has_focus_) {
				debug("no focus");
				return 0;
				}
		case GDK_Page_Up: {
			debug("Page_Up pressed");
			value=max(0,(gint)(value-height*0.9));
			get_vadjustment()->set_value(value);
			break;
			}
		case 32:
			if(!has_focus_) {
				debug("no focus");
				return 0;
				}
		case GDK_Page_Down: {
			debug("Page_Down pressed");
			value=min((gint)total_size.y-height,(gint)(value+height*0.9));
			get_vadjustment()->set_value(value);
			break;
			}
		case GDK_Up: {
			debug("Up pressed");
			value=max(0,(gint)(value-8));
			get_vadjustment()->set_value(value);
			break;
			}
		case GDK_Down: {
			debug("Down pressed");
			value=min((gint)total_size.y-height,(gint)(value+8));
			get_vadjustment()->set_value(value);
			break;
			}
		case GDK_Left: {
			if(!has_focus_) return 0;
			debug("Left pressed");
			hvalue=max(0,(gint)(hvalue-8));
			get_hadjustment()->set_value(hvalue);
			break;
			}
		case GDK_Right: {
			if(!has_focus_) return 0;
			debug("Right pressed");
			hvalue=min((gint)total_size.x-width,(gint)(hvalue+8));
			get_hadjustment()->set_value(hvalue);
			break;
			}
		default:
			debug("propagating keypress");
			return 0;
		}
	return 1; // Returning `1' means that we don't want the signal to propagate.
	          // If we return `0' the active widget will change too, which looks ugly.
	}

void gtkcanvas::flush(int x, int y, int w, int h)
	{
	window_.set_back_pixmap(pixmap_,false);
	window_.clear_area(x,y,w,h);
	window_.set_back_pixmap(null_pixmap_,false);
	}

void gtkcanvas::relayout_all(void)
	{
	debug("re-layout all "<<width()<<"x"<<height());
	if(width()==1 && height()==1) {
		debug("size is 1x1, let's not do that...");
		}
	else {
		gtkcanvas::point visible_size;
		visible_size.x=width();
		visible_size.y=height();
		total_size=owner_->layout_objects_impl(visible_size);
		debug("frozen size " << total_size.x << "," << total_size.y);
		if(total_size.y<height())  { // not enough content to fill visible area
			total_size.y=height();  
			owner_->scrollbar_vertical(false);
			}
		else owner_->scrollbar_vertical(true);
		if(total_size.x>visible_size.x) owner_->scrollbar_horizontal(true);
		else                            owner_->scrollbar_horizontal(false);
		
		set_size(total_size.x, total_size.y);
		}
	debug("re-layout all done");
	}

void gtkscrolledcanvas::redraw_all(void)
	{
	debug("redrawing all");

   redraw_rectangle(0,0,0,0);
	}

void gtkscrolledcanvas::redraw_rectangle(int32_t x, int32_t y, int32_t w, int32_t h)
	{
	long xoff=(long)(canvas_.get_hadjustment()->get_value());
	long yoff=(long)(canvas_.get_vadjustment()->get_value());

	if(w==0 && h==0) {
		x=xoff;
		y=yoff;
		w=canvas_.width();
		h=canvas_.height();
		}
	else {
		if(x<xoff) {
			w-=xoff-x;
			x=xoff;
			if(w<0) {
				debug("negative width");
				return;
				}
			}
		else if(x>canvas_.width()+xoff) {
			debug("to right of viewport");
			return;
			}

		if(y<yoff) {
			h-=yoff-y;
			y=yoff;
			if(h<0) {
				debug("negative height");
				return;
				}
			}
		else if(y>canvas_.height()+yoff) {
			debug("below bottom of viewport");
			return;
			}

		if(y-yoff+h>canvas()->height()) h=canvas()->height()-(y-yoff);
		if(x-xoff+h>canvas()->width())  w=canvas()->width()-(x-xoff);
		}
	debug("redraw rectangle " << x << "," << y << "-" << w << "," << h);

	gtkcanvas::draw_info inf;

	inf.viewport_loc.x=x;
	inf.viewport_loc.y=y;
	inf.bitmap_offset.x=x-xoff;
	inf.bitmap_offset.y=y-yoff;
	inf.viewport_size.x=w;
	inf.viewport_size.y=h;

	stopwatch sw;
	sw.start();

	draw_offscreen_impl(inf);

	debug("flushing all");
	canvas_.flush(inf.bitmap_offset.x,inf.bitmap_offset.y,w,h);
	sw.stop();
	debug("redraw_rectangle took " << sw);
	}

gint gtkcanvas::expose_event_impl(GdkEventExpose *e)
	{
	debug("gtkcanvas::expose_event_impl");
	
	gtkcanvas::draw_info inf;
	inf.viewport_loc.x=(long)(get_hadjustment()->get_value())+e->area.x;
	inf.viewport_loc.y=(long)(get_vadjustment()->get_value())+e->area.y;
	inf.bitmap_offset.x=e->area.x;
	inf.bitmap_offset.y=e->area.y;
	inf.viewport_size.x=e->area.width;
	inf.viewport_size.y=e->area.height;
	if(initial_move_done_==false) {
		inf.move_too=true;
		initial_move_done_=true;
		}
	else
		inf.move_too=false;
	
	// fill the background, just in case nothing gets drawn here
//	pixmap_.draw_rectangle(gc_,TRUE,e->area.x,e->area.y,e->area.width,e->area.height);
	
	owner_->draw_offscreen_impl(inf);
	
	stopwatch sw;
	sw.start();
	debug("flushing " << e->area.x << "," << e->area.y << " " << e->area.width << "," << e->area.height);
	flush(e->area.x,e->area.y,e->area.width,e->area.height);
	sw.stop();
	debug("flushing took " << sw);

	return Gtk::Layout::expose_event_impl(e);
	}

void gtkcanvas::renew_pixmap(int wnew, int hnew)
	{
	window_=gtkobj()->bin_window;
	gint x,y,w,h,depth;
	window_.get_geometry(x,y,w,h,depth); 
	if(pixmap_.connected()==false) {
		gc_=Gdk_GC(window_);
		}
	else {
		pixmap_.release();
		}

	pixmap_.create(window_,wnew,hnew,depth);
	debug("created new pixmap " << wnew << "," << hnew);
//	pixmap_.draw_rectangle(gc_,TRUE,0,0,wnew,hnew);
	}



gtkscrolledcanvas::gtkscrolledcanvas(void)
	: hs_on(false), vs_on(false), vadj_(0,0,20,8,8,5), hadj_(0,0,20,8,8,5), canvas_(hadj_,vadj_,this),
	  vscroll_(vadj_), hscroll_(hadj_)
	{
	uninitialised_=true;

//	Gtk::Window::add(&fixed_window_);
//	fixed_window_.put(vbox_,0,0);
//	fixed_window_.show();
	pack_start(hbox_,true);
//	hadj_=new Gtk::Adjustment(0,0,20,8,8,5);
//	vadj_=new Gtk::Adjustment(0,0,20,8,8,5);
//	canvas_=new gtkcanvas(*hadj_,*vadj_,this);
	hbox_.pack_start(canvas_,true);

//	vscroll_=new Gtk::VScrollbar(*(canvas_.get_vadjustment()));
//	hscroll_=new Gtk::HScrollbar(*(canvas_.get_hadjustment()));
	hbox_.pack_start(vscroll_,false);
	pack_start(hscroll_,false);

//	set_events(GDK_KEY_PRESS);
	}

void gtkscrolledcanvas::hide_impl(void)
	{
	vscroll_.hide();
	hscroll_.hide();
	hbox_.hide();
	canvas_.hide();
	debug("all gtkviewer's widgets are hidden");
	Gtk::VBox::hide_impl();
	}

void gtkscrolledcanvas::show_impl(void)
	{
	canvas_.show();
	if(vs_on) vscroll_.show_all();
	else      vscroll_.hide_all();
	if(hs_on) hscroll_.show_all();
	else      hscroll_.hide_all();
	hbox_.show();
	debug("all gtkviewer's widgets are showing");
	Gtk::VBox::show_impl();
	}

/*void gtkscrolledcanvas::show_all_impl(void)
	{
	Gtk::VBox::show_all_impl();
	canvas_.show();
	if(vs_on) vscroll_.show_all();
	else      vscroll_.hide_all();
	if(hs_on) hscroll_.show_all();
	else      hscroll_.hide_all();
	hbox_.show();
	debug("all gtkviewer's widgets are showing");
	}
*/

gint gtkscrolledcanvas::focus_in_event_impl(GdkEventFocus *ev)
	{
	return 0;
	}

gint gtkscrolledcanvas::key_press_event_impl(GdkEventKey *ev)
	{
	if(has_focus()) {
		}
	canvas_.key_press_event(ev);
	return true;
	}

gint gtkscrolledcanvas::delete_event_impl(GdkEventAny *)
	{
	return 0;
	}

gtkcanvas *gtkscrolledcanvas::canvas(void)
	{
	return &canvas_;
	}

bool gtkscrolledcanvas::scrollbar_vertical(bool onoff)
	{
	if(onoff!=vs_on) {
		vs_on=onoff;
		if(onoff) vscroll_.show_all();
		else      vscroll_.hide_all();
		return true;
		}
	return false;
	}

bool gtkscrolledcanvas::scrollbar_horizontal(bool onoff)
	{
	if(onoff!=hs_on) {
		hs_on=onoff;
		if(onoff) hscroll_.show_all();
		else      hscroll_.hide_all();
		return true;
		}
	return false;
	}

void gtkscrolledcanvas::draw_offscreen_impl(gtkcanvas::draw_info&)
	{
	}

gtkcanvas::point gtkscrolledcanvas::layout_objects_impl(gtkcanvas::point&)
	{
	gtkcanvas::point tmp;
	tmp.x=0;
	tmp.y=0;
	return tmp;
	}

