// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  GTK color code.

  Copyright 1998-1999 Kasper Peeters.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <string>
#include <layout/colors.hh>
#include "gtklayout/gtkboxes.hh"
#include "gtklayout/gtkcolors.hh"

#define DEBUG_CLASS "GTKCOLORS"
#include <mnemonic/debug.hh>

using namespace layout;

void gtk_color_manager::activate_foreground(layout::color *cb)
	{
	if(active_fg_color_!=cb) {
		active_fg_color_=cb;
		gtk_colorspec *gtkc=static_cast<gtk_colorspec *>(cb);
		assert(gtkc!=0);
		viewercanvas *vc=static_cast<viewercanvas *>(owner_->current_area());
		vc->canvas()->gc().set_foreground(*(gtkc->color_inf()));
		}
	}

void gtk_color_manager::activate_background(layout::color *cb)
	{
	if(active_bg_color_!=cb) {
		active_bg_color_=cb;
		gtk_colorspec *gtkc=static_cast<gtk_colorspec *>(cb);
		assert(gtkc!=0);
		viewercanvas *vc=static_cast<viewercanvas *>(owner_->current_area());
		vc->canvas()->gc().set_background(*(gtkc->color_inf()));
		}
	}

Gdk_Color *gtk_colorspec::color_inf(void) const
	{
	return fg_;
	}

gtk_colorspec::gtk_colorspec(const string& colorname, Gdk_Colormap& cm)
	: layout::color(colorname)
	{
	fg_=new Gdk_Color(colorname_);
	if(!(fg_->parse(colorname_))) {
		if(colorname_!="" && colorname_[0]!='#') {
			delete fg_;
			fg_=new Gdk_Color(string("#")+colorname_);
			}
		}
	cm.alloc(*fg_);
	}

gtk_color_manager::gtk_color_manager(layout::box *owner)
	: owner_(owner)
	{
	debug("gtk_color_manager constructed");
	}

layout::color *gtk_color_manager::create_color_impl(const string& name)
	{
	assert(owner_!=0);
	viewercanvas *vc=static_cast<viewercanvas *>(owner_->current_area());
	gdk_threads_enter();
	layout::color *tmp=new gtk_colorspec(name, vc->canvas()->colormap());
	gdk_threads_leave();
	return tmp;
	}


