// -*- mode: C++; c-file-style: "mnemonic"; -*-

#include "gtklayout/gtkboxes.hh"

#include <gtk--/main.h>
#include <fstream.h>
#include <stdexcept>
#include <unistd.h>
#include <gdk/gdkrgb.h>

#define DEBUG_CLASS "SHOWOFF"
#include <mnemonic/debug.hh>
#include <mnemonic/ptr.hh>

class Window2 : public Gtk::Window {
	public:
		Window2(bool wid, bool charm) 
			: Gtk::Window(), draw_widgets(wid), character_mode(charm),
			  button_("test for memory leaks (memory use should not increase)"),
			  reflow_("reflow") { 	
		set_events(GDK_ALL_EVENTS_MASK);
		};
		virtual ~Window2() { debug("~Window2"); };

		virtual gint delete_event_impl(GdkEventAny *) { 
/*		debug("closing window");
		Gtk::Window::iterator it=begin();
		while(it!=end()) {
			debug("removing child of window");
			(*it)->hide_all();
			remove(*it);
			++it;
			} */
		Gtk::Main::quit();
		return 0; 
		}
		void attach_viewer(void);
		void attach_button(void);
	protected:
		void realize_impl(void);
	private:
		bool draw_widgets, character_mode;
		Gtk::VBox vbox_;
		Gtk::Button button_;
		Gtk::Button reflow_;
		gtkviewer *mycanvas;
		void button_clicked_(void);
		void reflow_clicked_(void);
		gtkvbox  *tmpbig2;
};

void Window2::realize_impl(void)
	{
	Gtk::Window::realize_impl();
	}

void Window2::button_clicked_(void)
	{
	debug("button was clicked");
	mycanvas->scanvas().hide_all();
	vbox_.remove(mycanvas->scanvas());
	delete mycanvas;
	attach_viewer();
	vbox_.show_all();
	}

void Window2::reflow_clicked_(void)
	{
	debug("=> reflowing");
	mycanvas->reflow();
	debug("=> redrawing");
	mycanvas->redraw(layout::rectangle(layout::point(0,0), layout::point(0,0)));
	debug("redrawing done");
	}

void Window2::attach_button(void)
	{
	add(vbox_);
//	vbox_.set_events(GDK_ALL_EVENTS_MASK);
	vbox_.pack_start(button_,false);
	vbox_.pack_start(reflow_,false);
	button_.show();
	reflow_.show();
	vbox_.show();
	button_.clicked.connect(SigC::slot(this,&Window2::button_clicked_));
	reflow_.clicked.connect(SigC::slot(this,&Window2::reflow_clicked_));
	}

void Window2::attach_viewer(void)
	{
	mycanvas=new gtkviewer;
	debug("new viewer created");
	mycanvas->set_properties(new layout::property_box(0, // no inheritance, this is the default.
																	  mycanvas->font_mgr().get_font(
																		  layout::xlfd("lucida",
																							layout::xlfd::medium,
																							layout::xlfd::upright, 
																							layout::length("12px"))),
																	  mycanvas->color_mgr().get_color("black"),
																	  mycanvas->color_mgr().get_color("#c0c0c0")));
	vbox_.pack_start(mycanvas->scanvas(),true);
	debug("added to canvas");
	set_default_size(915,300); // visual size (plus scrollbars)

	debug(   "pointer       : " << sizeof(layout::box *) << endl
			<< "ptr           : " << sizeof(ptr<layout::box *>) << endl
			<< "box           : " << sizeof(layout::box) << endl
  			<< "resizable_box : " << sizeof(layout::resizable_box) << endl
			<< "container_box : " << sizeof(layout::container_box) << endl
			<< "hbox          : " << sizeof(layout::hbox) << endl
			<< "vbox          : " << sizeof(layout::vbox) << endl
			<< "parbox        : " << sizeof(layout::parbox) << endl
			<< "glue          : " << sizeof(layout::glue) << endl
			<< "string        : " << sizeof(string) << endl
			<< "gtktext_box   : " << sizeof(gtktext_box) << endl
			<< "table         : " << sizeof(layout::table) << endl
			<< "gtkhbox       : " << sizeof(gtkhbox) << endl);

	gtkhbox  *tmppage=new gtkhbox;
	tmppage->set_fill_background(true);
	mycanvas->add_child(tmppage);
	tmppage->set_properties(
		new layout::property_box(
			mycanvas->properties(),
			mycanvas->font_mgr().get_font(layout::xlfd("lucida",
																	  layout::xlfd::medium,
																	  layout::xlfd::upright, 
																	  layout::length("14px"))),
			mycanvas->color_mgr().get_color("blue"),
			mycanvas->color_mgr().get_color("yellow"))
		);
		
	// Make two columns of 0.7 and 0.3 times the full width.
	gtkvbox  *tmpbig1=new gtkvbox;
	tmppage->add_child(tmpbig1);
	tmpbig1->set_size_fraction((float)0.7,0);
	gtkvbox  *tmpbig2=new gtkvbox;
	tmppage->add_child(tmpbig2);
	tmpbig2->set_size_fraction((float)0.3,0);

	// Add a few items to the first column.
	tmpbig1->add_child(new gtktext_box("kasper"));
	gtk_border_box *bb=new gtk_border_box;
	tmpbig1->add_child(bb);
	bb->set_size_mixed(1.0,2);

	// Also add a table to the first column.
	layout::table *tt=new layout::table;
	tmpbig1->add_child(tt);
	tt->add_child(new gtktext_box("one"),1,1);
//	tt->add_child(new gtktext_box("two"),1,1);
	gtkvbox *vbb=new gtkvbox;
	layout::parbox *pb=new layout::parbox;
	tt->add_child(vbb,1,1);
	vbb->add_child(pb);
	pb->add_child(new gtktext_box("extremelylongwords"));
	pb->add_child(new layout::glue);
	pb->add_child(new gtktext_box("whichdonotfitonaline"));
	pb->add_child(new layout::glue);
	pb->add_child(new gtktext_box("canneverthelessbeaddedtoparboxes"));
	pb->add_child(new layout::glue);
	pb->add_child(new gtktext_box("insidetables"));
	pb->add_child(new layout::glue);
	pb->add_child(new gtktext_box("regardlessofallsortsofotherproblems"));
	pb->add_child(new layout::glue);
	pb->add_child(new gtktext_box("becauseitsimplyworksthatswhy!"));
	tt->end_row();
	tt->add_child(new gtktext_box("three"),1,1);
	tt->add_child(new gtktext_box("four"),1,1);

	// Some stuff after the table.
	if(draw_widgets) {
		gtkbuttonbox *tmppar3d=new gtkbuttonbox("testbutton");
		tmpbig1->add_child(tmppar3d); 
		}
	gtktext_box *japbox=new gtktext_box("\044\063");
	tmpbig1->add_child(japbox);
	japbox->set_properties(new layout::property_box(tmpbig1->properties(),
																	tmpbig1->font_mgr().
																	get_font(layout::xlfd("jis","*",
																								 layout::xlfd::medium,
																								 layout::xlfd::upright, 
																								 layout::length("24px"),
																	  "*")),
																	0,0));


/*
  Encoding and font mapping is done by hand here:

  UTF   E3 81 93   (some funky symbol)
  UCS2  30 53
  JISX  24 33    = 36 51 = \044\063  (from the jis.map table)
 */

	// Add paragraph boxes to both columns.
	layout::parbox *tmppar1=new layout::parbox;
 	tmpbig1->add_child(tmppar1);
	layout::parbox *tmppar4=new layout::parbox;
	tmpbig2->add_child(tmppar4);
	tmppar4->set_properties(new layout::property_box(tmppar4->properties(),
																	 tmppar4->font_mgr().
																	 get_font(layout::xlfd("lucida",
																								  layout::xlfd::medium,
																								  layout::xlfd::upright, 
																								  layout::length("14px"))),
																	 tmppar4->color_mgr().get_color("green"),0));
									
	// Another button box.
//	gtkbuttonbox *tmppar3e=new gtkbuttonbox("Inline");
//	tmppar1->add_child(tmppar3e); 
	
	debug("reading showoff.txt");
	ifstream str("showoff.txt");
	string tmp;
	long num_boxes=0;
	long num_characters=0;
	string collect;
	int colnum=0;
	while(str >> tmp) { 
		if(tmp=="<pic>") {
			if(draw_widgets) {
				gtkimagebox *tmppic=new gtkimagebox(50,50);
				tmppar1->add_child(tmppic); 
				tmppic=new gtkimagebox(60,30);
				tmppar4->add_child(tmppic);
				num_boxes+=2;
				}
			}
		else if(tmp=="<but>") {
			if(draw_widgets) {
				gtkbuttonbox *tmpbut=new gtkbuttonbox("click me\nplease");
				tmppar1->add_child(tmpbut);
				tmpbut=new gtkbuttonbox("click me\nplease");
				tmppar4->add_child(tmpbut);
				num_boxes+=2;
				}
			}
		else if(tmp=="<flow>") {
			if(draw_widgets) {
				gtkbuttonbox *tmpbut=new gtkbuttonbox("flow around\nthis button");
				tmppar1->add_child(tmpbut);
				tmpbut=new gtkbuttonbox("flow around\nthis button");
				tmppar4->add_child(tmpbut);
				num_boxes+=2;
				}
			}
		else {
			if(character_mode) {
				for(unsigned int i=0; i<tmp.size(); ++i) {
					string fik=tmp.substr(i,1);
					gtktext_box *tmpbox2=new gtktext_box(fik);
					tmppar1->add_child(tmpbox2);
					tmpbox2=new gtktext_box(fik);
					tmppar4->add_child(tmpbox2);
					num_boxes+=2;
					}
				}
			else {
				colnum++;
				collect+=tmp;
				// change number to see speedup
 				if(colnum==1) {
 					gtktext_box *tmpbox2=new gtktext_box(collect);
 					tmppar1->add_child(tmpbox2);
   				tmpbox2=new gtktext_box(collect);
					tmppar4->add_child(tmpbox2);
 					num_boxes+=2;
 					colnum=0;
 					collect="";
 					}
 				else {
 					collect+=" ";
 					}
 				}
 			num_characters+=2*tmp.size();
 			}
 		gtkglue *tmpglue=new gtkglue;
 		tmppar1->add_child(tmpglue);
		tmpglue=new gtkglue;
	 	tmppar4->add_child(tmpglue);
	  	num_boxes+=2;
	 	}
	debug("file filled up " << num_boxes << " boxes" << endl
	 		<< "   among which " << num_characters << " characters");

//	mycanvas->set_absolute_size(900,400,0);
	print_layout_tree(mycanvas);
	}

int main(int argc, char **argv)
   {
	bool character_mode=false;
	bool draw_widgets=true;
	unsigned long iterations=0;
	int ac=1;

	g_thread_init(NULL);

	while (ac < argc && argv[ac][0] == '-') {
			switch (argv[ac][1]) {
				case 'h':
					cerr << "Usage: showoff [-p|-c|-w|-dDEBUGCLASS] [iterations]" << endl;
					exit(1);
				case 'c':
					character_mode=true;
					break;
				case 'w':
					draw_widgets=false;
					break;
#ifdef DEBUG
				case 'd':
					debugger::instance().turn_on(argv[ac]+2);
					break;
#endif
				}
			++ac;
		}

	iterations=atol(argv[argc-1]);
	iterations++;

	if(character_mode)
		cerr << "storing each individual character in a box" << endl;
	else
		cerr << "storing entire words in boxes" << endl;
	if(draw_widgets)
		cerr << "drawing widgets too" << endl;
	else
		cerr << "widgets turned off" << endl;

	cerr << endl;

   Gtk::Main mymain(argc,argv);

	gdk_rgb_set_install(false);
	gdk_rgb_set_min_colors(2*2*2);
	gdk_rgb_init();
	gtk_widget_push_visual(gdk_rgb_get_visual());
	gtk_widget_push_colormap(gdk_rgb_get_cmap());

/* Schematically:

*/

	Window2 *mywin=new Window2(draw_widgets, character_mode);

	try { 
		debug("attaching button");
		mywin->attach_button();
		debug("attaching viewer");
		mywin->attach_viewer();
		debug("showing");
		mywin->show_all();
		debug("running");
		gdk_threads_enter();
		mymain.run();
		gdk_threads_leave();
		}
	catch(exception& err) {
		cerr << "[exception]: " << err.what() << endl;
		}
	debug("main.run() ended, deleting gtkviewer");
	debug("going to delete mywin");
	debug("end of loop");
	delete mywin;
   }
