// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Streamable raster images in GIF format.

  Copyright 1998-1999 Kasper Peeters.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef gifraster_hh__
#define gifraster_hh__

// We have to include this first, BEFORE anyone gets to set the signal
// handler behaviour to follow sysv (leading to EINTR all over the place).
#include <signal.h>

#include <rasterimage/raster.hh>
extern "C" {
#include <gif_lib.h>
}
#include <mnemonic/thread.hh>

class gif_decoder : public decoder {
	public:
		gif_decoder(void);
		virtual ~gif_decoder();

		virtual string        identifier(void) const;
		virtual bool          still_active(void) const;
	   virtual void          write(const unsigned char *, long length);
		virtual void          data_end(void);
	private:
		class gif_thread : public thread {
			public:
				gif_thread(gif_decoder *);
				virtual ~gif_thread();

				virtual void begin(void);
				bool    is_running(void) const;
				bool    error_encountered(void) const;
				bool    finished_decoding(void) const;
				int     writepipe(void) const;

				thread_cond finished_cond;
				thread_cond start_cond;
			private:
				bool         error_encountered_; // which implies datapipe_[0] is closed
				bool         finished_decoding_;
				bool         is_running_;
				int          datapipe_[2];
				bool         writepipe_open_, readpipe_open_;
				GifFileType *gif_ft_;
				gif_decoder *dec_;
				GifRowType   linebuffer_;
		};

		friend gif_thread;

		gif_thread *gif_thread_;
};

#endif


