// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Streamable raster images in JPG format.

  Copyright 1998-1999 Kasper Peeters.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef jpgraster_hh__
#define jpgraster_hh__

#include <rasterimage/raster.hh>
extern "C" {
#include <stdio.h>
#include <setjmp.h>
#include <jpeglib.h>
}

class jpg_decoder;

// extended versions of the _mgr structs in jpeglib.
struct jpg_error_obj {
		struct jpeg_error_mgr   pub;
		jpg_decoder            *dec;
		jmp_buf                 jmpb;
};
struct jpg_source_obj {
		struct jpeg_source_mgr  pub;
		jpg_decoder            *dec;
		unsigned char           buffer[32768L];
		long                    ignore_writes;
		long                    keep;
};

extern "C" {
 void jpeg_error_exit_(j_common_ptr cinfo);
 void my_output_message_(j_common_ptr cinfo);

 void    init_source_(j_decompress_ptr cinfo);
 boolean fill_input_buffer_(j_decompress_ptr cinfo);
 void    skip_input_data_(j_decompress_ptr cinfo, long num_bytes);
 void    term_source_(j_decompress_ptr cinfo);
};


class jpg_decoder : public decoder {
	public:
		jpg_decoder(void);
		virtual ~jpg_decoder();

		virtual string        identifier(void) const;
	   virtual void          write(const unsigned char *, long length);
		virtual void          data_end(void);
	private:
		bool        						 error_encountered_;
		int         						 row_stride_;
		JSAMPARRAY  						 buffer_;
		struct jpeg_decompress_struct  cinfo_;
		struct jpg_error_obj           jerr_;
		struct jpg_source_obj          jsource_;

		enum statuscode { read_header, start_decompress, read_scanlines, 
								finish_decompress, all_done };
		statuscode status_;
};

#endif


