// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Streamable raster images in PNG format.

  Copyright 1998-1999 Kasper Peeters.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef pngraster_hh__
#define pngraster_hh__

#include <rasterimage/raster.hh>
#include <png.h>
#include <mnemonic/thread.hh>

class png_decoder : public decoder {
	public:
		png_decoder(void);
		virtual ~png_decoder();

		virtual string         identifier(void) const;
		virtual void           write(const unsigned char *, long length);
		virtual bool           finished(void) const;
		virtual void           data_end(void);

		void row_read(png_structp png_ptr, png_bytep new_row, png_uint_32 row_num, int pass);
		void info(png_structp png_ptr, png_infop info);

		static void info_callback(png_structp png_ptr, png_infop info);
		static void row_callback(png_structp png_ptr, png_bytep new_row, png_uint_32 row_num, int pass);
		static void end_callback(png_structp png_ptr, png_infop info);
	private:
		png_structp png_ptr_;
		png_infop   info_ptr_, end_info_;
		enum png_state {png_start, png_valid, png_reading};
		png_state   state_;
		png_bytep  *crows_;
		png_bytep  *arows_;
		png_bytep  row_;
		int	      number_of_passes_;
};

#endif
