// -*- mode: C++; c-file-style: "mnemonic"; -*-
#include "config.h"
#ifdef HAVE_LIBZ
#ifdef HAVE_LIBPNG
#include <rasterimage/pngraster.hh>
#include <unistd.h>
#include <png.h>
#include <algo.h>
#include <stdexcept>

#define DEBUG_CLASS "PNG_DECODER"
#include <mnemonic/debug.hh>

png_decoder::png_decoder(void) : state_(png_start),crows_(NULL),arows_(NULL),row_(NULL)
	{
	debug("Initializing png_decoder");
	}

png_decoder::~png_decoder()
	{
	debug("destroying png_decoder");
	if(crows_) delete [] crows_;
	if(arows_) delete [] arows_;
	if(row_) delete row_;
	}

string png_decoder::identifier(void) const 
	{
	return "png";
	}

bool png_decoder::finished(void) const 
	{
	if(still_active_)
		return true;
	return false;
	}

void png_decoder::write(const unsigned char *dat, long siz) 
	{
	debug("png got a write!");
	if(!still_active_)
		return;
	debug("png switching state");
	switch(state_) {
		case png_start:
			if(png_sig_cmp(const_cast<unsigned char *>(dat), 0, min(siz,8L))!=0) {
				debug("sorry, not a png stream");
				still_active_=false;
				return;
				}
			state_ = png_valid;
		case png_valid:
			png_ptr_ = png_create_read_struct(PNG_LIBPNG_VER_STRING, 0, 0, 0);
			if (!png_ptr_)
				throw logic_error("png_create_read_struct failed");
			
			info_ptr_ = png_create_info_struct(png_ptr_);
			if (!info_ptr_) {
				png_destroy_read_struct(&png_ptr_, (png_infopp)NULL, (png_infopp)NULL);
				throw logic_error("png_create_info_struct failed");
				}
			end_info_ = png_create_info_struct(png_ptr_);
			if (!end_info_) {
				png_destroy_read_struct(&png_ptr_, &info_ptr_, (png_infopp)NULL);
				throw logic_error("png_create_info_struct two failed");
				}
			debug("initialising png io");
			if (setjmp(png_ptr_->jmpbuf)) {
				debug("Got an error from png library");
				still_active_=false;
				png_destroy_read_struct(&png_ptr_, &info_ptr_, &end_info_);
// FIXME: we don't handle this properly, so let's ignore it for the moment
//				throw logic_error("png library error");
				return;
				}
			png_set_progressive_read_fn(png_ptr_, (void *)this, &png_decoder::info_callback, &png_decoder::row_callback, &png_decoder::end_callback);
			state_ = png_reading;
		case png_reading:
			if (setjmp(png_ptr_->jmpbuf)) {
				debug("got an error from png library two");
				still_active_=false;
				png_destroy_read_struct(&png_ptr_, &info_ptr_, &end_info_);
// FIXME: we don't handle this properly, so let's ignore it for the moment
//				throw logic_error("got an error from png library two");
				return;
				}
			png_process_data(png_ptr_, info_ptr_, const_cast<unsigned char *>(dat), siz);
			break;
		}
	}

void png_decoder::data_end() 
	{
	debug("png data_end");
	}

void png_decoder::info(png_structp png_ptr, png_infop info_ptr) 
   {
	int bit_depth = png_get_bit_depth(png_ptr, info_ptr);
	int color_type = png_get_color_type(png_ptr, info_ptr);

	if (color_type == PNG_COLOR_TYPE_PALETTE && bit_depth <= 8)
		png_set_expand(png_ptr);
	if (color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8)
		png_set_expand(png_ptr);
	if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)) 
		png_set_expand(png_ptr);

	if (bit_depth == 16)
		png_set_strip_16(png_ptr); // convert 16 bit colors to 8 bit

// Keep alpha info:
//	if (color_type & PNG_COLOR_MASK_ALPHA)
//		png_set_strip_alpha(png_ptr);

	if (bit_depth < 8)
		png_set_packing(png_ptr); // expand values to 8 bit

	// Add alpha channel if it doesn't already exist
	png_set_filler(png_ptr, 255, PNG_FILLER_AFTER);

	if (color_type == PNG_COLOR_TYPE_GRAY ||
		 color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
		png_set_gray_to_rgb(png_ptr); // convert grayscale to rgb
	
	if (png_get_interlace_type(png_ptr, info_ptr) != PNG_INTERLACE_NONE) {
		number_of_passes_ = png_set_interlace_handling(png_ptr);
		}
	else number_of_passes_ = 1;
	png_read_update_info(png_ptr, info_ptr);
	png_uint_32 width = png_get_image_width(png_ptr,info_ptr);
	png_uint_32 height = png_get_image_height(png_ptr,info_ptr);
	image_= new scaled_raster_image(width, height);
	image_->setAlpha();
	arows_ = new png_bytep[height];
	crows_ = new png_bytep[height];
        row_ = new png_byte[width*4]; //RGBA space
	for(unsigned int row=0;row<height;row++) {
		crows_[row] = &(image_->image_data())[row*width*3];
		arows_[row] = &(image_->image_alpha())[row*width];
		}
	}

void png_decoder::info_callback(png_structp png_ptr, png_infop info) 
	{
	png_decoder *me = static_cast<png_decoder *>(png_get_progressive_ptr(png_ptr));
	me->info(png_ptr,info);
	}

void png_decoder::end_callback(png_structp png_ptr, png_infop info) 
	{
	png_decoder *me = static_cast<png_decoder *>(png_get_progressive_ptr(png_ptr));
	me->data_end();
	}

void png_decoder::row_callback(png_structp png_ptr, png_bytep new_row, png_uint_32 row_num, int pass) 
	{
	png_decoder *me = static_cast<png_decoder *>(png_get_progressive_ptr(png_ptr));
	me->row_read(png_ptr,new_row,row_num,pass);
	}

void png_decoder::row_read(png_structp png_ptr, png_bytep new_row, png_uint_32 row_num, int pass) 
	{
	debug("On pass "<<pass<<" of " << number_of_passes_ << " row number "<<row_num);
//	if(pass+1 == number_of_passes_)
//		image_->last_finished_scanline = row_num;
	png_progressive_combine_row(png_ptr, row_, new_row);
        for (unsigned int k=0;k<png_get_image_width(png_ptr_,info_ptr_);k++) {
          (crows_[row_num])[k*3  ]=row_[k*4  ];
          (crows_[row_num])[k*3+1]=row_[k*4+1];
          (crows_[row_num])[k*3+2]=row_[k*4+2];
          (arows_[row_num])[k    ]=row_[k*4+3];
	}
	image_->updated(0,row_num,png_get_image_width(png_ptr_,info_ptr_)-1,row_num);
	}
#endif
#endif
