
#include "config.h"
#include <rasterimage/raster.hh>
#ifdef HAVE_LIBZ
#ifdef HAVE_LIBPNG
#include <rasterimage/pngraster.hh>
#define HAVEPNGZ 1
#endif
#endif
#ifdef HAVE_LIBUNGIF
#include <rasterimage/gifraster.hh>
#endif
#ifdef HAVE_LIBJPEG
#include <rasterimage/jpgraster.hh>
#endif
#define DEBUG_CLASS "RASTERIMAGE"
#include <mnemonic/debug.hh>
/*
hasAlpha denotes if there is alpha information
if there is no alpha information the only interesting information 
is a map of 1s in the scaledalpha if one wants to know also can this 
be ignored when you request the availability of the alpha info

*/


decoder::decoder(void)
	: still_active_(true), finished_(false), image_(0)
	{
	    
	}

bool decoder::still_active(void) const
	{
	return still_active_;
	}

ptr<scaled_raster_image> decoder::image_data(void) const
	{
	return image_;
	}

decoder::~decoder()
	{
	debug("destroying decoder (and image if we're the only users)");
	}

raster_image::raster_image(long w, long h)
	: 
//	    last_finished_scanline(0), 
	    hasAlpha_(false),
	    updated_(false),
	    width_(w), 
	    height_(h)
	{
	debug("raster_image "<<w<<"x"<<h);
	frames=1; 
nod();
	raster_.push_back(new uint8_t[w*h*3]);
nod();
	debug("raster_ "<<(void*)raster_[0]);
	for(long i=0; i<w*h; ++i) {
		for(long j=0; j<3; ++j) {
			(raster_[0])[i*3+j]=0; // (i+j*30)%256
			}
		}
	debug("done");
	}

	
raster_image::~raster_image()
	{
	while (!raster_.empty())        {delete [] raster_.back();        raster_.pop_back();}
	while (!alpha_.empty())         {delete [] alpha_.back();         alpha_.pop_back();}

	}




void raster_image::updated(long,long,long,long)
	{
	//later someone may use the params to do something smart
	//for now we just notice it and only set a flag
	debug("updated");
	updated_=true;
	}

void raster_image::setAlpha(){
	hasAlpha_=true;
	for (int j=0;j<frames;j++){
  	  alpha_.push_back(new uint8_t[width_*height_]);
	  int tval=((j==frames-1)?0:255);
          for (long i=0;i<width_*height_;i++) (alpha_[j])[i]=tval;
	}
}
void raster_image::addFrame(){
        frames++;
	
	raster_.push_back(new uint8_t[width_*height_*3]);
	debug("raster_ "<<(void*)raster_.back());
	for(long i=0; i<width_*height_; ++i) {
		for(long j=0; j<3; ++j) {
			(raster_.back())[i*3+j]=0; // (i+j*30)%256
			}
		}
	if (hasAlpha_){	
  	  alpha_.push_back(new uint8_t[width_*height_]);
          for (long i=0;i<width_*height_;i++) (alpha_.back())[i]=0;
	}
}

raster_io::raster_io(void)
	: initialised_(false)
	{
	}

void raster_io::use_decoder(const string& mime)
	{
	if(!initialised_) {
		initialised_=true;
#ifdef HAVEPNGZ
		if(mime.find("image/png")!=mime.npos) {
			decoders_.push_back(new png_decoder);
			return;
			} 
#endif
#ifdef HAVE_LIBJPEG		  
		if(mime.find("image/jpg")!=mime.npos || mime.find("image/jpeg")!=mime.npos) {
			decoders_.push_back(new jpg_decoder);
			return;
			}
#endif
#ifdef HAVE_LIBUNGIF		
		if(mime.find("image/gif")!=mime.npos) {
			decoders_.push_back(new gif_decoder);
			return;
			}
#endif
		}
	}

void raster_io::write(const unsigned char *dat, long siz)
	{
	if(!initialised_) {
		initialised_=true;
#ifdef HAVEPNGZ
		decoders_.push_back(new png_decoder);
#endif
#ifdef HAVE_LIBUNGIF		  
		decoders_.push_back(new gif_decoder);
#endif
#ifdef HAVE_LIBJPEG		
		decoders_.push_back(new jpg_decoder);
#endif
		debug("decoders installed");
		}
	if(decoders_.size()==0) return;

	vector<decoder *>::iterator it=decoders_.begin();
	bool anyone=false;
	while(it!=decoders_.end()) {
		if((*it)->still_active()) {
			(*it)->write(dat,siz);
			if((*it)->still_active()) {
				debug("yes!");
				anyone=true;
				}
			}
		++it;
		}
	if(!anyone) {
		debug("all decoders have given up!");
		}
	}

long raster_io::read(unsigned char *, long)
	{
	return 0;
	}

raster_io::~raster_io()
	{
	debug("destroying raster_io");
	vector<decoder *>::iterator it=decoders_.begin();
	while(it!=decoders_.end()) {
		delete (*it);
		++it;
		}
	debug("destroying raster_io done");
	}

ptr<scaled_raster_image> raster_io::image(void) const
	{
	vector<decoder *>::const_iterator it=decoders_.begin();
	while(it!=decoders_.end()) {
		if((*it)->still_active()) 
			return (*it)->image_data();
		++it;
		}
   return 0;
	}

void raster_io::await_completion(void) const
	{
	vector<decoder *>::const_iterator it=decoders_.begin();
	while(it!=decoders_.end()) {
		if((*it)->still_active())
			(*it)->data_end();
		++it;
		}
	}

string raster_io::decoder_type(void) const
	{
	vector<decoder *>::const_iterator it=decoders_.begin();
	while(it!=decoders_.end()) {
		if((*it)->still_active()) 
			return (*it)->identifier();
		++it;
		}
   return "unknown";
	}
