
#include "config.h"
#include <rasterimage/raster.hh>
#ifdef HAVE_LIBZ
#ifdef HAVE_LIBPNG
#include <rasterimage/pngraster.hh>
#define HAVEPNGZ 1
#endif
#endif
#ifdef HAVE_LIBUNGIF
#include <rasterimage/gifraster.hh>
#endif
#ifdef HAVE_LIBJPEG
#include <rasterimage/jpgraster.hh>
#endif
#define DEBUG_CLASS "RASTERIMAGE"
#include <mnemonic/debug.hh>
/*
hasAlpha denotes if there is alpha information
if there is no alpha information the only interesting information 
is a map of 1s in the scaledalpha if one wants to know also can this 
be ignored when you request the availability of the alpha info

*/


scaled_raster_image::scaled_raster_image(long w, long h)
	: 
//	    last_finished_scanline(0), 
	    scaled_width_(-1),
	    scaled_height_(-1),
	    scaled_raster_(0),
	    scaled_alpha_(0)
	{
	debug("scaled_raster_image "<<w<<"x"<<h);
	curframe=0;
	image_data_=new raster_image(w,h);
	debug("done");
	}

	
scaled_raster_image::~scaled_raster_image()
	{
	while (!scaled_raster_.empty()) {delete [] scaled_raster_.back(); scaled_raster_.pop_back();}
	while (!scaled_alpha_.empty())  {delete [] scaled_alpha_.back();  scaled_alpha_.pop_back();}

	}

long scaled_raster_image::width(void) const
	{
	return image_data_->width();
	}

long scaled_raster_image::scaled_width(void) const
	{
	debug(scaled_width_<<" "<<width());
	return scaled_width_==-1?width():scaled_width_;
	}

long scaled_raster_image::height(void) const
	{
	return image_data_->height_;
	}

long scaled_raster_image::scaled_height(void) const
	{
	debug(scaled_height_<<" "<<height());
	return scaled_height_==-1?height():scaled_height_;
	}
/* moved into header
uint8_t *scaled_raster_image::image_data(void) const
	{
	//debug("data requested");
	return raster_;
	}
*/
uint8_t *scaled_raster_image::scaled_image_data(void) 
	{
	//debug("scaled_data requested");
//	if (updated_ && (last_finished_scanline>=height_-1)) {
//		scale(scaled_width_,scaled_height_);
//		}
	if (image_data_->updated_) scale(scaled_width_,scaled_height_);
debug("frame:"<<curframe);
	return (!scaled_raster_.empty())?scaled_raster_[curframe]:image_data_->raster_[curframe];
	}

uint8_t *scaled_raster_image::scaled_image_alpha(void) 
	{
	//debug("scaled_data requested");
//	if (hasAlpha_ && updated_ && (last_finished_scanline>=height_-1)) {
//		scale(scaled_width_,scaled_height_);
//		}
	if (!image_data_->hasAlpha_) return 0;
	if (image_data_->updated_) scale(scaled_width_,scaled_height_);
	return (!scaled_alpha_.empty())?scaled_alpha_[curframe]:image_data_->alpha_[curframe];
	}

void scaled_raster_image::updated(long x,long y,long w ,long h)
	{
	//later someone may use the params to do something smart
	//for now we just notice it and only set a flag
	debug("updated");
	image_data_->updated(x,y,w,h);
	}

void scaled_raster_image::setAlpha(){
	image_data_->setAlpha();
}
void scaled_raster_image::addFrame(){
	image_data_->addFrame();
}

void scaled_raster_image::scale(long w,long h)
	{
	debug("scaling to: "<<w<<"x"<<h);
	if ((w<1) || (h<1)) {
		debug("invalid value");
		return;
		}
         if (!scaled_raster_.empty()) {
	    for (int frm=0;frm<image_data_->frames;frm++) {		
		if (scaled_raster_.back()) {
			delete [] (scaled_raster_.back());
			scaled_raster_.pop_back();			
			}
		if (image_data_->hasAlpha_)
		if (scaled_alpha_.back()) {
			delete [] (scaled_alpha_.back());
			scaled_alpha_.pop_back();
	    		}               
	    }
	    }
	if ((w==width()) && (h==height())) {
		scaled_width_=-1;
		scaled_height_=-1;
		return;
	    
	}
for (int frm=0;frm<image_data_->frames;frm++) {		
	long ih=image_data_->height();
	long iw=image_data_->width();
nod();
	uint8_t *id=image_data_->raster_[frm];
	uint8_t *out=new uint8_t[w*ih*3];
	uint8_t *aid=0;
	uint8_t *aout=0;
nod();
	if (image_data_->hasAlpha_) {
		aid=image_data_->alpha_[frm];
		aout=new uint8_t[w*ih];
		}
nod();
	if (w>=iw) {
		//we have to stretch it a up by linear interpolating
		// first in width
		for (long j=0;j<ih;j++)
			for (long i=0;i<w;i++) {
				float k=static_cast<float>(i)/w*iw;
				for (long z=0;z<3;z++) {
					long xk=z+3*(j*iw+static_cast<long>(k));
					if (static_cast<long>(k)!=iw-1) { 	
						out[j*w*3+3*i+z]=static_cast<uint8_t>((id[xk+3]-id[xk])*(k-static_cast<long>(k))+id[xk]);
						} 
					else {
						out[j*w*3+3*i+z]=id[xk];
						}
					}
				if (image_data_->hasAlpha_) {
				    long xk=j*iw+static_cast<long>(k);
				    if (static_cast<long>(k)!=iw-1) { 	
					aout[j*w+i]=static_cast<uint8_t>((aid[xk+3]-aid[xk])*(k-static_cast<long>(k))+aid[xk]);
					} 
				    else {
					aout[j*w+i]=aid[xk];
					}
				    }
				}
		} 
	else { //shrinking 
		for (long j=0;j<ih;j++)
			for (long i=0;i<w;i++) { 
				//the rounding was causing some weirdness
				float k1=static_cast<float>(i-0.5)/w*iw;
				if (k1<0) k1=0;
				float k2=static_cast<float>(i+0.5)/w*iw;
				if (k2>=iw) k2=iw;
				//suppose k1 is a little smaller then 5		
				long ik1=static_cast<long>(k1);
				//ik1 is now 4
				long ik2=static_cast<long>(k2);
				for (long z=0;z<3;z++) {
					long xk1=z+3*j*iw+3*ik1;
					long xk2=z+3*j*iw+3*ik2+3;
					float r=0;
					long k=xk1+3;
					//start after ik1
					while (k<xk2) {
						r+=id[k];
						k+=3;
						}
					//add values until ik2+1
					//suppose k1=4.99 ik1=4 r-=id[+1]*(0.49)
					//suppose k1=4.6  ik1=4 r-=id[+1]*(0.1)
					//suppose k1=4.4  ik1=4 r+=id[  ]*(0.1)
					//suppose k1=4.01 ik1=4 r+=id[  ]*(0.49)
					
					if (k1-ik1>0.5) r-=id[xk1+3]*((k1-ik1)-0.5);
					else r+=id[xk1]*(0.5-(k1-ik1));
					
					if (k2-ik2>0.5) r+=id[xk2]*((k2-ik2)-0.5);
					else r-=id[xk2-3]*(0.5-(k2-ik2));
					r=r/(k2-k1);
					out[j*w*3+3*i+z]=static_cast<uint8_t>(r);
					}
				if (image_data_->hasAlpha_) {	
				    long xk1=j*iw+ik1;
				    long xk2=j*iw+ik2+1;
				    float r=0;
				    long k=xk1+1;
				    while (k<xk2) {
					r+=aid[k];
					k+=1;
					}
				    if (k1-ik1>0.5) r-=aid[xk1+1]*((k1-ik1)-0.5);
				    else r+=aid[xk1]*(0.5-(k1-ik1));
				    if (k2-ik2>0.5) r+=aid[xk2]*((k2-ik2)-0.5);
				    else r-=aid[xk2-1]*(0.5-(k2-ik2));
				    r=r/(k2-k1);
				    aout[j*w+i]=static_cast<uint8_t>(r);
				    }
				}
		}
	uint8_t *out2=new uint8_t[w*h*3];
nod();
	uint8_t *aout2=0;
	if (image_data_->hasAlpha_) aout2=new uint8_t[w*h];
	if (h>=ih) { 
		//stretch in height
		for (long i=0;i<w;i++)
			for (long j=0;j<h;j++) {
				float k=static_cast<float>(j)/h*ih;
				for (long z=0;z<3;z++) {
					long xk=z+3*(static_cast<long>(k)*w+i);
					if (static_cast<long>(k)!=ih-1) { 	
						out2[j*w*3+3*i+z]=static_cast<uint8_t>((out[xk+3*w]-out[xk])*(k-static_cast<long>(k))+out[xk]);
						} 
					else {
						out2[j*w*3+3*i+z]=out[xk];
						}
					}
				if (image_data_->hasAlpha_) {	
				    long xk=(static_cast<long>(k)*w+i);
				    if (static_cast<long>(k)!=ih-1) { 	
					aout2[j*w+i]=static_cast<uint8_t>((aout[xk+w]-aout[xk])*(k-static_cast<long>(k))+aout[xk]);
					} 
				    else {
					aout2[j*w+i]=aout[xk];
					}
				    }
				}
		} 
	else { 
		//shrink in height
		for (long i=0;i<w;i++)
			for (long j=0;j<h;j++) {
				float k1=static_cast<float>(j-0.5)/h*ih;
				if (k1<0) k1=0;
				float k2=static_cast<float>(j+0.5)/h*ih;
				if (k2>=ih) k2=ih;
				long ik1=static_cast<long>(k1);
				long ik2=static_cast<long>(k2);
				for (long z=0;z<3;z++) {
					long xk1=z+3*(ik1*w+i);
					long xk2=z+3*(ik2*w+i+w);
					float r=0;
					long k=xk1+1*3*w;
					while (k<xk2) {
						r+=out[k];
						k+=3*w;
						}
					if (k1-ik1>0.5) r-=out[xk1+1*3*w]*((k1-ik1)-0.5);
					else r+=out[xk1]*(0.5-(k1-ik1));
					if (k2-ik2>0.5) r+=out[xk2]*((k2-ik2)-0.5);
					else r-=out[xk2-3*w]*(0.5-(k2-ik2));
					r=r/(k2-k1);
					out2[j*w*3+3*i+z]=static_cast<uint8_t>(r);
					}
				if (image_data_->hasAlpha_) {	
				    long xk1=(ik1*w+i);
				    long xk2=(ik2*w+i+w);
				    float r=0;
				    long k=xk1+w;
				    while (k<xk2) 
					{
						r+=out[k];
						k+=w;
						}
				    if (k1-ik1>0.5) r-=aout[xk1+w]*((k1-ik1)-0.5);
				    else r+=aout[xk1]*(0.5-(k1-ik1));
				    if (k2-ik2>0.5) r+=aout[xk2]*((k2-ik2)-0.5);
				    else r-=aout[xk2-w]*(0.5-(k2-ik2));
				    r=r/(k2-k1);
				    aout2[j*w+i]=static_cast<uint8_t>(r);
				    }
				}
		}
nod();
	delete[] out;
debug(frm);	
nod();
	scaled_raster_.push_back(out2);
nod();
        if (image_data_->hasAlpha_) {
	    delete[] aout;
	    scaled_alpha_.push_back(aout2);
	}
}	
nod();
	scaled_height_=h;
	scaled_width_=w;
//	updated_=false;
	}

void scaled_raster_image::scale(float k)
	{
	scale(width()*k,height()*k);
	}
