// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Test program for raster image library. Reads in a graphics file
  in any of the formats supported and outputs some information about
  the image.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <glib.h>
#include <iostream.h>
#include <fstream.h>
#include <rasterimage/raster.hh>

#define DEBUG_CLASS "TEST_RASTERIMAGE"
#include <mnemonic/debug.hh>
#include <mnemonic/stopwatch.hh>

#define GTKHACK

#ifdef GTKHACK
#include <gtk--/window.h>
#include <gtk--/layout.h>
#include <gtk--/adjustment.h>
#include <gtk--/main.h>
#include <gdk--/pixmap.h>
#include <gdk--/color.h>
#include <gtk--/box.h>
#include <gdk/gdkrgb.h>
#endif

class Layout2 : public Gtk::Layout {
	public:
		Layout2(Gtk::Adjustment& h, Gtk::Adjustment& v) : Gtk::Layout(h,v) { };

		ptr<scaled_raster_image> myimage;
	protected:
		Gdk_Pixmap    mypixmap;
		Gdk_Window    win;
		Gdk_GC        gc;
	protected:
		virtual void realize_impl(void) {
		Gtk::Layout::realize_impl();
	   win=gtkobj()->bin_window;
		gint x,y,w,h,depth;
		win.get_geometry(x,y,w,h,depth); 
		debug("geometry: " << x << "," << y << "  " << w << "," << h << "  " << depth);
		// sizes are not valid yet, for some reason...
		debug("creating pixmap");
		mypixmap.create(win,myimage->scaled_width(),myimage->scaled_height());

		gc=Gdk_GC(win);
		gdk_draw_rgb_image(mypixmap.gdkobj(),
								 gc,

0,0,myimage->scaled_width(),myimage->scaled_height(),
								 GDK_RGB_DITHER_NORMAL,

								(guchar*)myimage->scaled_image_data(),

3*myimage->scaled_width());

		debug("drawing pixmap");
		win.set_back_pixmap(mypixmap,false);
		win.clear_area(0,0,myimage->scaled_width(),myimage->scaled_height());
		debug("done");  
		}
};

int main(int argc, char **argv)
	{
	int ac=1;
	int width = 0, height = 0;

	while (ac < argc && argv[ac][0] == '-') {
			switch (argv[ac][1]) {
				case 'H':
					if (argv[++ac])
						height = atoi(argv[ac]);
					break;
				case 'W':
					if (argv[++ac])
						width = atoi(argv[ac]);
					break;
				case 'h':
					cerr << "Usage: test_raster [-dDEBUGCLASS] [-H height] [-W width] [graphics file]" << endl;
					exit(1);
#ifdef DEBUG
				case 'd':
					debugger::instance().turn_on(argv[ac]+2);
					break;
#endif
				}
			++ac;
		}

	try {
		if(ac<argc) {
			raster_io myrio;
			ifstream  in(argv[ac],ios::binary | ios::in);
			unsigned char      buff[16384];
			long total=0;
			
			if(!in.is_open()) {
				cerr << "cannot open file " << argv[ac] << endl;
				exit(1);
				}
			while(in) {
				in.read(buff, 4096);
				if(in.gcount())
					myrio.write(buff,in.gcount());
				if(myrio.image()!=0) {
//					cout << "read up to scanline " << myrio.image()->last_finished_scanline << endl;
					}
				total+=in.gcount();
				}
			
			myrio.await_completion();
			cout << "raw data was " << total << " bytes in " << myrio.decoder_type() << " format" << endl;
			if(myrio.image()!=0) {
				cout << "image info: " << endl
					  << "     width: " << myrio.image()->width() << endl
					  << "     height:" << myrio.image()->height() << endl;
#ifdef GTKHACK
				string title="rasterimage: ";
				title+=argv[ac];

				Gtk::Main mymain(argc,argv);

//				gdk_rgb_set_verbose(true);
				gdk_rgb_set_install(false);
				gdk_rgb_set_min_colors(2*2*2);
				gdk_rgb_init();

				gtk_widget_push_visual(gdk_rgb_get_visual());
				gtk_widget_push_colormap(gdk_rgb_get_cmap());

				Gtk::Window     mywindow;

				mywindow.set_title(title);
				Gtk::Adjustment hadj(0,0,20,8,8,5), vadj(0,0,20,8,8,5);
				Gtk::HBox box;
				mywindow.add(box);
				for(int i=0;i<myrio.image()->frames();i++){
					Layout2   * mylayout=new Layout2(hadj,vadj);
					
					mylayout->myimage=myrio.image();
//				mylayout.myimage->scale(0.5);
					
					if (width != 0 || height != 0) {
						width = width ? width : myrio.image()->scaled_width();
						height = height ? height : myrio.image()->scaled_height();
						cout << "scaling image to " << width << "x" << height << endl;
						myrio.image()->scale(width, height);
						}
					
					mywindow.set_default_size(myrio.image()->scaled_width()*myrio.image()->frames(),
													  myrio.image()->scaled_height());
					
					box.add(*mylayout);
					mylayout->show();
					mywindow.show();
					box.show();
					myrio.image()->nextFrame();
					}
				mymain.run();
#endif
				}
			}
		else {
			cerr << "Usage: test_raster [-dDEBUGCLASS] [graphics file]" << endl;
			exit(1);
			}
		}
	catch(exception& er) {
		cerr << "[exception]:" << er.what() << endl;
		}
	}
