// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Threaded C++ wrappers for SSL based secure low-level tcp/ip 
  operations, interface.

  This class is based on the tcp_connection class that is used throughout
  mnemonic.  This class changes very little, except for the initial
  connection, and read and write functions.  This class directly inherits
  from the tcp_connection class so that minimal changes will be required 
  in the rest of the mnemonic source code.

  NOTE:  This class does not handle proxies, yet.


  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef stcp_connection_hh__
#define stcp_connection_hh__
#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <netdb.h>
#include <stdint.h>
#include <mnemonic/thread_cond.hh>
#include "tcp_connection.hh"
#include "ssleay_defs.h"

class stcp_connection : public tcp_connection {
	public:
		stcp_connection(const string &host, int port);
		/* **************************************************** */
		/* This version doesn't support proxies yet.            */
		/* **************************************************** */

		virtual ~stcp_connection();

		/* **************************************************** */
		/* The tcp_status enum is the same and does not need to */
		/* have changes made to it.                             */
		/* **************************************************** */

		/* **************************************************** */
		/* lookup is the same                                   */
		/* get_port is the same                                 */
		/* get_hostname is the same                             */
		/* set_max_transfer is the same                         */
		/* status is the same                                   */
		/* error_text is the same                               */
		/* **************************************************** */
		virtual int    connect(void);
		virtual bool   uses_proxy(void) const;

		virtual ssize_t read(char *buffer, size_t size);
		virtual ssize_t write(const char *buffer, size_t size);


	protected:
		virtual void        close_(int);

		/* **************************************************** */
		/* SSLeay special stuff:                                */
		/* **************************************************** */

		SSL_CTX *ctx_;
		SSL *con_;
		BIO *sbio_;


};

#endif // stcp_connection_hh__ defined
