// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Threaded C++ wrappers for low-level tcp/ip operations, interface.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef tcp_connection_hh__
#define tcp_connection_hh__
#include <string>
#include <netdb.h>
#include <stdint.h>
#include <mnemonic/thread_cond.hh>

class tcp_connection {
	public:
		tcp_connection(const string &host, int port);
		tcp_connection(const string &h, int p, const string &prox, int prox_p);
		virtual ~tcp_connection();

		enum tcp_status { ok,
		                  not_connected,      // never opened
		                  closed,             // closed by someone
		                  max_transferred,    // transferred enough
		                  connection_refused, // 
		                  timed_out,          // on connect
		                  network_unreachable,//
		                  lookup_failed,
		                  host_not_found,
		                  ns_bad,
		                  other_error,        // check error_text()
								no_route            // no route to host
		};

		virtual int    lookup(void);
		virtual int    connect(void);
		virtual int    get_port(void) const;
		virtual string get_hostname(void) const;
		virtual bool   uses_proxy(void) const;

		virtual void    set_max_transfer(size_t max_transfer); // 0 -> forever
		virtual ssize_t read(char *buffer, size_t size);
		virtual ssize_t write(const char *buffer, size_t size);

		virtual const char *tcp_connection::error_text() const;
		virtual int status(void) const;
	protected:
		static int          total_connections_;
		static thread_cond  total_connections_cond_;
		void                close_(int);

		int status_;
		size_t data_transferred_;

		string host_;
		int port_;
		string proxy_host_;
		int proxy_port_;

		uint32_t connect_ipnum_;
		int      connect_port_;

		thread_mutex mutex_;

		size_t max_transfer_;
		int fd_;
//		struct hostent h_;
		string error_;
};

#endif
