// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Threaded C++ wrappers for SSL based secure low-level tcp/ip
  operations, interface.

  This class is based on the tcp_connection class that is used throughout
  mnemonic.  This class changes very little, except for the initial
  connection, and read and write functions.  This class directly inherits
  from the tcp_connection class so that minimal changes will be required
  in the rest of the mnemonic source code.

  NOTE:  This class does not handle proxies, yet.


  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <string>
#include <strstream.h>
#include <tcpcon/stcp_connection.hh>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include <sys/socket.h>
#include <errno.h>
#include <netinet/in.h>
#include <stdexcept>
#include "s_socket.h"

#define DEBUG_CLASS "STCP"
#include <mnemonic/debug.hh>

/*
int         tcp_connection::total_connections_=0;
thread_cond tcp_connection::total_connections_cond_;
*/

stcp_connection::stcp_connection(const string &h, int p) 
	: tcp_connection(h,p)
	{
	/* *************************************************** */
	/* Use the parent constructor to get all of the normal */
	/* variables set correctly.                            */
	/* *************************************************** */
#if 0
	Old tcp_connection instantiation code left for reference:

	max_transfer_ = 0;
	data_transferred_ = 0;
	fd_ = -1;
	status_ = not_connected;
#endif

	/* *************************************************** */
	/* These two are necessary throughout the class.       */
	/* *************************************************** */
	ctx_ = NULL;
	con_ = NULL;

	/* *************************************************** */
	/* All of the initialization stuff for SSLeay is handled*/
	/* in the connect function, just like the parent obj.  */
	/* *************************************************** */


	}

/* *********************************************************** */
/* Proxies are not supported, so there is only one version of  */
/* the constructor.  That's above.                             */
/* *********************************************************** */
#if 0
tcp_connection::tcp_connection(const string &h, int p, const string &prox, int prox_p) 
	: host_(h), port_(p), proxy_host_(prox), proxy_port_(prox_p)
	{
	max_transfer_ = 0;
	data_transferred_ = 0;
	fd_ = -1;
	status_ = not_connected;
	}
#endif

stcp_connection::~stcp_connection() 
	{
	/* *************************************************** */
	/* SSLeay has it's own version of the shutdown that it */
	/* runs.                                               */
	/* *************************************************** */
	close_(-3);

        if (con_ != NULL)
                SSL_free(con_);
        if (ctx_ != NULL)
                SSL_CTX_free(ctx_);

	status_ = closed;
	}

/* *********************************************************** */
/* We use the parent version of the lookup function.  Since we */
/* do not allow proxy construction of this object, the parent  */
/* variable proxy_port_ will always be -1.                     */
/* *********************************************************** */


int stcp_connection::connect(void) 
	{
	// FIXME: the maximum number of concurrent connections is fixed here
	// at 5. We probably want to make that configurable.

	/* *************************************************** */
	/* This stuff is straight from the parent object.      */
	/* It duplicates code, but not too much.               */
	/* *************************************************** */
	total_connections_cond_.lock();
	while(total_connections_ > 4) {
		debug("total connections is " << total_connections_ << ", waiting...");
		total_connections_cond_.wait();
		}
	++total_connections_;
	total_connections_cond_.unlock();

	/* *************************************************** */
	/* From here on down, this function is totaly different*/
	/* than the parent function.                           */
	/* *************************************************** */

	/* *************************************************** */
	/* What follows now is all of the things that are      */
	/* necessary from the SSLeay library point of view to  */
	/* get things working.                                 */
	/* Most of this is taken directly from the SSLeay      */
	/* client examples and cleaned up thoroughly.          */
	/* *************************************************** */

	/* *************************************************** */
	/* These variables are only necessary in the connect   */
	/* *************************************************** */
	BIO *bio_c_out = NULL;
	int k, width, err;
	char *cbuf = NULL;
	char *CApath = NULL, *CAfile = NULL;
	SSL_METHOD *meth = NULL;

	debug("Connect function for stcp_connection client running\n");

	meth = SSLv23_client_method();

	apps_startup();

	if ((cbuf = (char *)Malloc(BUFSIZZ)) == NULL){
		throw logic_error(
			"out of memory in stcp_connection connect function");
	}

	bio_c_out = BIO_new_fp(stdout, BIO_NOCLOSE);

	SSLeay_add_ssl_algorithms();
	ctx_ = SSL_CTX_new(meth);
	if (ctx_ == NULL) {
		throw logic_error(
			"CTX is null in stcp_connection connect function");
	}

	SSL_CTX_set_options(ctx_, 0);
	if ((!SSL_CTX_load_verify_locations(ctx_, CAfile, CApath)) ||
	    (!SSL_CTX_set_default_verify_paths(ctx_))) {
		/* ******************************************* */
		/* There is typically an error here that has to*/
		/* do with the fact that a normal client does  */
		/* not have Certificates setup and handled on  */
		/* their machine.  This doesn't hurt anything, */
		/* so we ignore the error.                     */
		/* ******************************************* */
		debug("Standard error with Certificate file and path");
	}
	SSL_load_error_strings();

	con_ = (SSL *) SSL_new(ctx_);

	init_client(&fd_, (char *)host_.c_str(), port_);

	sbio_ = BIO_new_socket(fd_, BIO_NOCLOSE);

	SSL_set_bio(con_, sbio_, sbio_);
	SSL_set_connect_state(con_);

	/* *************************************************** */
	/* TODO: Make the error handling here match the parent */
	/*       error handling that everyone is used to.      */
	/* *************************************************** */

	/* ok, lets connect */
	width = SSL_get_fd(con_) + 1;

	debug("Doing the SSLeay initialization and handshakes");
	// Initialization and handshakes:
	err = SSL_in_init(con_);

	/* *************************************************** */
	/* This little spurt of blank data is necessary to get */
	/* the connection finalized and ready for real data.   */
	/* It doesn't seem necessary, but I've never been able */
	/* to use the SSLeay library without doing this little */
	/* dance with the null data.                           */
	/* *************************************************** */
	debug("Sending blank data\n");
	memset(cbuf, 0, BUFSIZZ);
	k = SSL_write(con_, cbuf, strlen(cbuf));

	/* *************************************************** */
	/* Then free up the buffer:                            */
	/* *************************************************** */
	free(cbuf);

	if (bio_c_out != NULL) {
		BIO_free(bio_c_out);
		bio_c_out = NULL;
	}

	debug("Secure tcp_connection connect function done");

	/* *************************************************** */
	/* That's all to it!                                   */
	/* *************************************************** */
	status_ = ok;
	return 0;

	/* *************************************************** */
	/* Here's the parent style error handling.  Use it as  */
	/* guide to make our function work the same.           */
	/* *************************************************** */
#if 0
	int retval=0;

	if( ::connect(fd_, (struct sockaddr *)&name, sizeof(name)) ) {
		switch(errno) {
			case ECONNREFUSED:
				status_=connection_refused;
				debug("connection refused");
				retval=-1;
				break;
			case ETIMEDOUT:
				status_=timed_out;
				debug("connection timed out");
				retval=-1;
				break;
			case ENETUNREACH:
				status_=network_unreachable;
				debug("network unreachable");
				retval=-1;
				break;
			default:
				error_ = "unhandled error on connect: "+string(strerror(errno));
				status_ = other_error;
				total_connections_cond_.lock();
				--total_connections_;
				total_connections_cond_.broadcast();
				total_connections_cond_.unlock();
				throw logic_error(error_);
			}
		}
	if(retval!=0) {
		total_connections_cond_.lock();
		--total_connections_;
		total_connections_cond_.broadcast();
		total_connections_cond_.unlock();
		}
	return retval;
#endif

	}

ssize_t stcp_connection::read(char *buffer, size_t buf_size) 
	{
	if(status_ != ok)
		return -1;
	if(max_transfer_!=0 && data_transferred_ >= max_transfer_) {
		status_ = max_transferred;
		return -1;
		}
	size_t amt_to_read;
	if(max_transfer_) {
		size_t amt_left_on_conn = max_transfer_ - data_transferred_;
		amt_to_read = (amt_left_on_conn < buf_size) ? (amt_left_on_conn) : buf_size;
		} 
	else {
		amt_to_read = buf_size;
		}
	debug("trying to read "<<amt_to_read);
	int amt_read=0;
	
	while((amt_read = SSL_read(con_, buffer, amt_to_read)) == -1 || amt_read == 0) {
		if(amt_read == 0) {
			debug("end of file in read()");
			status_ = closed;
			close_(-3);
			return -1;
			}
		/* ************************************************ */
		/* All of this error handling is based on the errno */
		/* variable.  Check to see if SSLeay is different.  */
		/* ************************************************ */
		switch(errno) {
			case EINTR:  // ok, let's try again
				debug("received EINTR in read()");
				continue;
			case EAGAIN: // can't happen
				debug("received EAGAIN in read()");
				continue;
			case EIO:    // oof, some bad things happen
				status_ = closed;
				close_(-3);
				return -1;
			case EBADF:
			case EINVAL:
			case EFAULT:
				error_ = "[tcp_connection]: invalid file descriptor for reading";
				status_ = other_error;
				close_(-3);
				return -1;
				// FIXME: handle ECONNRESET and retry (this is what makes netscape fail
				// with long downloads). Also see the error handling of wget.
			default:     // should not happen
				debug("read resulted in error " << errno);
				throw logic_error("[tcp_connection]: unhandled error during read");
			}
		}
	data_transferred_+=amt_read;
	return amt_read;
	}


ssize_t stcp_connection::write(const char *buffer, size_t size) 
	{
	if(status_!=ok) return -1;
	size_t amt_written = 0;
	int res;
	while(amt_written != size) {
		if((res=SSL_write(con_,(char *)(buffer+amt_written),size-amt_written)) == -1) {
			/* ************************************************ */
			/* All of this error handling is based on the errno */
			/* variable.  Check to see if SSLeay is different.  */
			/* ************************************************ */
			switch(errno) {
				case EBADF:
				case EINVAL:
					error_ = "[tcp_connection]: bad file descriptor for writing";
					status_ = other_error;
					close_(-3);
					return -1;
				case EIO:
					debug("Got EIO");
					status_ = closed;
					close_(-3);
					return -1;
				case EPIPE:
					debug("Got EPIPE");
					status_ = closed;
					close_(-3);
					return -1;
				case EINTR: 
					debug("received EINTR in write()");
					continue;
				default:
					status_ = other_error;
					error_ = "[tcp_connection]: unhandled error during write"; 
					throw logic_error(error_.c_str());
				}
			} 
		else amt_written+=res;
		debug("Trying to write data (so far "<<amt_written<<" of"<<size<<")");
		}
	debug("Done writing block");
	return 0;
	}

void stcp_connection::close_(int f)
	{
	if(f!=-3)
		throw logic_error("Only the internal methods of stcp_connection should ever call the close_() function.  Someone else has done this.");

	SSL_shutdown(con_);
	SHUTDOWN(SSL_get_fd(con_));
	total_connections_cond_.lock();
	--total_connections_;
	total_connections_cond_.broadcast();
	total_connections_cond_.unlock();
	}


bool stcp_connection::uses_proxy() const
	{
	return false;
	}
