
#ifndef texpp_eqtb_hh__
#define texpp_eqtb_hh__

#include <string>
#include <vector>
#include <layout/numbers.hh>
#include <texdoc/commands.hh>
#include <stdint.h>

namespace tex {
 
 class token_list;
 
 template<class T>
	 class tex_table {
		 public:
			 tex_table(int);
			 T          get_user(int) const;
			 void       set_user(int,T);
		 protected:
			 int        level(int) const;
			 T          get(int) const;
			 string     name(int) const; // move to derived?
		 private:
			 class saver {
				 public:
					 enum stype { restore_old_value, restore_zero, 
									  insert_token, level_boundary 
				};
					 stype type_;
					 T     value_;
			 };
			 vector<pair<int,T> >  memory_;
			 vector<saver>         stack_;
	 };
 
 class skip_registers : public tex_table<int16_t> {
	 public:
		 skip_registers(void) : tex_table<int16_t>(256) {} ;
		 enum skip_code { line_skip, baseline_skip, par_skip, above_display_skip, 
								above_display_short_skip, below_display_short_skip, left_skip, 
								right_skip, top_skip, split_top_skip, tab_skip, space_skip, 
								xspace_skip, par_fill_skip, thin_mu_skip, med_mu_skip,
								thick_mu_skip
		};
		 skip_code name2num(string) const; // move to derived?
		 int       name2num_user(string) const; // given an ascii sequence, return the integer 
		 
		 int   level(skip_code) const;
		 int16_t get(skip_code) const;
		 int16_t get_user(int) const;  // for the user-allocated registers
	 protected:
 };
 
 class penalty_registers : public tex_table<int32_t> {
	 public:
		 penalty_registers(void) : tex_table<int32_t>(256) {} ;
		 enum penalty { pretolerance, tolerance, line_penalty, hyphen_penalty,
							 ex_hyphen_penalty, club_penalty, widow_penalty,
							 display_penalty, broken_penalty, bin_op_penalty,
							 rel_penalty, pre_display_penalty, post_display_penalty,
							 inter_line_penalty, double_hyphen_demerits,
							 final_hyphen_demerits, adj_demerits, mag_, delimiter_factor,
							 looseness_, time_, day_, month_, year_
		 }; //FIXME: there are many others as well
};
 
 class dimensions : public tex_table<int32_t> {
	 public:
		 dimensions(void) : tex_table<int32_t>(256) {} ;
		 enum dimen { par_indent, math_surround, line_skip_limit, hsize_, vsize_,
						  max_depth, split_max_depth, box_max_depth, hfuzz_, vfuzz_,
						  deliiter_shortfall, null_delimiter_space, script_space, 
						  pre_display_size, display_width, display_indent, overfull_rule,
						  hang_indent, h_offset, v_offset, emergency_stretch
		 };
		 int     level(dimen) const;
		 int32_t get(dimen) const;
		 int     level_user(dimen) const;
		 int32_t get_user(dimen) const;
 };
 
 class token_references : public tex_table<token_list *> {
	 public:
	  token_references(void) : tex_table<token_list *>(256) {} ;
		 enum token_ref { output_routine_tl, every_par_tl, every_math_tl,
								every_display_tl, every_hbox_tl, every_vbox_tl,
								every_job_tl, every_cr_tl, err_help_tl 
		 };
		 int     level(token_ref) const;
		 int16_t get(token_ref) const;
		 int16_t get_user(int) const; // user-allocated token_list references
 };
 
 class cat_codes : public tex_table<cat::code> { 
	 public:
		 cat_codes(void) : tex_table<cat::code>(256) {} ;
		 int       level(int);
		 cat::code get_user(int num) const { return tex_table<cat::code>::get_user(num); };
};
 
 class tex_eqtb {
	 public:
		 tex_eqtb(void);
		 
		 skip_registers         skips_;
		 penalty_registers      penal_;
		 dimensions             dimen_;
		 token_references       token_;
		 cat_codes              ccode_;
 };

};

#endif
