
/*

Here is an example macro definition that illustrates these conventions.
After \TeX\ processes the text
$$\.{\\def\\mac a\#1\#2 \\b \{\#1\\-a \#\#1\#2 \#2\}}$$
the definition of \.{\\mac} is represented as a token list containing
$$\def\,{\hskip2pt}
\vbox{\halign{\hfil#\hfil\cr
(reference count), |letter|\,\.a, |match|\,\#, |match|\,\#, |spacer|\,\.\ ,
\.{\\b}, |end_match|,\cr
|out_param|\,1, \.{\\-}, |letter|\,\.a, |spacer|\,\.\ , |mac_param|\,\#,
|other_char|\,\.1,\cr
|out_param|\,2, |spacer|\,\.\ , |out_param|\,2.\cr}}$$
The procedure |scan_toks| builds such token lists, and |macro_call|
does the parameter matching. 

See section \[20] of tex.web. 

*/

#ifndef token_list_hh_
#define token_list_hh_

#include <unistd.h>
#include <locale.h>

namespace tex {
 
 class token_list {
	 public:
		 enum token_types { left_brace_token,
								  right_brace_token,
								  left_brace_limit,
								  right_brace_limit,
								  math_shift_token,
								  tab_token,
								  out_param_token,
								  space_token,
								  letter_token,
								  other_token,
								  match_token,
								  end_match_token };  // should these be represented by their Unicode values?
		 // Can we use the PUA for the special TeX stuff, like locations of macro parameters and such?
		 // Then a token_list is just a string with some matching members that know about the meaning
		 // of these PUA characters.

		 NOTE:

		 Catcode mess is simply input encoding -> Unicode in a very flexible way.

		 vector<parameters> match(token_list *);
	 protected:
		 int reference_count_; // should we do this with ptr<...>s?

		 vector<wchar_t> tokens_; // anticipating Unicode switch, in which case token_types can be stored
                          		  // here as well.
 };

};

#endif
