/*                              ENCODING TO PRINTABLE CHARACTERS
                                             
   File module provides functions HTUU_encode() and HTUU_decode() which convert a buffer
   of bytes to/from RFC 1113 printable encoding format. This technique is similar to the
   familiar Unix uuencode format in that it maps 6 binary bits to one ASCII character (or
   more aptly, 3 binary bytes to 4 ASCII characters).  However, RFC 1113 does not use the
   same mapping to printable characters as uuencode.

 changed for mnemonic - urut 31.12.1997   
 */

#ifndef htuu_string_hh__
#define htuu_string_hh__

#include <vector.h>

template<class InputIterator, class OutputIterator>
void htuu_encode(InputIterator begin, InputIterator end, OutputIterator out)
	{
	bool b2,b3;
   begin.value_type i1,i2,i3,i4;

	while(begin!=end) {
		i1=i2=i3=i4=0;
		b2=b3=false;

		i1=*(begin++);
		if(begin!=end) { i2=*(begin++); b2=true; }
		if(begin!=end) { i3=*(begin++); b3=true; }

		*(out++) = six2pr[ *(begin) >> 2 ];
		*(out++) = six2pr[ ((*(begin++) << 4) & 060) | ((i2 >> 4) & 017)]; 
		if(b2)
			*(out++) = six2pr[ ((i2 << 2) & 074) | ((i3 >> 6) & 03)];
		else 
			*(out++) = '=';
		if(b3)
			*(out++) = six2pr[ i3 & 077];
		else 
			*(out++) = '=';
		}
	}

//string htuu_decode(const string& in) const;

#endif
