// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  URL string, interface. 

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Based on code Copyright (C) 1997 Andrew Lewycky <plewycky@oise.utoronto.ca>
  Reference: RFC 2068 
*/

#ifndef url_hh__
#define url_hh__

#include <string>

class url_string {
	public:
		url_string(const char *url, const url_string& referrer); // we clone referrer
		url_string(const url_string& url);
		url_string(const char *url);
		url_string();
		virtual ~url_string();
		
		string            get_scheme(void) const;         // 'scheme:'
		string            get_original(void) const;       // string used in constructor, unparsed
		const url_string *get_referrer(void) const;
		string          	get_url(void) const;            // full url
		string          	get_net_path(void) const;       // network host
		string          	get_absolute_path(void) const;  // always return full path
		string          	get_relative_path(void) const;  // in case the url was relative
		string            get_object(void) const;         // path removed, just the file
		string          	get_parameters(void) const;     // ';parameters'
		int             	get_port(void) const;
		string          	get_query(void) const;          // '?query'
		string          	get_fragment(void) const;       // '#fragment'
		string          	get_username() const;
		string          	get_passwd() const;
		
		url_string& operator=(const url_string &);
		
		void              set_username(const string&);
		void              set_passwd(const string&);
	protected:
		virtual int default_port(void) const;

		/* Decode the URL as if it were a relative URL starting at iterator */
		/* All the following functions assuming they are working on original_url */
		virtual void DoRelDecode(string::const_iterator) const;
		virtual string::const_iterator DecodeScheme(string::const_iterator) const;
		virtual string::const_iterator DecodeNetPath(string::const_iterator) const;
		virtual string::const_iterator DecodeAbsPath(string::const_iterator) const;
		virtual string::const_iterator DecodeRelPath(string::const_iterator) const;
		virtual string::const_iterator DecodeParams(string::const_iterator) const;
		virtual string::const_iterator DecodeQuery(string::const_iterator) const;
		virtual string::const_iterator DecodeFragment(string::const_iterator) const;
		/* Called if the URL starts with scheme:, if you don't do anything
			(default), then DoRelDecode is called after scheme: */
		virtual string::const_iterator HandleAbs(string::const_iterator) const;
		void         finish_decoding_(void) const;
		virtual void fixup_(void) const;

		string original_url;     /* What was given in constructor */
		url_string *referrer;    /* What URL this was embedded in (copy of it) */

		mutable string url;           /* A non-relative URL created from above */
		mutable string scheme;       /* First section of URL */
		mutable string net_path;     /* Hostname, usually */
		mutable string abs_path;     /* Absolute path */
		mutable string rel_path;     /* Relative path */
		mutable string params;
		mutable string query;
		mutable string fragment;
		mutable string passwd;
		mutable string username;
		mutable int    port;
		
		mutable bool is_relative_;              /* If starts with scheme:, is not relative*/
		mutable bool decoding_finished_;        /* Whether we have finished parsing the URL */
};

#endif
