/*
Project Mnemonic - Roman Numerals helper function
Copyright (C) 1998 - Alejandro J. Cura

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Loosely based on Convert-O-Roman v1.0.0, public domain.
by Morten Wang, 1996. [warmor@hs.nki.no] & [warnckew@aft.sn.no]
Adapted to <string> and optimized a bit by alecu, 1998. [alecu@mnemonic.org]

Compile with:
	$ g++ -DSTANDALONE -o roman roman.cc

Test:
	$ ./roman 1998
	$ ./roman
*/

#include "roman.h"
#include <string>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

/* Function that does the actual conversion. */
/* Divides by 1000, 100 & 10, and adds the corresponding values */
/* to the string we store the roman numeral in. */
/* returns 0 if OK, -1 if exceeds 10000 */
char *hundreds[]={"","C","CC","CCC","CD","D","DC","DCC","DCCC","CM"};
char *tens[]={"","X","XX","XXX","XL","L","LX","LXX","LXXX","XC"};
char *units[]={"","I","II","III","IV","V","VI","VII","VIII","IX"};

int int2roman(int num, string &buf, bool lowercase) {
	string rom;
	int temp, i;

	if (num >= 10000 || num < 0)
		return(-1);

	temp = num/1000;
	num  = num%1000;
	assert( 0<=temp && temp<10 );
	for(i = 0; i < temp; i++)
		rom+= "M";

	temp = num/100;
	num  = num%100;
	assert( 0<=temp && temp<10 );
	rom+= hundreds[temp];

	temp = num/10;
	num  = num%10;
	assert( 0<=temp && temp<10 );
	rom+= tens[temp];

	temp = num;
	assert( 0<=temp && temp<10 );
	rom+= units[temp];

	if (lowercase) {
		const char* p= rom.c_str();
		while( *p != '\0' ) {
			buf+=tolower(*p++);
		}
	} else {
		buf= rom;
	}
	return(0);
}

#ifdef STANDALONE

main (int argc, char** argv) {
	int n;

	if (argc>1) {
		string sample;
		n=atoi(argv[1]);
		int2roman( n, sample );
		printf( "%d= %s \n", n, sample.c_str() );
	} else {
		for(n=-10; n< 10010; n++ ){
			string sample;
			int2roman( n, sample, true );
			printf( "%d= %s \n", n, sample.c_str() );
		}
	}
}

#endif /* STANDALONE */
