// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  W3C DOM events implemented using mnemonic messages.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef dom_event_hh__
#define dom_event_hh__

#include <mnemonic/messagebase.hh>
#include <mnemonic/ptr.hh>
#include <dom/dom.hh>

class msg_dom_event : public message<msg_dom_event> {
	public:
		ptr<DOM::Document>   document;
		vector<DOM::Node *>  targets;
		DOM::Node           *currentNode;
		bool cancelBubble;
		bool cancelCapture;
		bool returnValue;
};

class msg_dom_ui_event : public msg_dom_event, public message<msg_dom_ui_event> {
	public:
		enum event_type { click, dblclick, mousedown, mouseup,
								mouseover, mousemove, mouseout,
								keypress, keydown, keyup, resize,
								scroll };
		long screenX, screenY;
		long clientX, clientY;
		bool altKey;
		bool ctrlKey;
		bool shiftKey;
		unsigned long keyCode;
		unsigned long charCode;
		unsigned short button;

		virtual untyped_message::send_result send(void);
};
								
class msg_dom_create_event : public msg_dom_event, public message<msg_dom_create_event> {
	public:
		ptr<DOM::Document> new_dom_document;
		virtual untyped_message::send_result send(void);
};

class msg_dom_parser_detached_event : public msg_dom_event, public message<msg_dom_parser_detached_event> {
	public:
		virtual untyped_message::send_result send(void);
};

class msg_dom_mutation_event : public msg_dom_event, public message<msg_dom_mutation_event> {
	public:
		enum event_type { subtreeModified, nodeInsertedOntoParent,
								nodeRemovedFromParent, childInsertedOntoParent,
								childRemovedFromParent, nodeRemovedFromDocument,
								nodeInsertedIntoDocument, nodeRemovedFromSubtree,
								nodeInsertedIntoSubtree, attrModified,
								characterDataModified };
		
		virtual untyped_message::send_result send(void);
};

class msg_dom_html_event : public msg_dom_event, public message<msg_dom_html_event> {
	public:
		enum event_type { load, unload, abort, error, select,
								change, submit, reset, focus, blur };

		virtual untyped_message::send_result send(void);
};


#endif
