// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Message announcing creation of new gtk widgets.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef object_ready_hh_
#define object_ready_hh_

#include <mnemonic/messagebase.hh>
#include <mnemonic/ptr.hh>
#include <gtklayout/gtkboxes.hh>
#include <gtk--/widget.h>
#include <gtk--/main.h>

class msg_gtk_object_ready : public message<msg_gtk_object_ready> {
	public:
		msg_gtk_object_ready(void);

		// One of these will be filled by the receiver, the other one is 0.
		// The sender can use these to hook new widgets or layout boxes
		// into existing trees.
		Gtk::Container          *widget;
		layout::container_box  *layoutbox;

		// The box that the sender wants to add to the tree.
		layout::container_box  *new_layoutbox;
};

class Threadsafe_Main {
	public:
		Threadsafe_Main(int *argc, char ***argv, bool have_locale=false);
		Threadsafe_Main(int &argc, char **&argv, bool have_locale=false);

		// access to the singletons
		static Threadsafe_Main *instance(void);
		Gtk::Main              &base(void);

		// this can be called outside of the event loop to gain control of gtk
		void interrupt_event_loop(void);
		void restart_event_loop(void);
		
	protected:
		static void interrupt_event_loop_(void *arg, int fd, GdkInputCondition cond);
		// this should be called by the interrupted event loop before resuming
		void wait_for_restart_(void);
	private:
		void init_(void);
		static Threadsafe_Main *instance_;
		Gtk::Main                    main_;

		void set_interrupt_handler_(void);
		int fd_[2];
		thread_mutex display_working_;
		thread_cond  gtk_in_use_;
		thread_cond  waiting_;
};

#endif
