// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Network layer status messages, implementation.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.

  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <mnemonic/messagebase.hh>
#include <msg/network/network_status.hh>

msg_network_status::msg_network_status(msg_network_status::status ident, const char *m, const url_string& ur) 
	: message<msg_network_status>()
	{
	text = m;
	identifier = ident;
	url=ur;
	}

string msg_network_status::status_as_text(void) const
	{
	switch(identifier) {
		case connection_refused:
			return "Connection refused";
		case looking_up: 
			return "Looking up";
		case bad_host:
			return "Bad host name";
		case connecting:
			return "Connecting";
		case bad_connection:
			return "Connection is bad";
		case closed:
			return "Connection closed";
		case stalled:
			return "Connection stalled";
		case alive:
			return "Connection alive";
		case sending_request:
			return "Sending request to host";
		case request_sent:
			return "Request sent successfully";
		case unexpected_eof:
			return "Unexpected end of file or stream";
		case finished:
			return "Connection finished";
		case redirected:
			return "We are being redirected";
		case content:
			return "Content type identifier received";
		case authentication:
			return "Authentication request received";
		case bad_id:
			return "Illegal network id used";
		case generic_error:
			return "Generic network layer error";
		case read_error:
			return "Error reading network input";
		case object_truncated:
			return "Network stream truncated";
		case open_error:
			return "Error opening network stream";
		case connection_failed:
			return "Failed to make connection";
		default:
			return "Unknown network error";
		}
	}

untyped_message message<msg_network_status>::static_part;
template message<msg_network_status>;
template interest_base<msg_network_status>;
