// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Raw data to CSS DOM-like tree converter.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "csshandler.hh"

#define DEBUG_CLASS "CSSHANDLER"
#include <mnemonic/debug.hh>

void csshandler::add_parser_(group_identifiers& gid, ptr<CSS::stylesheet> cssptr)
	{
	cssparser *tmp=new cssparser(cssptr);
	active_parsers_[gid]=tmp;
	tmp->setup_default();
	debug("add_parser_ done");
	}

void csshandler::detach_parser_(group_identifiers& gid)
	{
	mutex_.lock();
	delete active_parsers_[gid];
	active_parsers_.erase(gid);
	mutex_.unlock();
	debug("css parser detached");
	}

bool csshandler::handle_mutation_(msg_dom_mutation_event& msg)
	{
	mutex_.lock();
	map<group_identifiers, cssparser *>::iterator it;
	if((it=active_parsers_.find(msg))==active_parsers_.end()) {
		debug("no css parser registered for this message");
		mutex_.unlock();
		return false;
		}
	cssparser *pars=(*it).second;
	assert(pars!=0);
	mutex_.unlock();

	debug("new style nodes found?");
	// check whether new STYLE nodes were found
	vector<DOM::Node *>::const_iterator nit=msg.targets.begin();
	while(nit!=msg.targets.end()) {
		DOM::Node::NodeIterator it((*nit), true, false);
		while((*it)) {
			if((*it)->nodeName()=="STYLE") {
				debug("appending style data");
				if((*it)->firstChild()!=0) {
					pars->append_data((*it)->firstChild()->nodeValue());
					pars->parse_buffer();
					}
				}
			it.nextnode();
			}
		++nit;
		}

	return true;
	}

template map<group_identifiers, cssparser *>;
