// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Raw data to CSS DOM-like tree converter.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <msg/dom/dom_event.hh>
#include <map.h>
#include "cssparser.hh"

class csshandler {
	public:
		csshandler(void);
		~csshandler();

		untyped_interest::callresult accept_dom_create_event(msg_dom_create_event &);
		untyped_interest::callresult accept_dom_mutation_event(msg_dom_mutation_event &);
		untyped_interest::callresult accept_dom_parser_detached_event(msg_dom_parser_detached_event &);
	protected:
		thread_mutex mutex_; 

		void add_parser_(group_identifiers& gid, ptr<CSS::stylesheet> cssptr);
		void detach_parser_(group_identifiers& ids);
		bool setup_parser_(group_identifiers& ids, ptr<DOM::Document>& doc);
		bool handle_mutation_(msg_dom_mutation_event& msg);

		ptr<interest<csshandler,msg_dom_create_event> > interest_dom_create_event_;
		ptr<interest<csshandler,msg_dom_mutation_event> > interest_dom_mutation_event_;
		ptr<interest<csshandler,msg_dom_parser_detached_event> > interest_dom_parser_detached_event_;

		map<group_identifiers, cssparser *> active_parsers_;		
};
