// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Raw data to CSS DOM-like tree converter.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "csshandler.hh"
#include <dom/css.hh>
#include <msg/dom/css_event.hh>

#define DEBUG_CLASS "CSSHANDLER"
#include <mnemonic/debug.hh>

csshandler::csshandler(void)
	{
	interest_dom_create_event_=interest<csshandler, msg_dom_create_event>::create(
		this, &csshandler::accept_dom_create_event,untyped_interest::lowest);
	interest_dom_create_event_->acceptance=untyped_interest::ignore_ids;
	interest_dom_mutation_event_=interest<csshandler, msg_dom_mutation_event>::create(
		this, &csshandler::accept_dom_mutation_event,untyped_interest::highest);
	interest_dom_mutation_event_->acceptance=untyped_interest::iid_contains_mid;
	interest_dom_parser_detached_event_=interest<csshandler, msg_dom_parser_detached_event>::create(
		this, &csshandler::accept_dom_parser_detached_event,untyped_interest::lowest);
	interest_dom_parser_detached_event_->acceptance=untyped_interest::iid_contains_mid;
	}

untyped_interest::callresult csshandler::accept_dom_create_event(msg_dom_create_event& msg)
	{
//	cerr << "new DOM document arrived at CSS parser" << endl;
	debug("notified of a new DOM document");
	interest_dom_mutation_event_->add_group_ids(msg);
	interest_dom_parser_detached_event_->add_group_ids(msg);
	if(msg.new_dom_document->doctype()) {
		if(setup_parser_(msg,msg.new_dom_document)) {
			interest_dom_mutation_event_->remove_group_ids(msg);
			interest_dom_parser_detached_event_->remove_group_ids(msg);
			return untyped_interest::pass_on;
			}
		else	
			return untyped_interest::call_next;
		}
	else {
		debug("no document type associated to it yet, so we wait a little");
		return untyped_interest::call_next;
		}

	return untyped_interest::pass_on;
	}

bool csshandler::setup_parser_(group_identifiers& ids, ptr<DOM::Document>& doc)
	{
	debug("doctype is now " << doc->doctype()->name());
	if(doc->doctype()->name().find("HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\"")>=0 ||
		doc->doctype()->name().find("HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\"")>=0 ) {
		debug("that's a document type I have a default stylesheet for");
		mutex_.lock();
		ptr<CSS::stylesheet> cssptr=ptr<CSS::stylesheet>(new CSS::stylesheet(doc,0));

		msg_css_create_event cssmsg;
		cssmsg.new_css_sheet=cssptr;
		cssmsg.start_here=doc->firstChild();
		cssmsg.set_group_ids(ids);
		try {
			cssmsg.send();
			}
		catch(messagebase_error& ex) {
			debug(ex);
			mutex_.unlock();
			return false;
			}
		debug("someone is listening to the CSSStyleSheet");
		add_parser_(ids,cssptr);
		mutex_.unlock();
		debug("mutex unlocked");
		}
	else {
		debug("well, I have no clue how to handle that doctype");
		return false;
		}

	return true;
	}


untyped_interest::callresult csshandler::accept_dom_mutation_event(msg_dom_mutation_event& msg)
	{
	debug("received a mutation event " << msg);
	if(!handle_mutation_(msg)) {
		debug("no parser registered for this DOM yet, let's see if the doctype changed");
		if(msg.document->doctype()) {
			if(setup_parser_(msg,msg.document)) {
				// parse STYLE nodes that are already present
				debug("call handle_mutation");
				handle_mutation_(msg);
				debug("mutation handled");
				return untyped_interest::call_next;
				}
			else	
				interest_dom_mutation_event_->remove_group_ids(msg);
			return untyped_interest::pass_on;
			}
		else {
			debug("still no doctype");
			}
		}

	return untyped_interest::call_next;
	}

untyped_interest::callresult csshandler::accept_dom_parser_detached_event(msg_dom_parser_detached_event& msg)
	{
	interest_dom_mutation_event_->remove_group_ids(msg);
	interest_dom_parser_detached_event_->remove_group_ids(msg);
	detach_parser_(msg);
	return untyped_interest::call_next;
	}


extern "C" { void *create_csshandler(void**); }

void *create_csshandler(void** ) 
   {
   return new csshandler();
   }

template interest<csshandler, msg_dom_create_event>;
template interest<csshandler, msg_dom_mutation_event>;
template interest<csshandler, msg_dom_parser_detached_event>;
