// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Raw data to CSS DOM-like tree converter.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef cssparser_hh__
#define cssparser_hh__

#include <string>
#include <map.h>
#include <mnemonic/ptr.hh>
#include <dom/css.hh>
#include <istream.h>
#include <stdexcept>

class css_parse_error : public logic_error {
	public:
		enum error_type { UNKNOWN,
								INTERNAL_STACK_UNDERFLOW
		};

		css_parse_error(const string& wh, error_type);

		error_type error;
};

ostream& operator<<(ostream&, const css_parse_error&);

class cssparser {
	public:
		cssparser(ptr<CSS::stylesheet>);
		~cssparser(void);

		void setup_default(void);
		void append_data(const string& str);
		friend istream& operator>>(istream& str, cssparser&);
		
		enum parse_result { OK };
		
		parse_result parse_buffer(void);

	private:
	   int                     keep_whitespace_;  // -1: don't know, 0: no, 1: yes

		string  raw_data_;
		unsigned long  offset_in_stream_;    // The place where raw_data[0] is located in the stream.
		unsigned long  lines_in_stream_;     // How many lines were spotted before offset_in_stream.

		// The following three are buffers used while scanning.
		string                   accumulator_;
		string                   property_name_;
		string                   selector_;
		string                   pseudo_class_;

		ptr<CSS::stylesheet>     stylesheet_;

		enum parse_state { INITIAL,
								 SLASH,
								 COMMENT,
								 MAYBE_COMMENT_END,
								 SELECTOR,
								 PSEUDO_CLASS,
								 BRACE_OPEN,
								 PROPVAL,
								 EAT_WHITESPACE,
								 INTERRUPTED
		};

		class history_element {
			public:
				history_element(parse_state, long, long);

				parse_state state;
				unsigned long        offset_in_stream;
				unsigned long        line_in_stream;
		};

		vector<history_element>  parse_history;

		CSS::selector              *current_selector_;
		vector<CSS::selector *>     selector_elements_;
		CSS::declaration           *current_declaration_;

		const history_element& peek_(unsigned int level=0) const;
		unsigned char          tokenise_(unsigned char c) const;
		bool                   add_declaration_element_(const string&, const string&, CSS::declaration *);

		void fill_string2enum_tables_(void);
		
		map<string, int> string2propnum_; // all enums collapsed
};

#endif
