// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Test program for xml parser. Eats raw html/xml/css from files and
  creates/displays a DOM tree.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <iostream.h>
#include <fstream.h>
#include "cssparser.hh"

#define DEBUG_CLASS "TEST_CSSPARSER"
#include <mnemonic/debug.hh>
#include <mnemonic/stopwatch.hh>

ptr<CSS::stylesheet> handle_file(const string& filename)
	{
	ptr<DOM::Document>   doc =new DOM::Document("test","",0);
	ptr<CSS::stylesheet> styl=new CSS::stylesheet(doc, 0);
	cssparser *myparser=new cssparser(styl);

	ifstream str(filename.c_str());

	debug("starting");
	stopwatch sw;
	try {
		while(str) {
			sw.start();
			str >> (*myparser);
			sw.stop();
			cerr << "stream read, this took " << sw << endl;
			sw.start();
		   myparser->parse_buffer();
			sw.stop();
			cerr << "stream parsed, this took " << sw << endl;
			}
		}
	catch(css_parse_error& er) {
		cerr << er << endl;
		}

	str.close();
	cout << endl;
	
	delete myparser;
	return styl;
	}

/*void print_document(ptr<DOM::Document> doc)
	{
	DOM::Node::NodeIterator it(&(*doc),true);
	int level=0;
	while(it.thisnode()!=0) {
		for(int i=0; i<3*level; ++i)
			cout << " ";
		cout << "<" << it.thisnode()->nodeName() << ">" ;
		if(it.thisnode()->nodeName()=="#text") {
			cout << " " << it.thisnode()->nodeValue().substr(0,5);
			if(it.thisnode()->nodeValue().size()>5)
				cout << "...";
			}
		if(it.thisnode()->nodeType()==DOM::Node::ELEMENT_NODE) {
			DOM::Node::NodeIterator it(static_cast<DOM::Element *>(it.thisnode())->firstAttribute(),true);
			while(it.thisnode()!=0) {
				cout << " " << it.thisnode()->nodeName() << "=" << it.thisnode()->nodeValue();
				it.nextnode();
				}
			}
		cout << endl;
		it.nextnode();
		level+=it.levels_changed();
		}
	}
*/

int main(int argc, char **argv)
	{
	int ac=1;

	while (ac < argc && argv[ac][0] == '-') {
			switch (argv[ac][1]) {
				case 'h':
					cerr << "Usage: test_xmlparser [-dDEBUGCLASS] [xml/html/css filename] [iterations]" << endl;
					exit(1);
#ifdef DEBUG
				case 'd':
					debugger::instance().turn_on(argv[ac]+2);
					break;
#endif
				}
			++ac;
		}

	try {
		if(ac<argc && atoi(argv[ac])==0 ) {
			ptr<CSS::stylesheet> doc1=handle_file(argv[ac]);
//			print_document(doc1);
			}
		else {
			ptr<CSS::stylesheet> doc1;
			int iters=1;
			if(ac<argc && atoi(argv[ac])>0)
				iters=atoi(argv[ac]);
			cout << "iterating " << iters << " times, watch my memory." << endl;
			for(int i=0; i<iters; ++i)
				doc1=handle_file("tests/test1.css");
//			print_document(doc1);
			}
		}
	catch(exception& ex) {
		cerr << ex.what() << endl;
		}

	return 0;
	}
