// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Mnemonic http handler implementation.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <url/url_string.hh>
#include "file_handler.hh"
#include "file_thread.hh"

#define DEBUG_CLASS "FILE"
#include <mnemonic/debug.hh>


void file_handler::setup_interests(void) 
	{
	interest_get_url_ = interest<file_handler, msg_get_url>::create(this,&file_handler::accept_get_url,untyped_interest::lowest);
	interest_post_url_ = interest<file_handler, msg_post_url>::create(this,&file_handler::accept_post_url,untyped_interest::lowest);
	interest_put_url_ = interest<file_handler, msg_put_url>::create(this,&file_handler::accept_put_url,untyped_interest::lowest);

	debug("setup_interests done");
	}

void file_handler::destroy_interests() 
	{
	interest_get_url_->unregister();
	interest_post_url_->unregister();
	interest_put_url_->unregister();
	interest_get_url_.wait();
	interest_post_url_.wait();
	interest_put_url_.wait();
	}

untyped_interest::callresult file_handler::accept_get_url(msg_get_url &m) 
	{
	debug("accept_get_url");
	me_.lock();
	debug("locked me_");
	url_string hurl(m.url); // provides all the stuff we need
	debug("Made an file_url out of the url\n");
	if (hurl.get_scheme()!="file") {
		me_.unlock();
		return untyped_interest::refused;
		}
	active_threads_.push_back(new file_thread(hurl, this, m));
	debug("removing old threads");
	while(done_threads_.size()>0) {
	    file_thread *t=(*done_threads_.begin());
	    done_threads_.erase(done_threads_.begin());
	    t->stop();
	    delete t;
	}	
	debug("About to unlock me_");
	me_.unlock();
 	return untyped_interest::call_next;
	}

untyped_interest::callresult file_handler::accept_post_url(msg_post_url &) 
	{
	return untyped_interest::refused;
	}

untyped_interest::callresult file_handler::accept_put_url(msg_put_url &)
	{
	return untyped_interest::refused;
	}

template interest<file_handler, msg_get_url>;
template interest<file_handler, msg_put_url>;
template interest<file_handler, msg_post_url>;
