// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Mnemonic http thread (does all the work)

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define DEBUG_CLASS "file_thread"
#include <mnemonic/debug.hh>
#include "file_thread.hh"
#include <string>
#include <set.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <dirent.h>

// copy all the information we need...
file_thread::file_thread(const url_string &url, file_handler *handler,const group_identifiers &ids) 
	: thread() 
	{
	ids_ = ids;
	url_ = url;
	hh_=handler;
	should_exit_ = 0;
	setup_interests_();
	start();
	}

file_thread::~file_thread() 
	{
	remove_interests_();
	}

void file_thread::cancel(void) 
	{
	should_exit_ = 1;
	}

// on return from this, our thread of execution is finished.
void file_thread::begin() 
	{
	//open file check the flag regularly
	debug("In begin()");

	string name=url_.get_absolute_path();
	if (name[name.size()-1]=='/') {
	    readdir();
		}
	else {
		readfile();
		}

	if(should_exit_) {
		debug("We got told to exit!");
	}
	
	hh_->thread_exit(this);
	debug("done");
	return;
	}

void file_thread::send_status_(msg_network_status::status a, const string& b)
	{
	msg_network_status mns(a,b.c_str(),url_);
	mns.add_group_ids(ids_);
	try{mns.send();} 
	catch(messagebase_error&) {
		debug("msg_network_status had no recipients, ignoring");
		}
	}

void file_thread::send_status_(msg_network_status::status a) 
	{
	send_status_(a,"");
	}




int file_thread::read_(int fd_,char *buf,int length){
  int retval=read(fd_,buf,length);
  switch (retval) {
  case -1:
    {
    string str_err(strerror(errno));
    send_status_(msg_network_status::closed,str_err);
    break;
    }
  case 0:
    send_status_(msg_network_status::finished);
    break;
  default:    
    //do nothing
    break;
  }
  return retval;
}


void file_thread::readfile() 
	{
	string name=url_.get_absolute_path();
	int fd_=open(name.c_str(),O_RDONLY);
	if (fd_==-1) {
		send_status_(msg_network_status::open_error,name+" does not exist.");
		return;
		}
	
   //send a content message based on the extension of the file
	debug("Sending content message   to " << ids_);	
	string extension=name.substr(name.find_last_of(".")+1,name.size()-1);
	string content;
	debug("extension seems to be "<<extension);
   //FIXME move this to a config file
	int buffer_length=16384; //4096;
	if(getenv("MN_FILE_CHUNKSIZE_TXT")) 
		buffer_length=atoi(getenv("MN_FILE_CHUNKSIZE_TXT"));
	if (extension=="jpg" || extension=="jpeg") {
		content="image/jpeg";
		buffer_length=8192;
		}
	else if (extension=="gif") {
		content="image/gif";
		buffer_length=8192;
		}
	else if (extension=="png") {
		content="image/png";
		buffer_length=8192;
		}
	else if (extension=="htm") content="text/html";
	else if (extension=="html") content="text/html";
	else if (extension=="ps") content="application/postscript";

	if (content.empty()) content="text/plain";
	msg_network_status ms(msg_network_status::content, content.c_str(), url_);
	ms.add_group_ids(ids_);
	try { ms.send(); }                                 
	catch(messagebase_error& ex) {                               
    	    debug(ex);
	    //throw;  we don't care whether someone receives this
	}
	debug("Done sending content message");



	debug("data_flow_()");
	char buf[buffer_length];
	ssize_t length=0;
	while((length = read_(fd_,buf,buffer_length)) != -1 && length!=0 && !should_exit_) {
		debug("Got a chunk of data to send as a data message");
		msg_network_data mnd(buf,length);
		mnd.add_group_ids(ids_);
		debug("msg constructed " << mnd);
		try { mnd.send(); } 
		catch(messagebase_error&ex) {
			debug("for id " << ids_ << ":" << endl << ex); 
			debug("nobody wants the data, shut down connection");
			send_status_(msg_network_status::object_truncated);
			should_exit_=true;
			}
		debug("msg sent");
		}
	debug("Done with data_flow, length last block is "<<length);
        close(fd_);
	}

void file_thread::readdir()
	{
	string name=url_.get_absolute_path();
	DIR* ds=opendir(name.c_str());
	if (!ds) {
		send_status_(msg_network_status::open_error,name+" does not exist.");
		return;
		}
	debug("Sending content message   to " << ids_);	
	msg_network_status ms(msg_network_status::content, "text/html", url_);
	ms.add_group_ids(ids_);
	try { ms.send(); }                                 
	catch(messagebase_error& ex) {                               
		debug(ex);
		//throw;  we don't care whether someone receives this
		}
	debug("Done sending content message");
	
	{
		//send pre block
		string prefab;
		prefab="<HTML>\n<HEAD>\n<TITLE>"+url_.get_url()+"</TITLE>\n</HEAD>\n<BODY>\n";
		prefab+="<h1>Directory listing of "+name+"</h1>";
		msg_network_data mnd(prefab.c_str(),prefab.size());
		mnd.add_group_ids(ids_);
		debug("msg constructed");
		try { mnd.send(); } 
		catch(messagebase_error&ex) {
			debug("for id " << ids_ << ":" << endl << ex); 
			}
		}
	
	struct dirent *data;
	struct stat    statdata;
	string prefab;
	set<string, less<string> > dircont;
	while ((data=::readdir(ds))!=0 && !should_exit_) {
		dircont.insert(data->d_name);
		}
	set<string, less<string> >::const_iterator it=dircont.begin();
	int groupthem=0;
	while(it!=dircont.end() && !should_exit_) {
		stat(string(name+"/"+(*it)).c_str(),&statdata);
		if(S_ISDIR(statdata.st_mode))
			prefab+="<A HREF=\""+(*it)+"/\">"+(*it)+"/</A><BR>\n";
		else
			prefab+="<A HREF=\""+(*it)+"\">"+(*it)+"</A><BR>\n";
		++it;
		if(groupthem==100) {
			msg_network_data mnd(prefab.c_str(),prefab.size());
			mnd.add_group_ids(ids_);
			try { mnd.send(); } 
			catch(messagebase_error&ex) {
				debug("for id " << ids_ << ":" << endl << ex); 
				}
			groupthem=0;
			prefab="";
			}
		else ++groupthem;
		}
	closedir(ds);
	//sent pre block
	prefab+="</BODY>\n";
	msg_network_data mnd(prefab.c_str(),prefab.size());
	mnd.add_group_ids(ids_);
	debug("msg constructed");
	try { mnd.send(); } 
	catch(messagebase_error&ex) {
		debug("for id " << ids_ << ":" << endl << ex); 
		}
	send_status_(msg_network_status::finished);
	}

