// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  FTP control thread.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
number- does not work

*/

#include <string>
#include <strstream.h>
#include "ftp_ctrl_thread.hh"
#include "ftp_data_thread.hh"
#include "ftp_list_thread.hh"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <tcpcon/tcp_connection.hh>
#define DEBUG_CLASS "ftp_ctrl_thread"
#include <mnemonic/debug.hh>

#define crlf string("\r\n")
// copy all the information we need...
ftp_ctrl_thread::ftp_ctrl_thread(const ftp_url &url, ftp_handler *handler,const midentifier &id) : thread() 
	{
	id_ = id;
	url_ = url;
	hh_=handler;
	should_exit_ = 0;
        ftp_ctrl_=0;
	fdt_=0;
	setup_interests_();
	start();
	
	}

ftp_ctrl_thread::~ftp_ctrl_thread() 
	{
	delete ftp_ctrl_;
	remove_interests_();
	if (fdt_) {
	    fdt_->stop();
	    delete fdt_;
	}
	}

void ftp_ctrl_thread::cancel(void) 
	{
	should_exit_ = 1;
	if (fdt_) fdt_->cancel();
	}

// on return from this, our thread of execution is finished.
void ftp_ctrl_thread::begin() 
	{
	debug("In begin()");

	if((connect_()) && !should_exit_) 
	    send_request_(); 
	    
	if(should_exit_) {
		debug("We got told to exit!");
	}
	
	hh_->thread_exit(this);
	debug("done");
	return;
	}

void ftp_ctrl_thread::send_status_(msg_network_status::status a, const char *b) 
	{
	msg_network_status mns(a,b,url_);
	mns.add_group_id(id_);
	try{mns.send();} 
	catch(messagebase_error&) {
		debug("msg_network_status had no recipients, ignoring");
		};
	}

void ftp_ctrl_thread::send_status_(msg_network_status::status a) 
	{
	send_status_(a,"");
	}

int ftp_ctrl_thread::read_(char *buf,int l){
        char c;
	int cnt=0;
        int res;
	while (1) {
  	    res=ftp_ctrl_->read(&c,1);
	    if (res!=1) break;
	    if (c!='\r') { buf[cnt]=c; cnt++; }
	    
            if (c=='\n') break;
	    if (cnt==l-1) break;
        }
	buf[cnt]=0;
	if (res==-1) {
	    switch(ftp_ctrl_->status()) {
	    default:
	    case tcp_connection::closed:
		send_status_(msg_network_status::closed);
		break;
	    case tcp_connection::other_error:
		send_status_(msg_network_status::closed,ftp_ctrl_->error_text());
		break;
	    }	    
	}
	return cnt;
}

int ftp_ctrl_thread::to_code_(char *buf){
  int code=(buf[0]-'0')*100+(buf[1]-'0')*10+(buf[2]-'0');
  return code;
}

int ftp_ctrl_thread::read_num_(){
    char buf[1024];
    if (read_(buf,1024)<=0)return -1;
    debug(this<<buf);
    if (buf[3]=='-') {
	int done=0;
	while (!done) {
	    if (read_(buf,1023)<=0) return -1;
	    debug(this<<buf);
	    if (isdigit(buf[0])) {
		if (buf[3]!='-') done=1; 
	    }
	}
    }
    return to_code_(buf);
}


int ftp_ctrl_thread::write_(string cmd){
        debug("write: "<<cmd);
	if (ftp_ctrl_->write(cmd.c_str(),cmd.length())==-1) {
	    switch(ftp_ctrl_->status()) {
	    default:
	    case tcp_connection::closed:
		send_status_(msg_network_status::closed);
		break;
	    case tcp_connection::other_error:
		send_status_(msg_network_status::closed,ftp_ctrl_->error_text());
		break;
	    }	    
	    return 0;
	}
	return 1;
}


int ftp_ctrl_thread::connect_() 
	{
//openfile
	string name;

	ftp_ctrl_=new tcp_connection(url_.get_net_path(),url_.get_port());	
	send_status_(msg_network_status::looking_up);
	if (ftp_ctrl_->lookup()==-1) {
	    switch(ftp_ctrl_->status()) {
	    case tcp_connection::host_not_found:
    		send_status_(msg_network_status::bad_host);
		break;
	    case tcp_connection::ns_bad:
		send_status_(msg_network_status::bad_host,"nameserver not responding");
		break;
	    default:
		break;
	    }
	    return 0;
	}
	if (should_exit_) return 0; //asked to abort
	send_status_(msg_network_status::connecting);
	if (ftp_ctrl_->connect()==-1) {
	    switch(ftp_ctrl_->status()){
	    case tcp_connection::timed_out:
		send_status_(msg_network_status::timed_out);
		break;
	    case tcp_connection::connection_refused:
		send_status_(msg_network_status::connection_refused);
		break;
	    case tcp_connection::network_unreachable:
		send_status_(msg_network_status::network_unreachable);
		break;
	    default:
		send_status_(msg_network_status::closed,"Unknown error while trying to connect to ftp server");
		break;
	    }
	    return 0;
	}
	return 1;
}

int ftp_ctrl_thread::send_request_(){
//sequence is:
//USER,[PASS],PORT,TYPE,RETR
	int l;
	string cmd1="USER anonymous"+crlf;
	string cmd2="PASS mnemonic@"+crlf;
	string cmdquit="QUIT"+crlf;
//change user and pass conforming information from URL
	if (!url_.get_username().empty()) cmd1="USER "+url_.get_username()+crlf;
	if (!url_.get_passwd().empty()) cmd2="PASS "+url_.get_passwd()+crlf;

//skip the adverts
	l=read_num_();
	switch(l){
	case 120:
	case 220:
	  //positive reply
	  break;
	default:
	  //case 421:
 	  write_(cmdquit);
	  return 0;
	}

//announce USER

	if (!write_(cmd1) || should_exit_ ) {
	    write_(cmdquit);
 	    return 0;
	}
//name send lets see what the server has to say...

	l=read_num_();
	switch(l){
	case 230:
	//user logged in proceed (no pass needed)
	break;
	default:
	//case 530:
	//not logged in 
	//case 500:
	//syntax error unknown command
	//case 501:
	//syntax error unknown params
	//case 421:
	//server quits
	//case 332:
	//login required
	write_(cmdquit);
        return 0;
	case 331:
 	  //name okay give password
	  //FIXME
          //if this is a non anonymous user
	  //and we have no passwd we should request one
	  //and also give some feedback if there is a problem with the
          //passwd (msg_get_user_authentication 
	    if (!write_(cmd2) || should_exit_ ) {
		write_(cmdquit);
 		return 0;
	    }
  	  l=read_num_();
	  switch(l){
	    case 202:
	    //command not implement superflous on this site
	    case 230:
	    //user logged in proceed 
	    break;
	    default:
	    //case 530:
	    //not logged in 
	    //case 500:
	    //syntax error unknown command
	    //case 501:
	    //syntax error unknown params
	    //case 421:
	    //server quits
	    //case 332:
	    //login required
 	    write_(cmdquit);
            return 0;
	    
	  }
	}
//If everything went as it should then we are logged in now.
//FIXME returnvalue checking!!!!!!
	if (should_exit_) return 0;
//because we want to setup the data port we must know
//what we are going to do
	string abs_path=url_.get_absolute_path();
	if (abs_path[abs_path.size()-1]=='/') {
	    fdt_=new ftp_list_thread(url_,id_);
	} else {
	    fdt_=new ftp_data_thread(url_,id_);

	}
	if (fdt_->setup_receive_port()==-1) {
	    write_(cmdquit);
	    return 0;
	};

        string tmpname=fdt_->addr;
//socket created and waiting tell server about it.
	strstream buf2;
	tmpname[tmpname.find('.')]=','; //replace the dots 
	tmpname[tmpname.find('.')]=',';
	tmpname[tmpname.find('.')]=',';

	buf2<<"PORT "<<tmpname<<","<<(fdt_->port>>8)<<","<<(fdt_->port%256)<<crlf<<ends;
    
	tmpname=buf2.str();
	delete [] buf2.str();
        debug("["<<tmpname<<"]");
	//FIXME http_handler does now delete[] buf.str() is that correct?
//server now is told of waiting port
	if (!write_(tmpname) || should_exit_ ) {
	    write_(cmdquit);
 	    return 0;
	    }
	l=read_num_();
	switch(l){
		default:
		//case 530:
	        //not logged in 
	    //case 500:
	    //syntax error unknown command
	    //case 501:
	    //syntax error unknown params
	    //case 421:
	    //server quits
	    //try PASV
	    write_("PASV"+crlf);
	    char buf3[1024];
	    read_(buf3,1023);
	    l=to_code_(buf3);
	    switch(l){
	    // returns 227 bla x,x,x,x,p,p
	    // on succes then connect to port
              case 227:
	        if (fdt_->setup_connect_port(buf3)!=-1) break;
		//fall through to the error
	      default:
 	        write_(cmdquit);
                return 0;
	    }
	    case 200:
	    //command okay
	    break;	
	}
//binary transfer 
	if (abs_path[abs_path.size()-1]=='/') {
	    tmpname="TYPE A"+crlf;
	} else {
    	    tmpname="TYPE I"+crlf;
	}
	
	if (!write_(tmpname) || should_exit_ ) {
	    write_(cmdquit);
 	    return 0;
	    }
	l=read_num_();
	switch(l){
	    default:
	    //case 530:
	    //not logged in 
	    //case 500:
	    //syntax error unknown command
	    //case 501:
	    //syntax error unknown params
	    //case 504:
	    //not implemented for that parameter
	    //case 421:
	    //server quits
 	    write_(cmdquit);
            return 0;
	    case 200:
	    //command okay
	    break;	
	}
//now were ready for a file

	int sep=abs_path.size()-1;
	while (abs_path[sep]!='/') sep--;
	
	tmpname="CWD "+abs_path.substr(0,sep+1)+crlf;
	if (!write_(tmpname) || should_exit_ ) {
	    write_(cmdquit);
 	    return 0;
	    }
	l=read_num_();
	switch(l){
	    default:
	    //case 530:
	    //not logged in 
	    //case 500:
	    //syntax error unknown command
	    //case 501:
	    //syntax error unknown params
	    //case 502:
	    //command not implemented
	    //case 421:
	    //server quits
	    //case 550:
	    //file unavailable
	    write_(cmdquit);
	    return 0;
	    case 250:
	    break;
	}	
	//changed dir.
	if (abs_path[abs_path.size()-1]=='/') {
	    tmpname="NLST"+crlf;
	} else {
	    tmpname="RETR "+abs_path.substr(sep+1,abs_path.size()-sep)+crlf;	
	}
	
	if (!write_(tmpname) || should_exit_ ) {
	    write_(cmdquit);
 	    return 0;
	    }
	l=read_num_();
	switch(l){
	    default:
	    //case 530:
	    //not logged in 
	    //case 500:
	    //syntax error unknown command
	    //case 501:
	    //syntax error unknown params
	    //case 421:
	    //server quits
	    //case 450:
	    //case 550:
	    //file unavailable
 	    write_(cmdquit);
            return 0;
	    case 125:
	    // connection already open starting transfer (should not happen)
	    case 150:
	    // file status okay going to open connection
	      {
		fdt_->send_content();
	      fdt_->start();

      		l=read_num_();
		switch(l){

	      // case 110:
	      // restart marker reply (not implemented)
	      case 226:
	      //closing connection transfer complete	      
	      case 250:
	      //requested file completed
	      //fdt_->send_finished(); is sent by data thread
              break;
	      default:
	      //case 425:
	      //can't open connection 
	      //case 426: 
	      //connection closed transfer aborted
	      //case 451:
	      //local error in handling transaction, aborted
		fdt_->cancel();
		//fdt_->send_closed();
		break;
		}
 	      write_(cmdquit);

	      }
	      break;
	}
		

	return 1; //all good
	}


