#include "ftp_data_thread.hh"
#include <unistd.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <msg/network/network_status.hh>
#include <msg/network/network_data.hh>
#define DEBUG_CLASS "ftp_data_thread"
#include <mnemonic/debug.hh>

#ifdef DEBUG
#include <errno.h>
#endif

ftp_data_thread::ftp_data_thread(){
  debug("construct()");
  fd_=-1;
  pipe(pfd_);
  should_exit_=0;
  nod();
}

ftp_data_thread::ftp_data_thread(const ftp_url &url, const midentifier &id):url_(url){
  debug("construct(url,id)");
  fd_=-1;
  id_=id;
  pipe(pfd_);
  should_exit_=0;
  nod();
  
  mode=PORT;
}

ftp_data_thread::~ftp_data_thread(){
  nod();
  if (fd_!=-1) close(fd_);
  close(pfd_[0]);
  close(pfd_[1]);
}

int ftp_data_thread::setup_receive_port()
	{
	char buf[256];
	if (should_exit_) return 0;
	if (gethostname(buf,255)==-1) return -1;
	struct hostent *host;
	//FIXME should this be freed?
	host=gethostbyname(buf);
	if (!host) return -1;
	struct sockaddr_in sin;
	memset(&sin,'\0',sizeof(sin));
	memcpy(&sin.sin_addr,host->h_addr,host->h_length);
	sin.sin_family=AF_INET;
	sin.sin_port=htons(0);
	fd_=socket(AF_INET,SOCK_STREAM,0);
	if (fd_==-1) return -1;
	if (bind(fd_,(struct sockaddr*)&sin,sizeof(sin))==-1) return -1;
	unsigned int s_sin=sizeof(sin);
	if (getsockname(fd_,(struct sockaddr*)&sin,&s_sin)==-1) return -1;
	if (listen(fd_,1)==-1) return -1;		
	addr=inet_ntoa(sin.sin_addr);
	port=ntohs(sin.sin_port);
	return 0; //later maybe a more advanced feadback?
	}
	
int ftp_data_thread::setup_connect_port(char *c)
	{
	//this is called after the PORT command has failed
	//so first cleanup the mess that left
	if (fd_!=-1) close(fd_);
        mode=PASV;
	string s=c; //contains port line from server:227 blablalba (x,x,x,x,p,p)
	debug (s);
	string::iterator b=find(s.begin(),s.end(),'(');
	string::iterator e=find(b,s.end(),')');
	 
	s=s.substr(1+b-s.begin(),e-b-1);
	debug(s);
	e=find(s.begin(),s.end(),','); //x,x,x,x,p,p
	*e='.'; //x.x,x,x,p,p
	e=find(e,s.end(),',');
	*e='.'; //x.x.x,x,p,p
	e=find(e,s.end(),',');
	*e='.'; //x.x.x.x,p,p
	e=find(e,s.end(),',');
	addr=s.substr(0,e-s.begin());
        debug(addr);
	s=s.substr(e-s.begin()+1,s.end()-e-1);
	debug(s);
	b=find(s.begin(),s.end(),',');
	port=256*atoi(s.substr(0,b-s.begin()).c_str());
	debug (port);
	port+=atoi(s.substr(b-s.begin()+1,s.end()-b-1).c_str());
	debug (port);
	  struct in_addr ia;
	  if (inet_aton(addr.c_str(),&ia)==0) {
	    debug("error");
	    return -1;
	    }
	  struct sockaddr_in sin;
	  sin.sin_family=AF_INET;
	  sin.sin_port=htons(port);
	  sin.sin_addr.s_addr=ia.s_addr;
	  fd_=socket(AF_INET,SOCK_STREAM,0);
	  if (fd_==-1) {
	    debug ("error");
	    return -1;
	    }
	  if (connect(fd_,(struct sockaddr*)&sin,sizeof(sin))<0) {
	    debug("error");
	    debug(strerror(errno));
	    return -1;
	    }
	  return 0;
	}

int ftp_data_thread::send_content()
	{
	debug("Sending content message   to " << id_);	
	string path=url_.get_absolute_path();
	string extension=path.substr(path.find_last_of(".")+1,path.size()-1);
	string content;
	debug("extension seems to be "<<extension);
//FIXME move this to a config file
	if (extension=="jpg") content="image/jpeg";
	if (extension=="gif") content="image/gif";
	if (extension=="htm") content="text/html";
	if (extension=="html") content="text/html";
	if (content.empty()) content="text/txt";
	msg_network_status ms(msg_network_status::content, content.c_str(), url_);
	ms.add_group_id(id_);
	try { ms.send(); }                                 
	catch(messagebase_error& ex) {                               
		debug(ex);
		//throw;  we don't care whether someone receives this
		}
	debug("Done sending content message");
	return 0; //later maybe a more advanced feadback?
	}

int ftp_data_thread::send_finished()
	{
	msg_network_status ms(msg_network_status::finished, "", url_);
	ms.add_group_id(id_);
	try { ms.send(); }                                 
	catch(messagebase_error& ex) {                               
		debug(ex);
		//throw;  we don't care whether someone receives this
		}
	return 0; //later maybe a more advanced feadback?
	}

int ftp_data_thread::send_closed()
	{
	msg_network_status ms(msg_network_status::closed, "", url_);
	ms.add_group_id(id_);
	try { ms.send(); }                                 
	catch(messagebase_error& ex) {                               
		debug(ex);
		//throw;  we don't care whether someone receives this
		}
	return 0; //later maybe a more advanced feadback?
	}

void ftp_data_thread::begin()
	{
	debug("thread started for watching data");
	fd_set mask;
	FD_ZERO(&mask);
	FD_SET(fd_,&mask);
	FD_SET(pfd_[0],&mask);
	if (select(getdtablesize(),&mask,0,0,0)==-1) return;
	if (FD_ISSET(pfd_[0],&mask)) {
		//we're told to stop
    return;
		}
	//if (!FD_ISSET(fd_,&mask)) {
	//hmm this is not supposed to be!!
	//}
	debug("accepting data");
        int fd2;
	if (mode==PORT) {
	  struct sockaddr_in sin;
	  unsigned int si=sizeof(sin);
	  fd2=accept(fd_,(sockaddr *)&sin,&si);
	  if (fd2==-1) return;
        } else {
          fd2=fd_;

	  
	}
        bool done=false;
	while(!done) {
		FD_ZERO(&mask);
		FD_SET(fd2,&mask);
		FD_SET(pfd_[0],&mask);
		if (select(getdtablesize(),&mask,0,0,0)==-1) {
			debug("FIXME: REASON");
			done=true;
			}
		if (FD_ISSET(pfd_[0],&mask)) {
			//we're told to stop
    			send_closed();
			done=true;
			}
		if (FD_ISSET(fd2,&mask)) {
			char buf[1024];
			int i=read(fd2,buf,1024);
			switch(i) {
				case 0:
					//send finished message! 
					debug("transfer finished?");
					send_finished();
					done=true;
					break;
				case -1:
					debug("error");
					send_closed();
					//error
					done=true;
					break;
				default:
					debug("Got a chunk of data to send as a data message");
					msg_network_data mnd(buf,i);
					mnd.add_group_id(id_);
					debug("msg constructed");
					try { mnd.send(); } 
					catch(messagebase_error&ex) {
						debug("for id " << id_ << ":" << endl << ex); 
						}
				}
			}
		}
	if (mode==PORT){
		close(fd2);
		}
	}


void ftp_data_thread::cancel(){
  write(pfd_[1],"a",1);
  should_exit_=1;
}
