// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Classes for browsing logic (history handling, connections to the
  network layer etc). GUI independent (will move to a separate library
  soon).

  Copyright 1998-1999 Kasper Peeters.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "browser.hh"

#define DEBUG_CLASS "BROWSER"
#include <mnemonic/debug.hh>

using namespace browser;

pageview::history_buffer::history_buffer(void)
	{
	history_pos_=history_.begin();
	}

pageview::pageview(void)
	{
	setup_interests_();
	}

pageview::~pageview()
	{
	destroy_interests_();
	}

void pageview::goto_url(string ur, bool nocache)
	{
	if(ur=="" && hbuf_.history_.size()>0) {
		ur=*hbuf_.history_pos_;
		}
	debug("goto url requested " << ur);
	clear_connections_();
	midentifier id;
	debug("using " << id);
	url_string theurl(ur.c_str()); // FIXME: I don't like the c_str() interface here
	msg_get_url msg(theurl, nocache);
	msg.add_group_id(id);
	register_connection_interests(msg);
	// Ah, it's a bit more tricky. When one page is visible and we are waiting for
	// the new one to come in, we should definitely already listen to the network
	// status/data messages for the new one. Even before the new gtk object has
	// been announced.
	debug("fetching " << theurl.get_url());
//	<< "ids for page: " << msg << endl;
	try {
		msg.send();
		set_broken_html(false);
		}
	catch(messagebase_error& ex) {
		set_status_display(string("No protocol handler for ")+theurl.get_url()+string("."));
		clear_connections_();
		}
	debug("network code is running");
	}

void pageview::back(void)
	{
	if(hbuf_.history_pos_!=hbuf_.history_.begin()) {
		--hbuf_.history_pos_;
		goto_url(*hbuf_.history_pos_,false);
		if(hbuf_.history_pos_==hbuf_.history_.begin())
			set_backward_sensitive(false);
		set_forward_sensitive(true);
		}
	}

void pageview::forward(void)
	{
	if(hbuf_.history_pos_!=hbuf_.history_.end()) {
		++hbuf_.history_pos_;
		if(hbuf_.history_pos_!=hbuf_.history_.end()) {
			vector<string>::iterator fik=hbuf_.history_pos_;
			++fik;
			if(fik==hbuf_.history_.end())
				set_forward_sensitive(false);
			set_backward_sensitive(true);
			goto_url(*hbuf_.history_pos_,false);
			}
		else --hbuf_.history_pos_;
		}
	}

void pageview::close(void)
	{
	}

void pageview::set_status_display(const string&)
	{
	}

void pageview::set_pageurl_display(const string&)
	{
	}

void pageview::set_url_display(const string&)
	{
	}

void pageview::set_broken_html(bool)
	{
	}

void pageview::set_forward_sensitive(bool)
	{
	}

void pageview::set_backward_sensitive(bool)
	{
	}

void pageview::set_connection_status(connection_status)
	{
	}

pair<string,string> pageview::handle_user_authentication(void)
	{
	return pair<string,string>("","");
	}

url_string pageview::find_connection_(const group_identifiers& gid)
	{
	url_string result;
	progress_mutex_.lock();
	if(attempted_connections_.find(gid)!=attempted_connections_.end()) {
		result=attempted_connections_[gid];
		}
	else if(open_connections_.find(gid)!=open_connections_.end()) {
		result=open_connections_[gid];
		}
	progress_mutex_.unlock();
	return result;
	}

void pageview::add_connection_(const group_identifiers& gid, const url_string& url)
	{
	progress_mutex_.lock();
	if(attempted_connections_.find(gid)!=attempted_connections_.end()) {
		attempted_connections_.erase(gid);
		open_connections_[gid]=url;
		set_connection_status(active_connections);
		}
	else {
		attempted_connections_[gid]=url;
		if(open_connections_.size()==0) set_connection_status(attempted_connections);
		else                            set_connection_status(active_connections);
		}
	progress_mutex_.unlock();
	}

void pageview::remove_connection_(const group_identifiers& gid, 
											 const string& alldonemessage="Document done.")
	{
	progress_mutex_.lock();
	if(attempted_connections_.find(gid)!=attempted_connections_.end())
		attempted_connections_.erase(gid);
	
	if(open_connections_.find(gid)!=open_connections_.end())
		open_connections_.erase(gid);
	
	if(open_connections_.size()==0) {
		if(attempted_connections_.size()==0) {
			set_connection_status(no_connections);
			set_status_display(alldonemessage);
			}
		else set_connection_status(attempted_connections);
		}
	progress_mutex_.unlock();
	}

void pageview::clear_connections_(void)
	{
	progress_mutex_.lock();
	open_connections_.clear();
	attempted_connections_.clear();
	set_connection_status(no_connections);
	progress_mutex_.unlock();
	}

