// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Main browser window.

  Copyright 1998-1999 Kasper Peeters and Preben Randhol.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include "gtk_browser.hh"
#include <gdk/gdkkeysyms.h>
#include <strstream.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/signal.h>
#include <fcntl.h>
#include <fstream.h>

#include <gtk--/main.h>
#include <gdk--/pixmap.h>
#include <gdk--/color.h>
#include <gdk/gdkrgb.h>

#define DEBUG_CLASS "GTKBROWSER"
#include <mnemonic/debug.hh>
#include <mnemonic/stopwatch.hh>
#include <rasterimage/raster.hh>

#include <stdio.h>

gtk_pageview_manager *gtk_pageview_manager::pm_=0;

extern "C" {
static gboolean gdk_rgb_do_colormaps (void)
	{
		return false;
/*	static const gint sizes[][3] = {
		{ 6, 6, 6 }, 
		{ 6, 6, 5 }, 
		{ 6, 6, 4 }, 
		{ 5, 5, 5 }, 
		{ 5, 5, 4 }, 
		{ 4, 4, 4 }, 
		{ 4, 4, 3 }, 
		{ 3, 3, 3 }, 
		{ 2, 2, 2 }
	};
	static const gint n_sizes = sizeof(sizes) / (3 * sizeof(gint));
	gint i;
	
	for (i = 0; i < n_sizes; i++)
		if (gdk_rgb_try_colormap (sizes[i][0], sizes[i][1], sizes[i][2]))
			return TRUE;
	return FALSE; */
	}
		}

gtk_pageview_manager::gtk_pageview_manager(void)
	: kiosk_(false)
	{
	ifstream      in(PREFIX"/lib/mnemonic/pixmaps/mnemonic.gif",ios::binary | ios::in);
	unsigned char buff[16384];
	if(!in.is_open()) {
		cerr << "cannot open spinner gif" << endl;
		}
	else {
		while(in) {
			in.read(buff, 4096);
			if(in.gcount())
				spinner_rio_.write(buff,in.gcount());
			}
		spinner_rio_.await_completion();
		}
	}

gtk_pageview_manager::~gtk_pageview_manager()
	{
	}

void gtk_pageview_manager::first_window_(void)
	{
	g_thread_init(NULL);
	bool install_cmap_=false;
	bool verbose_=false;
	int numreserve=16;
	for(int i=1; i<*argc_; ++i) {
		if(strcmp((*argv_)[i],"--install-cmap")==0) {
			install_cmap_=true;
			continue;
			}
		if(strcmp((*argv_)[i],"--verbose")==0) {
			verbose_=true;
			continue;
			}
		if(strcmp((*argv_)[i],"--kiosk")==0) {
			kiosk_=true;
			continue; 
			}
		if(strcmp((*argv_)[i],"--help")==0) {
			cerr << "Mnemonic gtk client module help:" << endl << endl << endl
				  << "Usage: mnemonic [options] [url]" << endl << endl
				  << "where the available options are" << endl
				  << "   --install-cmap  : use private color map" << endl
				  << "   --kiosk         : run in kiosk mode (full-screen)" << endl
				  << "   --verbose       : show gtk messages" << endl;
			exit(1);
			}
		string tmp=string("WWW_HOME=")+(*argv_)[i];
		putenv(const_cast<char*>(tmp.c_str())); 
		//FIXME putenv should be const char* but isn't for me in my lib
		//more nice would be to use const_cast but i don't know the exact syntax of that one
		}
/*		if(strcmp((*argv_)[i],"--reserve")==0) {
			if(i+1<*argc_) {
				numreserve=atoi((*argv_)[i+1]);
				cerr << "reserving " << numreserve << " colors for anti-aliasing" << endl;
				}
			} */
	Threadsafe_Main main_(argc_,argv_);
	GdkColor color[numreserve*2];
	if(gdk_visual_get_best_depth()==8) {
		// reserve colors
//		cerr << "reserving " << 2*numreserve << " colormap entries for anti-aliasing" << endl
//			  << "(note: this is a hack for 8 bit displays)." << endl;
		for(int i=0; i<numreserve; ++i) {
			color[i].red=65535-4095*(i+1);
			color[i].green=65535-4095*(i+1);
			color[i].blue=65535-4095*(i+1);
			if (!gdk_colormap_alloc_color(gdk_colormap_get_system(), &color[i], FALSE, FALSE)) {
				cerr << "short on colors" << endl;
				}
			}
		for(int i=0; i<numreserve; ++i) {
			color[i+numreserve].red=65535-4095*(i+1);
			color[i+numreserve].green=65535-4095*(i+1);
			color[i+numreserve].blue=65262-273*i;
			if (!gdk_colormap_alloc_color(gdk_colormap_get_system(), &color[i+numreserve], FALSE, FALSE)) {
				cerr << "short on colors" << endl;
				}
			}
		}
	if(verbose_)
		gdk_rgb_set_verbose(verbose_);
	gdk_rgb_set_install(install_cmap_);
	gdk_rgb_set_min_colors(2*2*2);
	gdk_rgb_init();
	gtk_widget_push_visual(gdk_rgb_get_visual());
	gtk_widget_push_colormap(gdk_rgb_get_cmap());

	// free the colors again
	// FIXME: for some reason, if we free them here, T1 can still not get
	// access to the empty slots. I have no idea who's to blame here.
/*	if(gdk_visual_get_best_depth()==8) 
		gdk_colormap_free_colors(gdk_colormap_get_system(), color, 2*numreserve); */

//	elc_.set_interrupt_handler(interrupt_loop_, this);
	
	debug("opening first window");
	open_window_();
	
	debug("starting gtk event loop");
	pm_=this;
// FIXME: if you set the signal handler GDB will get seriously upset.
	pipe(fd_);
	fcntl(fd_[0], F_SETFL, O_NONBLOCK);
	gdk_input_add(fd_[0],GDK_INPUT_READ, update_animations_, this);
	signal(SIGALRM,timer_handler_);
	struct itimerval itim;
	getitimer(ITIMER_REAL, &itim);
//	cerr << "timer set at " << itim.it_interval.tv_sec << "," << itim.it_interval.tv_usec << endl;
	itim.it_interval.tv_sec=0;
	itim.it_interval.tv_usec=100000;
	itim.it_value.tv_sec=0;
	itim.it_value.tv_usec=100000;
	setitimer(ITIMER_REAL, &itim, 0);
	gdk_threads_enter();
	main_.base().run();
	gdk_threads_leave();
	}

void gtk_pageview_manager::update_animations_(void *arg, int fd, GdkInputCondition cond)
	{
	pm_->update_animations();
	// clean the pipe
	static char buf[10];
	while(read(pm_->fd_[0],buf,10)>0);
	}

void gtk_pageview_manager::update_animations(void)
	{
	windows_mutex_.lock();
	vector<gtk_pageview *>::iterator it=windows_.begin();
	while(it!=windows_.end()) {
		(*it)->animate_spinner();
		++it;
		}
	windows_mutex_.unlock();
	}

void gtk_pageview_manager::initialise_spinner_pixmaps(Gdk_Window& win) 
	{
	Gdk_GC      gc=Gdk_GC(win);
	for(int i=0; i<spinner_rio_.image()->frames(); ++i) {
		debug("creating pixmap " << i);
		Gdk_Pixmap mypixmap;
		mypixmap.create(win, spinner_rio_.image()->scaled_width(), 
							 spinner_rio_.image()->scaled_height());
		gdk_draw_rgb_image(mypixmap.gdkobj(),
								 gc,
								 0,0,spinner_rio_.image()->scaled_width(),
								 spinner_rio_.image()->scaled_height(),
								 GDK_RGB_DITHER_NORMAL,
								 (guchar*)spinner_rio_.image()->scaled_image_data(),
								 3*spinner_rio_.image()->scaled_width());
		
		// You need the following patch for gtkmm-1.2.0:
/*
  
(applies to gtkmm-1.2.0/gdk--/gdk--/types.h)

diff -u -r1.27 types.h
--- types.h     2000/04/14 16:02:05     1.27
+++ types.h     2000/04/16 19:52:31
@@ -293,6 +293,8 @@
        Gdk_Handle(const Gdk_Handle &obj):obj_(obj.obj_) {}

      public:
+       void *operator new(size_t s,void* v) {return ::operator new(s,v);}
+
        ~Gdk_Handle() {}

        GdkObj* gdkobj() const

 */
			  spinner_animation_.push_back(mypixmap);
			  spinner_rio_.image()->nextFrame();
		}
	}

unsigned int gtk_pageview_manager::spinner_animation_size(void) const
	{
	return spinner_animation_.size();
	}

Gdk_Pixmap gtk_pageview_manager::spinner_pixmaps(unsigned int num) const
	{
	assert(num<spinner_animation_.size());
	return spinner_animation_[num];
	}

void gtk_pageview_manager::timer_handler_(int num)
	{
	write(pm_->fd_[1],"a",1);
	signal(SIGALRM,timer_handler_);
	}

void gtk_pageview_manager::open_window_(void)
	{
	gtk_pageview *tmp=new gtk_pageview(this,getenv("WWW_HOME"),kiosk_?GTK_WINDOW_DIALOG:GTK_WINDOW_TOPLEVEL);
	windows_mutex_.lock();
	windows_.push_back(tmp);
	windows_mutex_.unlock();
	Gtk::Main::timeout.connect(SigC::slot(tmp,&gtk_pageview::periodic),30);
	
	tmp->set_events(GDK_ALL_EVENTS_MASK);
	tmp->show();
//	tmp->get_window().gdkobj()set_decorations(GDK_DECOR_TITLE);
	}

extern "C" 
	{
	dlobject *create_gtk_client(void**);
	}

dlobject *create_gtk_client(void** /* Args */) 
	{
	return new gtk_pageview_manager();
	}

menubar::menubar(gtk_pageview *cl) 
   : Gtk::HBox(),  owner_(cl)
   {
	Gtk::Menu                    *menu_file = manage( new Gtk::Menu() );
	Gtk::Menu_Helpers::MenuList&  list_file = menu_file->items();
	list_file.push_back(Gtk::Menu_Helpers::MenuElem("_New"));
	list_file.back()->activate.connect(bind(slot(this,&menubar::back), "New"));

/*   itemfac_->create_item( "/File", 0, "<Branch>",0);
   itemfac_->create_item( "/File/tearoff1", 0, "<Tearoff>",0);

   itemfac_->create_item( "/File/New browser window", "<alt>N", "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"New"));

   itemfac_->create_item( "/File/Open file", "<control>O", "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Open"));

   itemfac_->create_item( "/File/Save", "<control>S", "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Save"));

   itemfac_->create_item( "/File/Save as...", 0, "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Save as"));

   itemfac_->create_item( "/File/sep1", 0, "<Separator>",0);
   itemfac_->create_item( "/File/Close window", "<alt>W", "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Close"));

   itemfac_->create_item( "/File/Exit program", "<alt>Q", "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Exit"));

   itemfac_->create_item( "/Edit", 0, "<Branch>",0);
   itemfac_->create_item( "/Edit/tearoff1", 0, "<Tearoff>",0);
   itemfac_->create_item( "/Edit/Copy", "<control>C", "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Copy"));
   itemfac_->create_item( "/Edit/Cut", "<control>X", "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Cut"));
   itemfac_->create_item( "/Edit/Paste", "<control>V", "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Paste"));

   itemfac_->create_item( "/Navigate", 0, "<Branch>",0);
   itemfac_->create_item( "/Navigate/tearoff1", 0, "<Tearoff>",0);
   itemfac_->create_item( "/Navigate/Previous", 0, "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Previous"));
   itemfac_->create_item( "/Navigate/Next", 0, "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Next"));
   itemfac_->create_item( "/Navigate/Reload", 0, "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Reload"));

   itemfac_->create_item( "/View", 0, "<Branch>",0);
   itemfac_->create_item( "/View/tearoff1", 0, "<Tearoff>",0);
   itemfac_->create_item( "/View/Page Source", 0, "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Page Source"));
   itemfac_->create_item( "/View/History", 0, "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"History"));

// FIXME: I'd like to see this auto-generated from the available OILs someday. 
   itemfac_->create_item( "/Modules", 0, "<Branch>",0);
   itemfac_->create_item( "/Modules/tearoff1", 0, "<Tearoff>",0);
   itemfac_->create_item( "/Modules/Raster images (GIF, JPG, PNG)", 0, "<CheckItem>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Images"));
   itemfac_->create_item( "/Modules/Mathematics", 0, "<CheckItem>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Mathematics"));

// I'm not sure how to add option menus, as they should really be controlled
// by the OILs themselves. Anyhow, we need to have options to toggle image loading,
// image animation and so on. The OILs should also be able to add buttons to the
// toolbar.
//
// What we really need is a good idea about how user-interface controls (menus, buttons)
// are associated with the data that is being manipulated. Probably look at Bonobo as well,
// not that we want to use that, but we could at least see if they implement useful ideas.

   itemfac_->create_item( "/Bookmarks", 0, "<Branch>",0);
   itemfac_->create_item( "/Bookmarks/tearoff1", 0, "<Tearoff>",0);
   itemfac_->create_item( "/Bookmarks/Add Bookmark", "<control>Q", "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Add Bookmark"));
   itemfac_->create_item( "/Bookmarks/sep1", 0, "<Separator>",0);
//     itemfac_->create_item( "/Bookmarks/sep1", 0, "<Separator>",0);

	// CONFIG: Put <LastBranch> here to right-align help menu [kasper]
   itemfac_->create_item( "/Help", 0, "<Branch>",0);
   itemfac_->create_item( "/Help/tearoff1", 0, "<Tearoff>",0);
   itemfac_->create_item( "/Help/About", 0, "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"About"));
   itemfac_->create_item( "/Help/Help", 0, "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Mnemonic Help"));
   itemfac_->create_item( "/Help/Mnemonic web site", 0, "<Item>",
			  ItemFactoryConnector<menubar,const char*>
			  (this,&menubar::back,"Website"));

   menu_ = itemfac_->get_menubar_widget(""); 
   g_return_if_fail(menu_.get_object());
   add(menu_); */

	menu_.items().push_front(Gtk::Menu_Helpers::MenuElem("_File","<control>f",*menu_file));
	add(menu_);
   menu_.show();
   }

void menubar::back( const char *m) 
   {
	if(strcmp(m,"Mnemonic Help")==0) {
		owner_->goto_url("file://"PREFIX"/lib/mnemonic/help/help.html",false);
		}
	else if(strcmp(m,"About")==0) {
		owner_->goto_url("file://"PREFIX"/lib/mnemonic/help/about.html",false);
		}
	else if(strcmp(m,"Website")==0) {
		owner_->goto_url("http://www.mnemonic.org",false);
		}
	else if(strcmp(m,"New")==0) {
		debug("opening new window");
		assert(owner_!=0);
		assert(owner_->owner()!=0);
		owner_->owner()->open_window_();
		}
//	else if(strcmp(m,"Exit")==0) {
//		owner_->quit();
//		}
	else if(strcmp(m,"Close")==0) {
//		owner_->close(
		cerr << "about to close window" << endl;
		}
	else {
		cerr << m << " not yet implemented..." << endl;
		}
   }


navigationbar::navigationbar(gtk_pageview *cl)
   : Gtk::HandleBox(), back_widget_("back"), forward_widget_("forward"), toolbar_(), owner_(cl)
   {
   add(toolbar_);

	toolbar_.add(back_widget_);
	toolbar_.add(forward_widget_);

	back_widget_.clicked.connect(SigC::bind(slot(this,&navigationbar::back),"navbar/back"));
	forward_widget_.clicked.connect(SigC::bind(slot(this,&navigationbar::back),"navbar/forward"));

	if(getenv("HTTP_PROXY")) {
		string helptxt="reload the current page from proxy "+string(getenv("HTTP_PROXY"));
		toolbar_.tools().push_back(
			Gtk::Toolbar_Helpers::ButtonElem("reload", 
														bind(slot(this,&navigationbar::back),"navbar/reload"),
														helptxt));

		helptxt="request proxy "+string(getenv("HTTP_PROXY"))+string(" to fetch latest version of this page");
		toolbar_.tools().push_back(
			Gtk::Toolbar_Helpers::ButtonElem("update", 
														bind(slot(this,&navigationbar::back),"navbar/reloadnocache"),
														helptxt));
		}
	else {
		toolbar_.tools().push_back(
			Gtk::Toolbar_Helpers::ButtonElem("reload", 
														bind(slot(this,&navigationbar::back),"navbar/reload"),
														"reload the current page"));
		}
	if(getenv("WWW_HOME")) {
		toolbar_.tools().push_back(
			Gtk::Toolbar_Helpers::ButtonElem("home", 
														bind(slot(this,&navigationbar::back),"navbar/home"),
														"go to your home page"));
		}
   toolbar_.tools().push_back(
		Gtk::Toolbar_Helpers::ButtonElem("stop", 
													bind(slot(this,&navigationbar::back),"navbar/stop"),
													"stop network transfer"));

   toolbar_.show();
   }

void navigationbar::back(const char *what)
   {
	if(strcmp(what,"navbar/back")==0)               owner_->back();
	else if(strcmp(what,"navbar/forward")==0)       owner_->forward();
	else if(strcmp(what,"navbar/home")==0) {
		if(getenv("WWW_HOME"))
			owner_->goto_url(getenv("WWW_HOME"),false);
		}
	else if(strcmp(what,"navbar/reload")==0)        owner_->goto_url("",false);
	else if(strcmp(what,"navbar/reloadnocache")==0) owner_->goto_url("",true);
	else if(strcmp(what,"navbar/stop")==0)          owner_->stop();
	else
		cerr << what << " pressed" << endl;
   }

urlbar::urlbox::urlbox(gtk_pageview *owner)
   : owner_(owner)
   {
   }

void urlbar::urlbox::changed_impl(void)
   {
   Gtk::Entry::changed_impl();
   }

void urlbar::urlbox::activate_impl(void)
   {
   owner_->goto_url(get_text(),false);
   Gtk::Entry::activate_impl();
   }

urlbar::urlbar(gtk_pageview *owner) 
   : Gtk::HandleBox(), url_entry_(owner), hbox_(), url_label_(" URL:  "),  url_goto_("go")
   {
   add(hbox_);
   hbox_.pack_start(url_label_,false);
   hbox_.pack_start(url_entry_,true);
   hbox_.pack_start(url_goto_,false);

   url_label_.show();
   url_entry_.show();
   url_goto_.show();
   hbox_.show();
   }

authdialog::authdialog(gtk_pageview *owner)
	: owner_(owner), name_label_("user name:"), password_label_("password:"), ok_("ok"), cancel_("cancel")
	{
   set_title("Password entry");

	get_action_area()->add(ok_);
	get_action_area()->add(cancel_);

	name_box_.add(name_label_);
	name_box_.add(name_entry_);
	get_vbox()->add(name_box_);

	password_box_.add(password_label_);
	password_box_.add(password_entry_);
	get_vbox()->add(password_box_);

	name_box_.show();
	password_box_.show();
	name_label_.show();
	name_entry_.show();
	password_entry_.set_visibility(false);
	password_label_.show();
	password_entry_.show();
	ok_.show();
	cancel_.show();

	authdialog_mutex_.lock();

	ok_.clicked.connect(SigC::slot(this,&authdialog::authdialog_closed_ok));
	cancel_.clicked.connect(SigC::slot(this,&authdialog::authdialog_closed_cancel));
	}

string authdialog::user(void) const
	{
	return user_;
	}

string authdialog::password(void) const
	{
	return password_;
	}

void authdialog::authdialog_closed_ok(void)
	{
	hide();
	user_=name_entry_.get_text();
	password_=password_entry_.get_text();
	authdialog_mutex_.unlock();
	}
   
void authdialog::authdialog_closed_cancel(void)
	{
	hide();
	user_="";
	password_="";
	authdialog_mutex_.unlock();
	}
   
void authdialog::wait(void)
	{
	authdialog_mutex_.lock();
	authdialog_mutex_.unlock();
	}


gtk_pageview::gtk_pageview(gtk_pageview_manager *owner, const char *starturl, 
									GtkWindowType t=GTK_WINDOW_TOPLEVEL) 
   : Gtk::Window(t), pagebox_(), new_widget(0), wtype_(t), old_widget_(0), label_("Mnemonic"), 
	  navbar_(this), menubar_(this), urlbar_(this), 
	  brokenhtml_button_("html errors"), html_is_broken_(false), html_actual_(true),
	  statbar2_button_("status history"),
	  spinner_request_(no_connections), spinner_actual_(no_connections),
	  spinner_index_(0), authdialog_(this), owner_(owner), lbox_(0)
   {
	if(starturl!=0)
		starturl_=starturl;
	else 
		starturl_="file://"PREFIX"/lib/mnemonic/help/about.html";
			
   set_title("Mnemonic");

	if(wtype_==GTK_WINDOW_DIALOG) {
		set_default_size((int)(gdk_screen_width()), (int)(gdk_screen_height()));
		}
	else {
		set_default_size((int)(gdk_screen_width()*0.6), (int)(gdk_screen_height()*0.875));
		}

   add(vbox_);
	if(wtype_==GTK_WINDOW_TOPLEVEL)
		vbox_.pack_start(menubar_,false);
	vbox_.pack_start(hbox_,false);
	hbox_.pack_start(vbox2_,true);
   vbox2_.pack_start(navbar_,true);
   vbox2_.pack_start(urlbar_,false);
	hbox_.pack_start(spinner_button_,false);
	spinner_button_.add(spinner_area_);
	spinner_area_.set_usize(48,48); // FIXME: perhaps scale based on spinner gif size?
	spinner_button_.set_sensitive(false);
	spinner_area_.set_app_paintable(true);
//   vbox_.pack_start(notebook_,true);
	vbox_.pack_start(pagebox_,true);
	vbox_.pack_start(statbar2_box_,false);
	statbar2_box_.pack_start(statbar2_,true);
	statbar2_box_.pack_start(brokenhtml_button_,false);
	set_broken_html(false);
	statbar2_box_.pack_start(statbar2_button_,false);
   vbox_.pack_start(statbar_,false);
//   notebook_.pages().push_back(Gtk::Notebook_Helpers::TabElem(pagebox_, label_));
//   notebook_.set_tab_pos(GTK_POS_BOTTOM);

//	if(wtype_==GTK_WINDOW_TOPLEVEL) 
//		accelgroup_ = menubar_.get_accelgroup();

//   accelgroup_.attach(*this);
	context_=statbar_.get_context_id("link");
	context2_=statbar2_.get_context_id("messages");
	statbar_.push(context_,"Welcome to Mnemonic!");
	statbar2_.push(context_,"");

	statbar2_button_.clicked.connect(SigC::slot(this,&gtk_pageview::show_status_history));
	brokenhtml_button_.clicked.connect(SigC::slot(this,&gtk_pageview::show_parser_errors));

   vbox_.show_all();
	
	set_events(GDK_KEY_PRESS);

	setup_interests_();
   }

gtk_pageview::~gtk_pageview()
	{
	destroy_interests_();
	}

void gtk_pageview::realize_impl(void)
	{
	Gtk::Window::realize_impl();
   set_policy(true, true, false);
	statbar_.set_usize(10,statbar_.height());
	statbar2_.set_usize(10,statbar2_.height());
	if(wtype_==GTK_WINDOW_DIALOG)
		gdk_window_set_decorations(get_window().gdkobj(),GDK_DECOR_MAXIMIZE);
	if(starturl_!="") goto_url(starturl_,false);
	}

// FIXME: Both delete_event_impl and quit have to signal the other
// mnemonic parts that the window is going away (for instance, the 
// display has to know that the canvas will dissappear and cannot
// be used anymore). This will require another message type, similar
// to msg_gtk_object_ready. Right now, we ignore this ;-)  [kasper]

gint gtk_pageview::delete_event_impl(GdkEventAny *)
	{ 
	debug("closing window");
	hide_all();
	Gtk::Main::instance()->quit();
	return 0;
	}

gint gtk_pageview::key_press_event_impl(GdkEventKey *ev)
	{
	assert(ev!=0);
	switch(ev->keyval) {
		case GDK_Page_Up:
		case GDK_Page_Down:
		case GDK_Up:
		case GDK_Down: {
			Gtk::Box_Helpers::BoxList::iterator it=pagebox_.children().begin();
			while(it!=pagebox_.children().end()) {
				debug("propagating event to gtk_pageview children");
				assert((*it)!=0);
				(*it)->get_widget()->key_press_event(ev);
				++it;
				}
			break;
			}
		default:
			debug("not up/down pup/pdown; call default handler");
			return Gtk::Window::key_press_event_impl(ev);
		}
	debug("no more children");
	return true;
	}

void gtk_pageview::redraw(void)
	{
#ifdef DEBUG
	debugger::instance().lock();
	cerr << "redraw " << lbox_ << endl;
	debugger::instance().unlock();
#endif
	stopwatch sw;
	sw.start();
	lbox_->redraw();
	sw.stop();
#ifdef DEBUG
	debugger::instance().lock();
	cerr << "redraw took " << sw << endl;
	debugger::instance().unlock();	
#endif
	}

pair<string,string> gtk_pageview::popup_authdialog(void)
	{
	authdialog_.set_uposition((int)(gdk_screen_width()*0.45), (int)(gdk_screen_height()*0.45));
	authdialog_.show();
	authdialog_.wait();
	return pair<string, string>(authdialog_.user(), authdialog_.password());
	}

void gtk_pageview::show_status_history(void)
	{
	status_mutex_.lock();
	cout << "Status history:" << endl;
	for(unsigned int i=0; i<status2_history_.size(); ++i) 
		cout << status2_history_[i] << endl;
	cout << endl;
	status_mutex_.unlock();
	}

void gtk_pageview::show_parser_errors(void)
	{
//	status_mutex_.lock();
	cout << "parser messages window should pop up" << endl;
//	status_mutex_.unlock();
	}

browser::pageview_manager *gtk_pageview::owner(void) const
	{
	return owner_;
	}

void gtk_pageview::set_status_display(const string& ur)
	{
	status_mutex_.lock();
	if(status2_text_!=ur) {
		status_erase_delay_=150;
		status2_text_=ur;
		status2_history_.push_back(ur);
		}
	status_mutex_.unlock();
	}

void gtk_pageview::set_url_display(const string& ur)
	{
	status_mutex_.lock();
	statbar_.pop(context_);
	statbar_.push(context_,ur);
	status_mutex_.unlock();
	}

void gtk_pageview::set_pageurl_display(const string& ur)
	{
	debug("setting pageurl display");
	urlbar_.url_entry_.set_text(ur);
	}

void gtk_pageview::set_broken_html(bool onoff)
	{
	html_is_broken_=onoff;
	}

void gtk_pageview::set_backward_sensitive(bool onoff)
	{
	navbar_.back_widget_.set_sensitive(onoff);
	}

void gtk_pageview::set_forward_sensitive(bool onoff)
	{
	navbar_.forward_widget_.set_sensitive(onoff);
	}

void gtk_pageview::set_connection_status(connection_status onoff)
	{
	spinner_mutex_.lock();
	spinner_request_=onoff;
	spinner_mutex_.unlock();
	}

void gtk_pageview::quit(const char *what) 
   {
   hide();
   Gtk::Main::instance()->quit();
   }

gint gtk_pageview::periodic(void)
	{
	if(getenv("MN_PRINT_MEMORY")) {
		pid_t mypid=getpid();
		char prname[300];
		sprintf(prname,"/proc/%d/stat",mypid);
		FILE *pr=fopen(prname,"r");
		if(pr) {
			fgets(prname,299,pr);
			unsigned vsize, rss;
			sscanf(prname,
					 "%*d %*s %*c %*d %*d %*d %*d %*d %*u %*u %*u %*u %*u %*d %*d %*d %*d %*d %*d %*u %*u %*d %u %u",
					 &vsize, &rss);
#ifdef DEBUG
			debugger::instance().lock();
			cerr << vsize << "," << rss << endl;
			debugger::instance().unlock();
#endif
			fclose(pr);
			}
		}
	spinner_mutex_.lock();
	if(html_actual_!=html_is_broken_) {
		brokenhtml_button_.set_sensitive(html_is_broken_);
		html_actual_=html_is_broken_;
		}

	if(spinner_request_ != spinner_actual_) {
		spinner_actual_=spinner_request_;
		}

	status_mutex_.lock();
	if(status_erase_delay_==1) {
		status2_text_="";
		status2_history_.push_back("");
		}

	if(status_erase_delay_==150 || status_erase_delay_==1) {
		statbar2_.pop(context2_);
		statbar2_.push(context2_,status2_text_);
		}
	
	if(spinner_actual_==no_connections) {
		if(status_erase_delay_!=0) 
			--status_erase_delay_;
		}
	else {
		if(status_erase_delay_==150) 
			--status_erase_delay_; // to prevent flicker
		}

	status_mutex_.unlock();
	spinner_mutex_.unlock();
	return 1;
	}

void gtk_pageview::animate_spinner(void)
	{
	if(!is_drawable()) return;
	bool drawit=false;
	Gdk_Window spinner_window_=spinner_area_.get_window();

	if(owner_->spinner_animation_size()==0)
		owner_->initialise_spinner_pixmaps(spinner_window_);

	if(spinner_actual_==active_connections || spinner_actual_==attempted_connections) {
		++spinner_index_;
		if(spinner_index_==owner_->spinner_animation_size())
			spinner_index_=0;
		drawit=true;
		}
	else if(spinner_index_!=0) {
		spinner_index_=0;
		drawit=true;
		}
	if(drawit) {
		Gdk_GC     spinner_gc_(spinner_window_);
		Gdk_Pixmap tmp=owner_->spinner_pixmaps(spinner_index_);
		spinner_window_.set_back_pixmap(tmp,false);
		spinner_window_.clear_area(0,0,47,47);
		}
	}


