// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Main browser window.

  Copyright 1998-1999 Kasper Peeters and Preben Randhol.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef clientwindow_hh__
#define clientwindow_hh__

#include <gtk--/handlebox.h>
#include <gtk--/toolbar.h>
#include <gtk--/button.h>
#include <gtk--/window.h>
#include <gtk--/dialog.h>
#include <gtk--/notebook.h>
#include <gtk--/drawingarea.h>
#include <gtk--/label.h>
#include <gtk--/entry.h>
#include <gtk--/box.h>
#include <gtk--/statusbar.h>
#include <gtk--/accelgroup.h>
#include <gtk--/menu.h>
#include <gtk--/menubar.h>
#include <gtk--/menushell.h>
#include <gtk--/object.h>
#include <gtk--/main.h>
#include <gtk--/pixmap.h>
#include <gdk--/pixmap.h>

#include <mnemonic/dlobject.hh>
#include <mnemonic/thread_mutex.hh>
#include <msg/gtkdisplay/object_ready.hh>

#include "browser.hh"

 class gtk_pageview;
 
 class gtk_pageview_manager : public dlobject, public browser::pageview_manager {
	 public:
		 gtk_pageview_manager(void);
		 virtual ~gtk_pageview_manager();
		 
		 virtual void first_window_(void);
		 virtual void open_window_(void);
		 virtual void update_animations(void);

		 void         initialise_spinner_pixmaps(Gdk_Window&);
		 unsigned int spinner_animation_size(void) const;
		 Gdk_Pixmap   spinner_pixmaps(unsigned int) const;
	 private:
		 static gtk_pageview_manager *pm_;
		 static void timer_handler_(int);
		 static void update_animations_(void *arg, int fd, GdkInputCondition cond);
		 int         fd_[2];
		 
		 raster_io              spinner_rio_;
		 vector<Gdk_Pixmap>     spinner_animation_;
		 vector<gtk_pageview *> windows_;
		 thread_mutex           windows_mutex_;
		 bool                   kiosk_;
 };

class navigationbar : public Gtk::HandleBox {
   public:
      navigationbar(gtk_pageview *);
		Gtk::Button  back_widget_, forward_widget_;
   private:
      Gtk::Toolbar   toolbar_;
		gtk_pageview *owner_;

      void back(const char *);
};

class menubar : public Gtk::HBox {
   public:
      menubar(gtk_pageview *);
   private:
		gtk_pageview  *owner_;
      Gtk::MenuBar   menu_;

      void back(const char *);
};


 class urlbar : public Gtk::HandleBox {
	 public:
		 urlbar(gtk_pageview *);
		 class urlbox : public Gtk::Entry {
			 public:
				 urlbox(gtk_pageview *);
				 void         set_url(const string&);
				 virtual void changed_impl();
				 virtual void activate_impl();
//				 virtual gint key_press_event_impl(GdkEventKey *);
			 private:
				 gtk_pageview *owner_;
		 };
		 
		 urlbox      url_entry_;
	 private:
		 Gtk::HBox    hbox_;
		 Gtk::Label   url_label_;
		 Gtk::Button  url_goto_;
 };

 class authdialog : public Gtk::Dialog {
	 public:
		 authdialog(gtk_pageview *);

		 string user(void) const;
		 string password(void) const;
		 void   wait(void);  // for dialog to close
	 protected:
		 void authdialog_closed_ok(void); // this is a callback
		 void authdialog_closed_cancel(void); // this is a callback

	 private:
		 thread_mutex        authdialog_mutex_; // locked while dialog is open

		 gtk_pageview *owner_;

		 Gtk::HBox      name_box_;
		 Gtk::Label     name_label_;
		 Gtk::Entry     name_entry_;
		 Gtk::HBox      password_box_;
		 Gtk::Label     password_label_;
		 Gtk::Entry     password_entry_;
		 Gtk::Button    ok_;
		 Gtk::Button    cancel_;
		 
		 string        user_;
		 string        password_;
 };

 class gtk_pageview : public Gtk::Window, public browser::pageview {
	 public:
		 gtk_pageview(gtk_pageview_manager *owner, const char *starturl, GtkWindowType t=GTK_WINDOW_TOPLEVEL);
		 virtual ~gtk_pageview();

		 Gtk::VBox      pagebox_; 
		 Gtk::Widget   *new_widget;
		 
		 untyped_interest::callresult accept_gtk_object_ready(msg_gtk_object_ready &);

		 pair<string,string> popup_authdialog(void);
		 void                show_status_history(void); // this is a callback
		 void                show_parser_errors(void); // this is a callback

		 void quit(const char *); 
		 gint periodic(void);
		 void animate_spinner(void);

		 virtual browser::pageview_manager *owner(void) const;

		 void redraw(void); // for profiling purposes.
	 protected:
		 virtual void set_status_display(const string&);
		 virtual void set_url_display(const string&);
		 virtual void set_pageurl_display(const string&);
		 virtual void set_broken_html(bool);
		 virtual void set_forward_sensitive(bool);
		 virtual void set_backward_sensitive(bool);
		 virtual void set_connection_status(connection_status);
		 virtual pair<string,string> handle_user_authentication(void);
		 virtual void register_connection_interests(const group_identifiers&);

		 virtual gint delete_event_impl(GdkEventAny *);
		 virtual gint key_press_event_impl(GdkEventKey *);
		 virtual void realize_impl(void);
//		 virtual gint expose_event_impl(GdkEventExpose *);

	 private:
		 void setup_interests_(void);
		 void destroy_interests_(void);

		 GtkWindowType  wtype_;
		 Gtk::Widget    *old_widget_;
		 Gtk::Notebook   notebook_;
		 Gtk::Label      label_;
		 navigationbar  navbar_;
		 menubar        menubar_;
		 urlbar         urlbar_;
 
		 Gtk::Statusbar  statbar_, statbar2_;
		 Gtk::HBox       statbar2_box_;
		 Gtk::Button     brokenhtml_button_;
		 bool           html_is_broken_;
		 bool           html_actual_;
		 Gtk::Button     statbar2_button_;
		 string         status2_text_;
		 int            status_erase_delay_;
		 thread_mutex   status_mutex_;
		 
		 connection_status  spinner_request_;
		 connection_status  spinner_actual_;
		 thread_mutex       spinner_mutex_;
		 Gtk::Button        spinner_button_;
		 Gtk::DrawingArea   spinner_area_;
		 unsigned int       spinner_index_;

		 Gtk::VBox       vbox_, vbox2_;
		 Gtk::HBox       hbox_;
		 authdialog     authdialog_;
		 guint          context_;
		 guint          context2_;
		 string         starturl_;
		 gtk_pageview_manager        *owner_;
		 vector<string> status2_history_;

		 layout::box   *lbox_;

		 ptr<interest<gtk_pageview, msg_gtk_object_ready> >        interest_gtk_object_ready_;
 };
 


#endif

