// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Gtk client message interface.

  Copyright 1998-1999 Kasper Peeters.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <unistd.h>
#include <fcntl.h>

#include "gtk_browser.hh"
#include <url/url_string.hh>

#define DEBUG_CLASS "GTKBROWSER_MIO"
#include <mnemonic/debug.hh>

void gtk_pageview::setup_interests_(void) 
	{
	interest_gtk_object_ready_=interest<gtk_pageview, msg_gtk_object_ready>::
		create(this, &gtk_pageview::accept_gtk_object_ready,untyped_interest::lowest);
	}

void gtk_pageview::destroy_interests_(void)
	{
	interest_gtk_object_ready_->unregister();
	interest_gtk_object_ready_.wait();
	}

pair<string, string> gtk_pageview::handle_user_authentication(void)
	{
	debug("user authentication dialog requested.");
	// gtk locking is done by popup_authdialog.
	return popup_authdialog();
	}

void gtk_pageview::register_connection_interests(const group_identifiers& gid)
	{
	debug("registering interests for gtk_object_ready, " << gid);
	interest_gtk_object_ready_->set_group_ids(gid);
	interest_gtk_object_ready_->acceptance=untyped_interest::mid_contains_iid;
	pageview::register_connection_interests(*interest_gtk_object_ready_);
	}

untyped_interest::callresult gtk_pageview::accept_gtk_object_ready(msg_gtk_object_ready& msg)
	{
	debug("request to hook into our widget tree");
	debug("passing widget pointer");

	msg.widget=&(pagebox_);
	msg.layoutbox=0;
	debug("registering link_clicked interests");
	interest_link_clicked_->set_group_ids(geturl_ids_);

	debug("updating history and setting urlbar");
	Gtk::Box_Helpers::BoxList::iterator it=pagebox_.children().begin();
	if(it!=pagebox_.children().end()) {
		debug("removing a widget");
		Gtk::Widget *tmp=(*it)->get_widget();
		tmp->hide_all();
		pagebox_.remove(*tmp);
//		delete tmp;
		}
	debug("deleting lbox");
	if(lbox_) {
		delete lbox_;
		debug("box deleted");
		}
	debug("creating new lbox");
	lbox_=msg.new_layoutbox;

	// FIXME: we may want to put this in the base class `browser'.
	debug("manipulating history");
	url_string theurl=find_connection_(msg);
	debug("url: " << theurl.get_url());
	set_pageurl_display(theurl.get_url());
	if(hbuf_.history_pos_==hbuf_.history_.end() ||
		(*hbuf_.history_pos_)!=theurl.get_url()) {
		if(hbuf_.history_pos_!=hbuf_.history_.end()) {
			++hbuf_.history_pos_;
			if(hbuf_.history_pos_!=hbuf_.history_.end())
				hbuf_.history_.erase(hbuf_.history_pos_,hbuf_.history_.end());
			}
		hbuf_.history_.push_back(theurl.get_url());
		hbuf_.history_pos_=hbuf_.history_.end();
		if(hbuf_.history_.size()==1) set_backward_sensitive(false);
		else                         set_backward_sensitive(true);
		set_forward_sensitive(false);
		--hbuf_.history_pos_;
		}
	debug("event loop restarted");
	
	return untyped_interest::exit_loop;
	}

template interest<gtk_pageview, msg_gtk_object_ready>;


