// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  GTK based rendering unit (one for each document).

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "gtkrenderer.hh"
#include <strstream.h>
#include <msg/display/link_clicked.hh>
#include <msg/dom/css_event.hh>
#include <gtklayout/gtkcolors.hh>
#include <gtklayout/gtkfonts.hh>

#define DEBUG_CLASS "GTKRENDERER"
#include <mnemonic/debug.hh>

gtkw3renderer::gtkw3renderer(const group_identifiers& ids, ptr<CSS::stylesheet> sheet,
									  DOM::Node *startnode)
	: w3renderer(sheet, startnode, new gtkfactory), 
	  canvas_added_(false), toplevel_outer_(0), toplevel_inner_(0), my_ids_(ids)
	{
	setup_interests_(ids);
	}

gtkw3renderer::testwidget::testwidget(gtkw3renderer *own)
	: owner_(own)
	{
	debug("creating testwidget");
	}

gtkw3renderer::testwidget::~testwidget()
	{ 
	debug("~testwidget");
	// The gtkrenderer has to be notified of this, so that it no longer
	// updates this widget. Lower layers (parser, network) will pick up this
	// change by virtue of the fact that gtkdisplayhandler no longer listens
	// to msg_dom_mutation_event messages.
	}

int gtkw3renderer::testwidget::propagate_event(layout::point off, event_info& inf, layout::box *fe)
	{
	int ret=layout::hbox::propagate_event(off,inf,fe);
	owner_->handle_mouse_event(inf, ret);
	return ret;
	}

void gtkw3renderer::handle_mouse_event(layout::box::event_info& inf, int ret)
	{
	if(ret==10) {
		debug("event accepted, resolving");
		DOM::Node *acnode=active_boxes_[inf.hovering_box];
 		DOM::Element *el=dynamic_cast<DOM::Element *>(acnode);
 		if(el && el->getAttribute("HREF")!="") { 
			url_string ur(el->getAttribute("HREF").c_str(), url_string(sheet_->document()->url().c_str()));
			int code=0;
			if(inf.etype==layout::box::event_info::button_up) code=1;
			msg_link_clicked msg(ur.get_url(),code);
			msg.set_group_ids(my_ids_);
			try {
				msg.send();
				}
			catch(messagebase_error& ex) {
				debug(ex);
				}
			}
		}
	else {
		debug("sending empty msg_link_clicked");
		msg_link_clicked msg("",0);
		msg.set_group_ids(my_ids_);
		gdk_window_set_cursor(toplevel_outer_->scanvas().canvas()->window(),0);
		inf.hovering_box=0;
		inf.accepting_box=0;
		try {
			msg.send();
			}
		catch(messagebase_error&ex) {
			debug(ex);
			}
		}
	}

bool gtkw3renderer::build_gtktree_(const vector<DOM::Node *>& new_nodes)
	{
	return dom_to_layout(new_nodes);
	}

int gtkw3renderer::widget_deleted(GdkEventAny *)
	{
	debug("WIDGET DELETED");
	return 0;
	}
 
