// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  GTK based rendering unit (one for each document).

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef gtkrenderer_hh__
#define gtkrenderer_hh__

#include <gdk--.h>
#include <gtk--/widget.h>

#include <msg/dom/dom_event.hh>
#include <dom/css.hh>

#include <mnemonic/ptr.hh>
#include <w3render/w3render.hh>
#include <gtklayout/gtkboxes.hh>
#include <msg/gtkdisplay/object_ready.hh>

#include <multimap.h>

class gtkw3renderer : public w3renderer {
	public:
		gtkw3renderer(const group_identifiers&, ptr<CSS::stylesheet>, DOM::Node *);
		virtual ~gtkw3renderer();
		
		int widget_deleted(GdkEventAny *);

		virtual external_status handle_externals_impl(treeinfo&);

		class testwidget : public gtkviewer {
			public:
				testwidget(gtkw3renderer *);
				virtual ~testwidget();
				virtual int propagate_event(layout::point, event_info &, layout::box *forced_end);
			private:
				gtkw3renderer *owner_;
		};

		untyped_interest::callresult gtkw3renderer::accept_dom_mutation_event(msg_dom_mutation_event & msg);
		untyped_interest::callresult accept_gtk_object_ready(msg_gtk_object_ready &);

		void handle_mouse_event(layout::box::event_info& inf, int);
	private:		
		void setup_interests_(const group_identifiers& );

		msg_gtk_object_ready msg2; 
		ptr<interest<gtkw3renderer, msg_gtk_object_ready> >   interest_gtk_object_ready_;
		ptr<interest<gtkw3renderer, msg_dom_mutation_event> > interest_dom_mutation_event_;

		// FIXME: we need to keep this pointer between sending a message requesting
		// a helper and filling in the msg_gtk_object_ready message to tell the
		// helper which box to use. Should probably be a map between midentifier and
		// gtkcontainer_box, but more importantly, this is something that will occur more often
		// and therefore should probably move to a base library for oils that are
		// based on lib-gtklayout.
		layout::container_box *container_for_helperbox;
		
		bool                   canvas_added_;
		testwidget            *toplevel_outer_;
		layout::container_box *toplevel_inner_;
		layout::area          *toplevel_area_; // need to keep a pointer for locking
		group_identifiers      my_ids_;

		bool     build_gtktree_(const vector<DOM::Node *>&); // true if relayout needed
};


#endif
