// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  GTK based rendering unit (one for each document).

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "gtkrenderer.hh"
#include <msg/gtkdisplay/object_ready.hh>

#include <unistd.h>
#include <dom/dom.hh>
#include <dom/css.hh>
#include <msg/dom/css_event.hh>

#define DEBUG_CLASS "GTKRENDERER_MIO"
#include <mnemonic/debug.hh>
#include <mnemonic/stopwatch.hh>

#include <gtk--/button.h>

void gtkw3renderer::setup_interests_(const group_identifiers& gids)
	{
	interest_gtk_object_ready_=interest<gtkw3renderer, msg_gtk_object_ready>::create(this, &gtkw3renderer::accept_gtk_object_ready,untyped_interest::lowest);
	interest_gtk_object_ready_->acceptance=untyped_interest::iid_contains_mid;
	interest_dom_mutation_event_=interest<gtkw3renderer, msg_dom_mutation_event>::create(this, &gtkw3renderer::accept_dom_mutation_event,untyped_interest::lowest);
	interest_dom_mutation_event_->acceptance=untyped_interest::iid_contains_mid;

	interest_dom_mutation_event_->set_group_ids(gids);
	}

gtkw3renderer::~gtkw3renderer()
	{
	if(toplevel_outer_!=0) {
		debug("someone signalled me to go away");

		interest_gtk_object_ready_->unregister();
		interest_dom_mutation_event_->unregister();
		interest_gtk_object_ready_.wait();
		interest_dom_mutation_event_.wait();
		debug("detached from message core");

//		debug("locking client once more to remove widget");
//		msg2.elc->interrupt_event_loop();

		// This is no longer our task, the client handles widget delete
//		toplevel_widget_->scanvas().hide();
//		msg2.widget->remove(toplevel_widget_->scanvas());
//		Threadsafe_Main::instance()->interrupt_event_loop();
//		delete toplevel_outer_;
//		Threadsafe_Main::instance()->restart_event_loop();

//		debug("releasing client forever");
//		msg2.elc->restart_event_loop();
		debug("bye bye");
		}
	else {
		debug("deleted, but we didn't even have a widget open yet");
		}
	}

untyped_interest::callresult gtkw3renderer::accept_dom_mutation_event(msg_dom_mutation_event & msg)
	{
	debug("Document was modified, updating display");
	if(msg.targets.size()==0) {
		debug("no new targets?");
		return untyped_interest::call_next;
		}

	if(!canvas_added_) {
		msg2.set_group_ids(msg);
//		my_ids_=msg;

		// testwidget is not derived from Gtk:: so we don't have to thread_enter.
		toplevel_outer_=new testwidget(this);
		toplevel_area_=toplevel_outer_->current_area();
		
		msg2.new_layoutbox=toplevel_outer_;
		debug("TRYING to announce the widget, so go into gdk thread lock");
		gdk_threads_enter();
		try {
			msg2.send();
			}
		catch(messagebase_error &ex) {
			debug(ex);
			gdk_threads_leave();
			return untyped_interest::pass_on; 
			}
		
		if(msg2.widget==0) {
			debug("Uh oh, no widget returned");
			gdk_threads_leave();
			return untyped_interest::pass_on;
			}
			
//		debug("our ids are " << my_ids_);
//		toplevel_widget_->ids=my_ids_;
		debug("widget created");
		assert(msg2.widget);
		msg2.widget->add(toplevel_outer_->scanvas());
		gdk_threads_leave();

		debug("widget added");
		toplevel_area_->lock();
		toplevel_outer_->set_properties(
			new layout::property_box(0,
											 toplevel_outer_->font_mgr().get_font(
												 layout::xlfd("helvetica", 
																  layout::xlfd::medium, 
																  layout::xlfd::upright, 
																  layout::length("10pt"))),
											 toplevel_outer_->color_mgr().get_color("black"),
											 toplevel_outer_->color_mgr().get_color("#c0c0c0")));

		toplevel_inner_=new gtkvbox;
		toplevel_outer_->add_child(toplevel_inner_);
		set_toplevel(toplevel_outer_,toplevel_inner_);
		gdk_threads_enter();
		toplevel_outer_->scanvas().get_parent()->show_all();
		toplevel_outer_->scanvas().canvas()->grab_focus();
		toplevel_area_->unlock();
		toplevel_outer_->scanvas().canvas()->relayout_all();
		gdk_threads_leave();
		((layout::box *)toplevel_outer_)->redraw();
		toplevel_area_->lock();
		
		canvas_added_=true;
		stopwatch sw;
		sw.start();
		if(build_gtktree_(msg.targets)) {
			sw.stop();
			debug("building: " << sw);
			sw.reset();
			sw.start();
			toplevel_area_->unlock();
			gdk_threads_enter();
			toplevel_outer_->reflow();
			gdk_threads_leave();
			sw.stop();
			debug("relayout: " << sw);
			sw.reset();
			sw.start();
			((layout::box *)toplevel_outer_)->redraw();
			sw.stop();
			debug("redraw  : " << sw);
			}
		else toplevel_area_->unlock();
		}
	else {
		debug("building tree");
		stopwatch sw;
		sw.start();
		toplevel_area_->lock();
		if(build_gtktree_(msg.targets)) {
			sw.stop();
			debug("building: " << sw);
			sw.reset();
			sw.start();
			toplevel_area_->unlock();
			gdk_threads_enter();
			toplevel_outer_->reflow();
			gdk_threads_leave();
			sw.stop();
			debug("relayout: " << sw);
			sw.reset();
			sw.start();
			((layout::box *)toplevel_outer_)->redraw();
			sw.stop();
			debug("redraw  : " << sw);
			}
		else toplevel_area_->unlock();
		}
	
	
	debug("releasing client");
	return untyped_interest::call_next;
	}

untyped_interest::callresult gtkw3renderer::accept_gtk_object_ready(msg_gtk_object_ready & msg)
	{
	debug("received gtkboxes object from helper handler");
	msg.layoutbox=container_for_helperbox;
	msg.widget=0;
	return untyped_interest::exit_loop;
	}

w3renderer::external_status gtkw3renderer::handle_externals_impl(treeinfo& inf)
	{
	if(inf.current_dom_node->nodeName()=="IMG") {
		layout::vbox *tmpv=new gtkvbox;
		inf.target->add_child(tmpv);
		container_for_helperbox=tmpv;
		inf.current_empty_boxes.push_back(tmpv);
		msg_css_create_event cssmsg;
		cssmsg.new_css_sheet=sheet_;
		cssmsg.start_here=inf.current_dom_node;
		cssmsg.add_group_id(midentifier());
		cssmsg.add_group_ids(msg2);

//		cerr << "ids for helper request: " << cssmsg << endl;
		interest_gtk_object_ready_->add_group_ids(cssmsg);
		
		// FIXME: this makes the GTK loop interrupted for too long: the
		// message has to be sent, the image module has to pick it up,
		// broadcast the gtk_object_ready message and then fork off an
		// HTTP thread to fetch the image, before it can return here.
		// We should try to move most of that to a later stage. Perhaps
		// only ask for a gtk object and then, after GTK has regained
		// control, send a `go' message to the helper to make it start
		// the network connection. It's a bit ugly though. Maybe we should
		// just improve http thread startup time.
		debug("asking for helper for IMG tag");
		debug("using message with " << cssmsg);
		try {
			cssmsg.send();
			debug("found a helper for IMG tags");
			}
		catch(messagebase_error& ex) {
			debug(ex);
			debug("O well, no images then...");
			interest_gtk_object_ready_->remove_group_ids(cssmsg);
			return failure;
			}
		}
	else return not_applicable;

	return success;
	}

template interest<gtkw3renderer, msg_gtk_object_ready>;
template interest<gtkw3renderer, msg_dom_mutation_event>;
