// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  GTK based display.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#define DEBUG_CLASS "GTKIMAGEDISPLAY"
#include <mnemonic/debug.hh>
#include <unistd.h>

#include <msg/dom/dom_event.hh>
#include <msg/dom/css_event.hh>
#include <msg/gtkdisplay/object_ready.hh>
#include <gtklayout/gtkboxes.hh>
#include <gtk--/button.h>
#include <gtk--.h>

#include "gtkimagedisplay.hh"

gtkimagedisplayhandler::gtkimagedisplayhandler(void)
	{
	interest_css_create_event_=interest<gtkimagedisplayhandler, msg_css_create_event>::create(this, &gtkimagedisplayhandler::accept_css_create_event,untyped_interest::normal);
	interest_css_create_event_->acceptance=untyped_interest::ignore_ids;

	interest_dom_mutation_event_=interest<gtkimagedisplayhandler, msg_dom_mutation_event>::create(this, &gtkimagedisplayhandler::accept_dom_mutation_event,untyped_interest::normal);
	interest_dom_mutation_event_->acceptance=untyped_interest::iid_contains_mid;
	}

gtkimagedisplayhandler::~gtkimagedisplayhandler()
	{
	interest_css_create_event_->unregister();
	interest_dom_mutation_event_->unregister();

	interest_css_create_event_.wait();
	interest_dom_mutation_event_.wait();
	}

untyped_interest::callresult gtkimagedisplayhandler::accept_dom_mutation_event(msg_dom_mutation_event & msg)
	{
	debug("Document was modified, updating display");
	mutex_.lock();
	map<group_identifiers, gtkimagerenderer *>::iterator it;
	if((it=active_renderers_.find(msg))==active_renderers_.end()) {
		debug("no renderer registered for this message");
		mutex_.unlock();
		return untyped_interest::call_next;
		}
// we do not handle these events yet, so let's not upset the compiler ;-)
//	gtkimagerenderer *renderer=(*it).second;
	mutex_.unlock();

	debug("feeding data to renderer");
	return untyped_interest::call_next; //renderer->accept_dom_mutation_event(msg);
	}

untyped_interest::callresult gtkimagedisplayhandler::accept_css_create_event(msg_css_create_event & msg)
	{
	debug("informed about new CSSStyled document");

	// All CSS'ed documents get announced this way.This module accepts
	// the IMG tag.

	if(msg.start_here && msg.start_here->nodeName()=="IMG") {
		if(getenv("MN_DISABLE_IMAGES")) return untyped_interest::pass_on;
		try {
			const DOM::NamespaceDecl *ns=msg.start_here->nodeNamespace();
			if(ns) {
				debug("IMG element found in namespace " << ns->uri());
				}
			else {
				debug("no namespace associated to IMG element");
				}
			}
		catch(exception&) {
			// FIXME: add a default namespace in the parser and don't check here.
			debug("not a DOM tree with namespace info");
			}
		
		// FIXME: as there is currently no msg_dom_destroy_event or something similar,
		// we should make sure that we are never building images for more than
		// one document. But we do not do that yet, which has the nasty side
		// effect that documents with images never get deleted from memory anymore
		// (because we keep a ptr<DOM::Document> as well as a ptr<CSS::stylesheet>).
		// Check for this and delete the renderers when needed.


		// FIXME: when javascript is working we should be registering interest
		// in modifications to this element. Forget about that for now.
		// interest_dom_mutation_event_->add_group_ids(msg);

		add_imagerenderer_(msg,msg.new_css_sheet,msg.start_here);
		
		debug("tree starts with IMG so we can handle it");
		return untyped_interest::exit_loop;
		}
//	else {
		debug("sorry, tree does not start with IMG");
		return untyped_interest::pass_on;
//		}
	}

template interest<gtkimagedisplayhandler, msg_css_create_event>;
template interest<gtkimagedisplayhandler, msg_dom_mutation_event>;
