// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  GTK based rendering unit (one for each document).

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef gtkimagerenderer_hh__
#define gtkimagerenderer_hh__

#include <gdk--.h>
#include <gtk--/widget.h>

#include <msg/dom/dom_event.hh>
#include <dom/css.hh>

#include <mnemonic/ptr.hh>
#include <gtklayout/gtkboxes.hh>
#include <msg/gtkdisplay/object_ready.hh>
#include <msg/network/network_data.hh>
#include <msg/network/network_status.hh>
#include <msg/network/stop.hh>
#include <rasterimage/raster.hh>
#include <url/url_string.hh>

#include <multimap.h>

class gtkimagerenderer {
	public:
		gtkimagerenderer(const group_identifiers&, ptr<CSS::stylesheet>, DOM::Node *);
		virtual ~gtkimagerenderer();
		
		bool announce(const group_identifiers& gid);
		untyped_interest::callresult accept_network_data(msg_network_data &);
		untyped_interest::callresult accept_network_status(msg_network_status &);
		untyped_interest::callresult accept_stop(msg_stop &);

		gtkimagebox         *image_layout_box(void) const;
	private:		
		msg_gtk_object_ready msg2; 
		ptr<interest<gtkimagerenderer, msg_network_data> >    interest_network_data_;
		ptr<interest<gtkimagerenderer, msg_network_status> >  interest_network_status_;
		ptr<interest<gtkimagerenderer, msg_stop> >            interest_stop_;

		raster_io           *rio_;
		gtkimagebox         *toplevel_widget_;
		layout::area        *toplevel_area_; // need to keep a pointer for locking
		int                  forced_width_;
		int                  forced_height_;
		url_string           imgurl_;
		ptr<CSS::stylesheet> sheet_;
		DOM::Node           *start_here_;

		void update_box_(void);
};


#endif
