// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Handler that spawns external helper programs for data streams
  of specific mime types.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "mimehandler.hh"
#define DEBUG_CLASS "HELPERHANDLER"
#include <mnemonic/debug.hh>

helperhandler::helperhandler(void) 
   {
	interest_network_status_=interest<helperhandler, msg_network_status>::create(
		this, &helperhandler::accept_network_status,untyped_interest::lowest);
	interest_network_data_=interest<helperhandler, msg_network_data>::create(
		this, &helperhandler::accept_network_data,untyped_interest::lowest);
   }

helperhandler::~helperhandler()
   {
	interest_network_status_->unregister();
	interest_network_data_->unregister();

	interest_network_status_.wait();
	interest_network_data_.wait();
   }

untyped_interest::callresult helperhandler::accept_network_data(msg_network_data& msg)
   {
   return untyped_interest::call_next;
   }

untyped_interest::callresult helperhandler::accept_network_status(msg_network_status& msg)
   {
	switch(msg.identifier) {
		case msg_network_status::content: {
			// FIXME: we compare the first 9 characters because there can be 
			// additional information in the content header. Eg.
			// text/html; charset=iso-8859-1. 
			// I guess we handle all of these additional things (well, we ignore them).
			// Is that correct?
			if(msg.text.find("application/x-postscript")!=msg.text.npos || 
			   msg.text.find("application/postscript")!=msg.text.npos) {
				debug("received content " << msg.text << " announcement");
				mutex_.lock();
				// 1. register interest
				// 2. create temporary file
				// 3. wait until file is complete
				// 4. spawn helper on temporary file, in thread
				// 5. when it returns, delete temporary file and exit thread.
				mutex_.unlock();
				return untyped_interest::call_next;
				}
			}
		case msg_network_status::object_truncated:
		case msg_network_status::finished: {
			debug("network data flow ended, detaching parser");
			return untyped_interest::call_next;
			}
		default:
			break;
		}
	
	return untyped_interest::pass_on;
   }

extern "C" { void *create_mimehandler(void**); }

void *create_mimehandler(void** ) 
   {
   return new helperhandler();
   }

template interest<helperhandler, msg_network_status>;
template interest<helperhandler, msg_network_data>;

