// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Mnemonic http handler implementation.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <stdio.h>
#include <url/url_string.hh>
#include "http_handler.hh"
#include "http_thread.hh"

#define DEBUG_CLASS "HTTP"
#include <mnemonic/debug.hh>

http_handler::http_handler() {
	debug("http_handler");
	setup_interests();
}

http_handler::~http_handler() 
	{
	debug("~http_handler()");
// wait for all people using us to finish
	destroy_interests();
	
// Now we can no longer receive any requests to
// create any new threads
	me_.lock();
	
// Now, since we have our lock, we are isolated from the outside
// world.  The only active things left that deal with us are
// our child threads, and active_threads_ is uptodate now
	
	for(list<http_thread *>::iterator i = active_threads_.begin();
	    i != active_threads_.end(); i++) {
		(*i)->cancel(); // ask them to cancel
		}
	me_.unlock();
	//we told every one to exit so done_threads_ will not change anymore.
	while(done_threads_.size()>0) {
	    http_thread *t=(*done_threads_.begin());
	    done_threads_.erase(done_threads_.begin());
	    t->stop();
	    delete t;
	}	
	while(active_threads_.size()>0) {
	    http_thread *t=(*active_threads_.begin());
	    active_threads_.erase(active_threads_.begin());
	    t->stop();
	    delete t;
	}	
	
	// All threads have now exited.	
/*	while(reuseable_.begin() != reuseable_.end()) {
		tcp_connection *a = reuseable_.begin()->second;
		reuseable_.erase(reuseable_.begin());
		delete a;
		} */
	debug("~http_handler() finished");
	}

void http_handler::donate_to_pool(string hostname, tcp_connection *t) 
	{
	debug("donate_to_pool");
// FIXME: reusables do not work at all and the logic is twisted anyhow.
// re-implement. For now, we just ignore this stuff.
	delete t;
	
/*	me_.lock();
	map<string, tcp_connection *>::iterator it=reuseable_.find(hostname);
	if(it!=reuseable_.end()) {
		delete (*it).second;
		reuseable_.erase(it);
		}
	reuseable_[hostname] = t;
	me_.unlock();*/
	}
	
//this is called from the thread to make clear it is done
void http_handler::thread_exit(http_thread *t) 
	{
	debug("thread_exit");
	me_.lock();
	debug("Locked me_ in thread_exit");
	for(list<http_thread *>::iterator i = active_threads_.begin();
	    i != active_threads_.end(); i++) {
		debug("loop");
		if( (*i) == t ) {
			debug("erasing a thread...");
			active_threads_.erase(i);
			debug("done erasing a thread...");
			break;
			}
		}
	done_threads_.push_back(t); 
	debug("unlocking me");
	me_.unlock();
	debug("signalling exit_thread");
	}

extern "C" 
	{
	dlobject *create_http_handler(void**);
	}

dlobject *create_http_handler(void** /* Args */) 
	{
	return new http_handler();
	}
