// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Mnemonic http handler interface.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef http_handler_hh__
#define http_handler_hh__

#include <string>
#include <map.h>
#include <mnemonic/thread.hh>
#include <mnemonic/thread_mutex.hh>
#include <mnemonic/dlobject.hh>
#include <mnemonic/midentifier.hh>
#include <msg/network/get_url.hh>
#include <msg/network/post_url.hh>
#include <msg/network/put_url.hh>
#include <msg/network/network_status.hh>
#include <msg/network/network_data.hh>
#include <tcpcon/tcp_connection.hh>

class http_thread;

class http_handler : public dlobject {
	public:
		http_handler(void);
		~http_handler();
		
		untyped_interest::callresult	accept_get_url(msg_get_url &);
		untyped_interest::callresult	accept_post_url(msg_post_url &);
		untyped_interest::callresult	accept_put_url(msg_put_url &);

		void donate_to_pool(string hostname, tcp_connection *); //grabs a thread context
		void thread_exit(http_thread *);
	private:
		ptr<interest<http_handler,msg_get_url>	>	interest_get_url_;
		ptr<interest<http_handler,msg_post_url> >	interest_post_url_;
		ptr<interest<http_handler,msg_put_url> >	interest_put_url_;

		list<http_thread *> active_threads_; //locked by me_
		list<http_thread *> done_threads_; //locked by me_
//		map<string, tcp_connection *> reuseable_; //locked by me_ <hostname, tcp_conn>
		thread_mutex me_;
		
		void setup_interests();
		void destroy_interests();
};

#endif
