// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Mnemonic http handler implementation.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <url/url_string.hh>
#include "http_handler.hh"
#include "http_url.hh"
#include "http_thread.hh"

#define DEBUG_CLASS "HTTP"
#include <mnemonic/debug.hh>


void http_handler::setup_interests(void) 
	{
	interest_get_url_ = interest<http_handler, msg_get_url>::create(this,&http_handler::accept_get_url,untyped_interest::normal);
	interest_post_url_ = interest<http_handler, msg_post_url>::create(this,&http_handler::accept_post_url,untyped_interest::normal);
	interest_put_url_ = interest<http_handler, msg_put_url>::create(this,&http_handler::accept_put_url,untyped_interest::normal);

	debug("setup_interests done");
	}

void http_handler::destroy_interests() 
	{
	interest_get_url_->unregister();
	interest_post_url_->unregister();
	interest_put_url_->unregister();
	interest_get_url_.wait();
	interest_post_url_.wait();
	interest_put_url_.wait();
	}

untyped_interest::callresult http_handler::accept_get_url(msg_get_url &m) 
	{
	debug("accept_get_url");
	me_.lock();
	debug("locked me_");
	http_url hurl(m.url); // provides all the stuff we need

	if (hurl.get_scheme()!="http") {
  	me_.unlock();
	  return untyped_interest::refused;
	}
	debug("Made an http_url out of the url : " << hurl.get_url());
	tcp_connection *t = 0; //reuseable_[hurl.get_net_path()];
	debug("Checked for reuseable connection \n");
	if(t) {
		// FIXME: what's the logic here? [kasper]
		debug("Found a reuseable connection");
		if(t->get_port() == hurl.get_port()) {
//			reuseable_.erase(hurl.get_net_path()); // not reuseable anymore :)
			}
		} 
	else {
		debug("No reuseable connection");
		if(getenv("HTTP_PROXY")) {
			url_string proxy(getenv("HTTP_PROXY"));
			t=new tcp_connection(hurl.get_net_path(),hurl.get_port(),
										proxy.get_net_path(),proxy.get_port());
			}
		else {
			debug("port: "<<hurl.get_port());
			t = new tcp_connection(hurl.get_net_path(),hurl.get_port());
			}
		}
	try {
		active_threads_.push_back(new http_thread(hurl, m.no_cache, this, m, t));
		debug("removing old threads");
		while(done_threads_.size()>0) {
			debug("deleting a finished thread entirely");
			http_thread *t=(*done_threads_.begin());
			done_threads_.erase(done_threads_.begin());
			t->stop();
			delete t;
			}	
		debug("About to unlock me_");
		me_.unlock();
		}
	catch(exception& er) {
		me_.unlock();
		cerr << "exception: " << er.what() << endl;
		throw;
		}

 	return untyped_interest::call_next;
	}

untyped_interest::callresult http_handler::accept_post_url(msg_post_url &) 
	{
	return untyped_interest::refused;
	}

untyped_interest::callresult http_handler::accept_put_url(msg_put_url &)
	{
	return untyped_interest::refused;
	}

template interest<http_handler, msg_get_url>;
template interest<http_handler, msg_put_url>;
template interest<http_handler, msg_post_url>;
