// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Mnemonic http thread (does all the work)

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef http_thread_hh_
#define http_thread_hh_

#include "http_handler.hh"
#include <tcpcon/tcp_connection.hh>
#include <mnemonic/thread.hh>
#include <mnemonic/midentifier.hh>
#include <msg/network/network_status.hh>
#include <msg/network/stop.hh>
#include "http_url.hh"

#define BUFFER_LENGTH 4096

class http_thread : public thread {
	public:
		http_thread(const http_url &, bool no_cache, http_handler *, const group_identifiers &, 
						tcp_connection *t);
		~http_thread();
		void begin();
		void cancel(void);
		untyped_interest::callresult    accept_stop(msg_stop &);
	private:
		group_identifiers ids_;
		tcp_connection *tc_;
		http_handler *hh_;
		http_url url_;
		bool     no_cache_;
		int response_code_;
		int have_length_;
		char buf[BUFFER_LENGTH];

		void send_status_(msg_network_status::status, const char *);
		void send_status_(msg_network_status::status);
		int read_(char *, int);

		int connect_();
		int send_request_();
		int read_response_();
		int check_header_();
		int data_flow_();

		int should_exit_;

		void escape_(const string &url, string &res);

		map<string,string>                headers;

		ptr<interest<http_thread,msg_stop> >  interest_stop_;
		void setup_interests_();
		void remove_interests_();
};
#endif
