// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  GTK based display.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#define DEBUG_CLASS "GTKDISPLAY"
#include <mnemonic/debug.hh>
#include <unistd.h>

#include <msg/dom/dom_event.hh>
#include <msg/gtkdisplay/object_ready.hh>
#include <gtklayout/gtkboxes.hh>
#include <gtk--/button.h>
#include <gtk--.h>

#include "gtkdisplay.hh"

gtkdisplayhandler::gtkdisplayhandler(void)
	{
	interest_dom_create_event_=interest<gtkdisplayhandler, msg_dom_create_event>::create(this, &gtkdisplayhandler::accept_dom_create_event,untyped_interest::lowest);
	interest_dom_create_event_->only_matching_groups=false;

	interest_dom_mutation_event_=interest<gtkdisplayhandler, msg_dom_mutation_event>::create(this, &gtkdisplayhandler::accept_dom_mutation_event,untyped_interest::lowest);
	interest_dom_mutation_event_->only_matching_groups=true;
	}

gtkdisplayhandler::~gtkdisplayhandler()
	{
	interest_dom_create_event_->unregister();
	interest_dom_mutation_event_->unregister();

	interest_dom_create_event_.wait();
	interest_dom_mutation_event_.wait();
	}

untyped_interest::callresult gtkdisplayhandler::accept_dom_mutation_event(msg_dom_mutation_event & msg)
	{
	debug("Document was modified, updating display");
	mutex_.lock();
	map<group_identifiers, gtkw3renderer *>::iterator it;
	if((it=active_renderers_.find(msg))==active_renderers_.end()) {
		debug("no renderer registered for this message");
		return untyped_interest::call_next;
		}
	gtkw3renderer *renderer=(*it).second;
	mutex_.unlock();

	debug("feeding data to renderer");
	return renderer->accept_dom_mutation_event(msg);
	}

untyped_interest::callresult gtkdisplayhandler::accept_dom_create_event(msg_dom_create_event & msg)
	{
	debug("informed about new DOM");

	// register our interest in mutation events for this document
	interest_dom_mutation_event_->set_group_ids(msg);

	// FIXME: we should only register a renderer if the DTD of the DOM
	// structure is html or something else we can display. It might for instance be
	// one of type XBEL (http://www.python.org/topics/xml/xbel/docs/html/xbel.html)
	// which should be handled by a bookmark handler, not by us.
	// So this add_w3renderer_ has to move to accept_dom_mutation_event, eventually.

	add_w3renderer_(msg,msg.new_dom_document);

	return untyped_interest::call_next;
	}

template interest<gtkdisplayhandler, msg_dom_create_event>;
template interest<gtkdisplayhandler, msg_dom_mutation_event>;
