// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Raw data to XML DOM-like tree converter.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define DEBUG_CLASS "XMLHANDLER"
#include <mnemonic/debug.hh>

#include "xmlhandler.hh"

void xmlhandler::add_parser_(group_identifiers *gid, ptr<DOM::Document> docptr)
	{
	debug("adding parser");
	debug("for group_identifiers " << *gid);
	xmlparser *fik=new xmlparser(docptr);
	debug("parser created");
	active_parsers_[*gid]=fik;
	debug("ok");
	}

void xmlhandler::detach_parser_(group_identifiers *gid)
	{
	debug("detaching parser for group_identifiers " << *gid);
	mutex_.lock();
	delete active_parsers_[*gid];
	active_parsers_.erase(*gid);
	mutex_.unlock();
	}

void xmlhandler::handle_datamess_(group_identifiers *gid, int length, const char *data,
											 msg_dom_mutation_event& msg)
	{
	debug("received chunk of " << length << " bytes of data for");
	debug(*gid);
	mutex_.lock();
	map<group_identifiers, xmlparser *>::iterator it;
	if((it=active_parsers_.find(*gid))==active_parsers_.end()) {
		debug("no parser registered for this message");
		mutex_.unlock();
		return;
		}
	xmlparser *parser=(*it).second;
	mutex_.unlock();

	debug("feeding data to parser");
	assert(parser!=0);
	parser->append_data(length, data);
	xmlparser::parse_result res;
	do {
		res=parser->parse_buffer(msg.targets);
		if(res==xmlparser::DOCTYPE_FOUND)
			debug("document type = " << parser->document->doctype()->nodeName() );
		} while(res!=xmlparser::DOCUMENT_END && res!=xmlparser::OUT_OF_DATA);

	msg.document=parser->document;
	}


template map<group_identifiers, xmlparser *>;
