// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Message wrapper around raw data to XML DOM-like tree converter.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef xmlhandler_hh__
#define xmlhandler_hh__

#include <mnemonic/dlobject.hh>
#include <msg/network/network_status.hh>
#include <msg/network/network_data.hh>
#include <msg/network/stop.hh>

#include <dom/dom.hh>
#include <msg/dom/dom_event.hh>

#include "xmlparser.hh"

#include <map.h>

class xmlhandler : public dlobject {
   public:
		xmlhandler(void);
		~xmlhandler();

		untyped_interest::callresult accept_network_status_all(msg_network_status &);
		untyped_interest::callresult accept_network_status(msg_network_status &);
		untyped_interest::callresult accept_network_data(msg_network_data &);
		untyped_interest::callresult accept_stop(msg_stop &);
   private:
		thread_mutex mutex_; 

		ptr<interest<xmlhandler,msg_network_status> > interest_network_status_all_;
		ptr<interest<xmlhandler,msg_network_status> > interest_network_status_;
		ptr<interest<xmlhandler,msg_network_data> >   interest_network_data_;
		ptr<interest<xmlhandler,msg_stop> >           interest_stop_;

		void handle_datamess_(group_identifiers *gid, int length, const char *data,
									 msg_dom_mutation_event& );

		void add_parser_(group_identifiers *gid, ptr<DOM::Document> docptr);
		void detach_parser_(group_identifiers *gid);
		map<group_identifiers, xmlparser *> active_parsers_;
};

#endif
