// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Raw data to XML DOM-like tree converter.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <msg/dom/dom_event.hh>
#include "xmlhandler.hh"
#define DEBUG_CLASS "XMLHANDLER"
#include <mnemonic/debug.hh>

xmlhandler::xmlhandler(void) 
   {
	interest_network_status_all_=interest<xmlhandler, msg_network_status>::create(
		this, &xmlhandler::accept_network_status_all,untyped_interest::normal);
	interest_network_status_all_->acceptance=untyped_interest::ignore_ids;

	interest_network_status_=interest<xmlhandler, msg_network_status>::create(
		this, &xmlhandler::accept_network_status,untyped_interest::normal);
	interest_network_status_->acceptance=untyped_interest::iid_contains_mid;

	interest_network_data_=interest<xmlhandler, msg_network_data>::create(
		this, &xmlhandler::accept_network_data,untyped_interest::normal);
	interest_network_data_->acceptance=untyped_interest::iid_contains_mid;

	interest_stop_=interest<xmlhandler, msg_stop>::create(
		this, &xmlhandler::accept_stop,untyped_interest::normal);
	interest_stop_->acceptance=untyped_interest::iid_contains_mid;
   }

xmlhandler::~xmlhandler()
   {
	interest_network_status_all_->unregister();
	interest_network_status_->unregister();
	interest_network_data_->unregister();

	interest_network_status_all_.wait();
	interest_network_status_.wait();
	interest_network_data_.wait();
   }

untyped_interest::callresult xmlhandler::accept_network_status_all(msg_network_status& msg)
   {
	switch(msg.identifier) {
		case msg_network_status::content: {
			// FIXME: we compare the first 9 characters because there can be 
			// additional information in the content header. Eg.
			// text/html; charset=iso-8859-1. 
			// I guess we handle all of these additional things (well, we ignore them).
			// Is that correct?
			if(msg.text.find("text/html")!=msg.text.npos || 
				msg.text.find("text/plain")!=msg.text.npos ) {
				debug("received content announcement, creating new parser");
				mutex_.lock();
				ptr<DOM::Document> docptr=ptr<DOM::Document>(new DOM::Document(msg.url.get_url(),
																									msg.text,0));
				
				msg_dom_create_event dommsg;
				dommsg.new_dom_document=docptr;
				dommsg.set_group_ids(msg);
				try {
					dommsg.send();
					}
				catch(messagebase_error& ex) {
					debug(ex);
					mutex_.unlock();
					return untyped_interest::pass_on; // FIXME: maybe we need another return type,
					// something like `dead_end'.
					}
				debug("someone is listening to the DOM, so let's register for data on this midentifier");
				interest_stop_->add_group_ids(msg);
				interest_network_data_->add_group_ids(msg);
				interest_network_status_->add_group_ids(msg);
				add_parser_(&msg,docptr);
				debug("unlocking");
				mutex_.unlock();
				debug("returning to message loop");
				return untyped_interest::call_next;
				}
			else return untyped_interest::pass_on;
			}
		default:
			break;
		}
	return untyped_interest::pass_on;
	}

untyped_interest::callresult xmlhandler::accept_network_status(msg_network_status& msg)
   {
	switch(msg.identifier) {
		case msg_network_status::object_truncated:
		case msg_network_status::finished: {
			debug("network data flow ended, detaching parser");
			detach_parser_(&msg);
			msg_dom_parser_detached_event detmes;
			detmes.set_group_ids(msg);
			try {
				detmes.send();
				}
			catch(messagebase_error& ex) {
				debug("Hmm, nobody interested to know that we go away?");
				}
			return untyped_interest::call_next;
			}
		default:
			break;
		}
	
	return untyped_interest::pass_on;
   }

untyped_interest::callresult xmlhandler::accept_network_data(msg_network_data& msg)
   {
	debug("receiving data with id: " << msg);
	debug("while my interests are: " << *(interest_network_data_));
	// feed the lot through the parser
	msg_dom_mutation_event mutmes;
	mutmes.set_group_ids(msg);
	handle_datamess_(&msg,msg.length,msg.data,mutmes);

	try { 
		debug("sending dom mutation message");
		mutmes.send();
		}
	catch(messagebase_error& ex) {
		// FIXME: cancel current parser
		debug(ex);
		}
	
   return untyped_interest::call_next;
   }

untyped_interest::callresult xmlhandler::accept_stop(msg_stop &m) 
	{
	interest_network_data_->remove_group_ids(m);
	interest_network_status_->remove_group_ids(m);
	interest_stop_->remove_group_ids(m);
	detach_parser_(&m);
	return untyped_interest::call_next;
	}

extern "C" { void *create_xmlhandler(void**); }

void *create_xmlhandler(void** ) 
   {
   return new xmlhandler();
   }

template interest<xmlhandler, msg_network_status>;
template interest<xmlhandler, msg_network_data>;
template interest<xmlhandler, msg_stop>;
